@@ -1,5 +1,5 @@
-#include <sys/socket.h>
 #include <sys/types.h>
+#include <sys/socket.h>
 #include <stdio.h>
 #include <strings.h>
 #include <string.h>
@@ -30,32 +30,31 @@ gint send_browse_get(gpointer data, gint source,
 
 gint send_browse_files(gpointer data, gint source, 
 		      GdkInputCondition condition) {
-  char t2[2048];
-  char t[1024];
+  char* command;
   socket_t* socket = (socket_t*)data;
   file_t* file;
   GList* dlist = (GList*)(socket->data);
 
+  if (condition != GDK_INPUT_WRITE) {
+    socket_destroy(socket, NULL);
+    return 1;
+  }
+
   if (socket->data) {
     file = (file_t*)(dlist->data);
     if ((file->flags & FLAG_SHARED) ||
 	(file->flags & FLAG_TO_SHARE)) {
-      strcpy(t2, "\"");
-      strcat(t2, file->winname);
-      strcat(t2, "\" ");
-      strcat(t2, file->md5);
-      sprintf(t, " %ld %d %d %d\n",
-	      file->size, file->bitrate, 
-	      file->frequency, file->duration);
-      strcat(t2, t);
-      
-      send(source, t2, strlen(t2), 0);
+      command = g_strdup_printf("\"%s\" %s %lu %d %d %d\n",
+				file->winname, file->md5, file->size,
+				file->bitrate, file->frequency, file->duration);
+      send(source, command, strlen(command), 0);
+      g_free(command);
       socket->cnt = 0;
     }
     socket->data = (void*)(dlist->next);
   } else {
     send(source, "\n", 1, 0);
-    socket_destroy(socket, 1);
+    socket_destroy(socket, (void*)1);
   }
   
   return 1;
@@ -67,7 +66,7 @@ gint send_browse_nick(gpointer data, gint source,
   char* temp_str;
   
   if (condition != GDK_INPUT_WRITE) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
 
@@ -103,15 +102,15 @@ gint get_browse_files(gpointer data, gint source,
   char* pos2;
 
   if (condition != GDK_INPUT_READ) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   switch (res = recv(source, buf, 1024, MSG_PEEK)) {
   case -1:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   case 0:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   default:
     break;
@@ -142,12 +141,12 @@ gint get_browse_files(gpointer data, gint source,
     file->user = strdup((char*)(socket->data));
     file->winname = strdup(filename);
     file->md5 = strdup(md5);
-    sscanf(size, "%ld", &(file->size));
-    sscanf(bitrate, "%d", &(file->bitrate));
-    sscanf(frequency, "%d", &(file->frequency));
-    sscanf(duration, "%d", &(file->duration));
+    file->size = strtoul(size, NULL, 10);
+    file->bitrate = atoi(bitrate);
+    file->frequency = atoi(frequency);
+    file->duration = atoi(duration);
     file->longname = strdup(file->winname);
-    convert_to_unix(file->longname);
+    convert_to_unix(file->longname, 1);
     file->shortname = strdup(extract_short_name(file->longname));
     file->mime_type = get_mimetype(file->longname);
     file->flags = 0;
@@ -163,9 +162,9 @@ gint get_browse_files(gpointer data, gint source,
   return 1;
 
  close:
-  socket_destroy(socket, 1);
   strcpy(buf, (char*)socket->data);
   cmd_browse_end(buf);
+  socket_destroy(socket, (void*)1);
   return 1;
 }
 
@@ -179,7 +178,7 @@ gint get_browse_nick(gpointer data, gint source,
   socket_t* socket2;
 
   if (condition != GDK_INPUT_READ) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   gdk_input_remove(socket->input);
@@ -187,10 +186,10 @@ gint get_browse_nick(gpointer data, gint source,
 
   switch (res = recv(source, buffer, 1024, MSG_PEEK)) {
   case -1:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   case 0:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   default:
     break;
@@ -199,7 +198,7 @@ gint get_browse_nick(gpointer data, gint source,
   buffer[res] = 0;
   pos = strchr(buffer, '\n');
   if (!pos) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   cnt = (long)pos-(long)buffer;
@@ -208,10 +207,11 @@ gint get_browse_nick(gpointer data, gint source,
   
   socket2 = browse_search(buffer);
   if (!socket2) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   } else if (socket != socket2) {
-    socket_destroy(socket2, 1);
+    // socket is an incoming connection, mapping to found one
+    socket_destroy(socket2, (void*)1);
     socket->data = strdup(buffer);
     socket2 = socket;
   }
@@ -233,6 +233,7 @@ void browse_connect_and_start(socket_t* socket) {
     return;
   }
   
+  //  printf("connecting browse\n");
   socket->input = 
     gdk_input_add(socket->fd, GDK_INPUT_READ, 
 		  GTK_SIGNAL_FUNC(await_conn_ack2), socket);
@@ -245,22 +246,22 @@ gint await_conn_ack3(gpointer data, gint source,
   socket_t* socket = (socket_t*)data;
   
   if (condition != GDK_INPUT_READ) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   switch (res = recv(source, &c, 1, 0)) {
   case -1:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   case 0:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   default:
     break;
   }    
 
   if (c != '1') {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   
@@ -278,22 +279,22 @@ gint await_conn_ack2(gpointer data, gint source,
   socket_t* socket = (socket_t*)data;
   
   if (condition != GDK_INPUT_READ) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   switch (res = recv(source, &c, 1, 0)) {
   case -1:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   case 0:
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   default:
     break;
   }    
 
   if (c != '1') {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   
@@ -310,7 +311,7 @@ gint send_browse_get(gpointer data, gint source,
   socket_t* socket = (socket_t*)data;
 
   if (condition != GDK_INPUT_WRITE) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
 
@@ -329,7 +330,7 @@ gint send_browse_send(gpointer data, gint source,
   socket_t* socket = (socket_t*)data;
 
   if (condition != GDK_INPUT_WRITE) {
-    socket_destroy(socket, 0);
+    socket_destroy(socket, NULL);
     return 1;
   }
   send(source, "SENDLIST", 8, 0);
@@ -362,6 +363,9 @@ socket_t* browse_search(char* nick) {
 void browse_direct(char* nick) {
   socket_t* socket;
   
+  socket = browse_search(nick);
+  if (socket) return;   // already browsing
+
   socket = socket_new(S_BROWSE);
   socket->data = strdup(nick);
   send_command(CMD_CLIENT_BROWSE_DIRECT, nick);
@@ -373,7 +377,8 @@ GtkWidget* create_browse_popup() {
   GtkWidget *trennlinie20;
   GtkWidget *customize_list6;
   file_t* file;
-  GtkWidget *download2;
+  GtkWidget *item;
+  GtkWidget *user_popup;
 
   popup = gtk_menu_new ();
   gtk_object_set_data (GTK_OBJECT (popup), "popup", popup);
@@ -385,48 +390,63 @@ GtkWidget* create_browse_popup() {
   
   if (global.popup_row != -1) {
     if (!file) {
-      download2 = gtk_menu_item_new_with_label (_("Download Folder"));
-      gtk_widget_ref (download2);
-      gtk_object_set_data_full (GTK_OBJECT (popup), "download2", download2,
-				(GtkDestroyNotify) gtk_widget_unref);
-      gtk_widget_show (download2);
-      gtk_container_add (GTK_CONTAINER (popup), download2);
-      gtk_signal_connect (GTK_OBJECT (download2), "activate",
+      item = gtk_menu_item_new_with_label (_("Download Folder"));
+      gtk_widget_show (item);
+      gtk_container_add (GTK_CONTAINER (popup), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
 			  GTK_SIGNAL_FUNC (on_download_folder_activate),
 			  (void*)0);
     } else {
-      download2 = gtk_menu_item_new_with_label (_("Download Selected"));
-      gtk_widget_ref (download2);
-      gtk_object_set_data_full (GTK_OBJECT (popup), "download2", download2,
-				(GtkDestroyNotify) gtk_widget_unref);
-      gtk_widget_show (download2);
-      gtk_container_add (GTK_CONTAINER (popup), download2);
-      gtk_signal_connect (GTK_OBJECT (download2), "activate",
+      item = gtk_menu_item_new_with_label (_("Download Selected"));
+      gtk_widget_show (item);
+      gtk_container_add (GTK_CONTAINER (popup), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
 			  GTK_SIGNAL_FUNC (on_download2_activate),
 			  (void*)0);
-      download2 = gtk_menu_item_new_with_label (_("Download Selected with Foldername"));
-      gtk_widget_ref (download2);
-      gtk_object_set_data_full (GTK_OBJECT (popup), "download2", download2,
-				(GtkDestroyNotify) gtk_widget_unref);
-      gtk_widget_show (download2);
-      gtk_container_add (GTK_CONTAINER (popup), download2);
-      gtk_signal_connect (GTK_OBJECT (download2), "activate",
+      item = gtk_menu_item_new_with_label (_("Download Selected with Foldername"));
+      gtk_widget_show (item);
+      gtk_container_add (GTK_CONTAINER (popup), item);
+      gtk_signal_connect (GTK_OBJECT (item), "activate",
 			  GTK_SIGNAL_FUNC (on_download2_activate),
 			  (void*)1);
     }
 
     trennlinie20 = gtk_menu_item_new ();
-    gtk_widget_ref (trennlinie20);
-    gtk_object_set_data_full (GTK_OBJECT (popup), "trennlinie20", trennlinie20,
-			      (GtkDestroyNotify) gtk_widget_unref);
     gtk_widget_show (trennlinie20);
     gtk_container_add (GTK_CONTAINER (popup), trennlinie20);
     gtk_widget_set_sensitive (trennlinie20, FALSE);
   }
+  item = gtk_menu_item_new_with_label (_("Remove Browse"));
+  gtk_widget_show (item);
+  gtk_container_add (GTK_CONTAINER (popup), item);
+  gtk_signal_connect (GTK_OBJECT (item), "activate",
+		      GTK_SIGNAL_FUNC (on_remove_browse_activate),
+		      NULL);
+  item = gtk_menu_item_new_with_label (_("Refresh Files"));
+  gtk_widget_show (item);
+  gtk_container_add (GTK_CONTAINER (popup), item);
+  gtk_signal_connect (GTK_OBJECT (item), "activate",
+		      GTK_SIGNAL_FUNC (on_browse_files_activate),
+		      (gpointer)M_BROWSE);
+  trennlinie20 = gtk_menu_item_new ();
+  gtk_widget_show (trennlinie20);
+  gtk_container_add (GTK_CONTAINER (popup), trennlinie20);
+  gtk_widget_set_sensitive (trennlinie20, FALSE);
+
+
+  item = gtk_menu_item_new_with_label (_("User Menu"));
+  gtk_widget_show (item);
+  gtk_container_add (GTK_CONTAINER (popup), item);
+
+  user_popup = create_user_popup(M_BROWSE);
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), user_popup);
+
+  trennlinie20 = gtk_menu_item_new ();
+  gtk_widget_show (trennlinie20);
+  gtk_container_add (GTK_CONTAINER (popup), trennlinie20);
+  gtk_widget_set_sensitive (trennlinie20, FALSE);
+
   customize_list6 = gtk_menu_item_new_with_label (_("Customize List"));
-  gtk_widget_ref (customize_list6);
-  gtk_object_set_data_full (GTK_OBJECT (popup), "customize_list6", customize_list6,
-                            (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (customize_list6);
   gtk_container_add (GTK_CONTAINER (popup), customize_list6);
   gtk_signal_connect (GTK_OBJECT (customize_list6), "activate",
@@ -459,7 +479,7 @@ file_t* file_create_from_browse_response(char* data) {
   sscanf(pos1, "%d", &(file->duration));
   
   file->longname = strdup(file->winname);
-  convert_to_unix(file->longname);
+  convert_to_unix(file->longname, 1);
   pos1 = extract_short_name(file->longname);
   file->shortname = strdup(pos1);
   file->mime_type = get_mimetype(file->longname);
@@ -506,7 +526,7 @@ void file_create_from_browse_new(char* data) {
     file->mime_type = get_mimetype(file->winname);
 
     file->longname = strdup(file->winname);
-    convert_to_unix(file->longname);
+    convert_to_unix(file->longname, 1);
     pos1 = extract_short_name(file->longname);
     file->shortname = strdup(pos1);
     file->mime_type = get_mimetype(file->longname);
@@ -521,7 +541,7 @@ void file_create_from_browse_new(char* data) {
   }
 }
 
-void file_insert_browse_real(file_t *file) {
+int file_insert_browse_real(file_t *file, GtkCTree* ctree) {
   GtkWidget* temp;
   char* text;
   char* filename;
@@ -529,9 +549,8 @@ void file_insert_browse_real(file_t *file) {
   GtkCTreeNode *node;
   GtkCTreeNode *node2 = NULL;
   file_t* file2;
-  GtkCTree* ctree;
 
-  ctree = GTK_CTREE(lookup_widget(global.win, "ctree3"));
+  //  printf("inserting real\n");
   tstr[1][0] = tstr[2][0] = tstr[3][0] = tstr[4][0] = 0;
   node = NULL;
 
@@ -569,7 +588,7 @@ void file_insert_browse_real(file_t *file) {
     file2 = (file_t*)gtk_ctree_node_get_row_data(ctree, node);    
     if (!strcmp(file2->longname, file->longname)) {
       //      printf("do not insert [%s]\n", file->longname);
-      return;
+      return 0;
     }
   }
 
@@ -604,217 +623,407 @@ void file_insert_browse_real(file_t *file) {
   } while (pos);
   
   free(filename);
-  hotlist_update_user();
+  return 1;
 }
 
 void file_insert_browse(file_t *file) {
-  char* pos;
-  
-  GtkCList* clist;
-  int row;
+  GtkWidget* temp;
   user_t* userinfo;
   user_t* stats;
-  GtkWidget* temp;
 
-  temp = lookup_widget(global.win, "ctree3");
-  stats = (user_t*)gtk_object_get_data(GTK_OBJECT(temp), "userinfo");
+  userinfo = browse_search_nick(file->user);
+  if (!userinfo) {
+    destroy_file_row(file);
+    return;
+  }
 
-  if (stats && !strcmp(stats->username, file->user))
-    file_insert_browse_real(file);
-  
-  clist = GTK_CLIST(lookup_widget(global.win, "clist16"));
-
-  stats = (user_t*)gtk_object_get_data(GTK_OBJECT(clist), "userinfo");
-  stats->shared++;
-  stats->bytes += file->size;
-  hotlist_update_stats();
-
-  row = is_user_in_hotlist(file->user);
-  if (row >= 0) {
-    userinfo = gtk_clist_get_row_data(clist, row);
+  if (userinfo->link && file_insert_browse_real(file, GTK_CTREE(userinfo->link))) {
     userinfo->shared++;
     userinfo->bytes += file->size;
+    browse_update_user(userinfo);
     userinfo->files = g_list_append(userinfo->files, file);
-    pos = g_strdup_printf("%d", userinfo->shared);
-    gtk_clist_set_text(clist, row, 1, pos);
-    g_free(pos);
-  } else if ((userinfo = browse_list_search(file->user)) != NULL) {
-    userinfo->shared++;
-    userinfo->bytes += file->size;
-    userinfo->files = g_list_append(userinfo->files, file);
-    hotlist_update_stats();
+
+    temp = lookup_widget(global.win, "notebook6");
+    stats = (user_t*)gtk_object_get_data(GTK_OBJECT(temp), "browse_stats");
+    stats->shared++;
+    stats->bytes += file->size;
+    browse_update_stats(stats);
   }
 }
 
 void browse_user_files(char* nick, int max) {
-  GtkCTree* ctree;
-  GtkCList* clist;
   user_t* userinfo;
-  int row;
   char* command;
+  GtkWidget* temp;
 
   if (!nick || !(*nick)) return;
   
-  ctree = GTK_CTREE(lookup_widget(global.win, "ctree3"));
-  clist = GTK_CLIST(lookup_widget(global.win, "clist16"));
-  row = is_user_in_hotlist(nick);
-  if (row >= 0) {
-    userinfo = gtk_clist_get_row_data(clist, row);
-    hotlist_remove_user_files(userinfo);
-    browse_clear(NULL);
-    gtk_object_set_data(GTK_OBJECT(ctree), "userinfo", userinfo);
+  userinfo = browse_search_nick(nick);
+  if (userinfo) {
+    browse_clear_files(userinfo);
   } else {
-    userinfo = browse_list_search(nick);
-    if (userinfo) hotlist_remove_user_files(userinfo);
-    else {
-      userinfo = (user_t*)malloc(sizeof(user_t));
-      userinfo->files = NULL;
-      userinfo->timeout = 0;
-      userinfo->bytes = 0;
-      userinfo->shared = 0;
-      strcpy(userinfo->username, nick);
-      global.browses = g_list_append(global.browses, userinfo);
-    }
-    browse_clear(NULL);
-    gtk_object_set_data(GTK_OBJECT(ctree), "userinfo", userinfo);
+    userinfo = browse_create_new(nick);
   }
   
   if (max > 0) command = g_strdup_printf("%s %d", nick, max);
   else command = g_strdup_printf("%s", nick);
 
-  if (opennap_version(0, 39) || !opennap_version(0, 0)) {
+  gtk_clist_freeze(GTK_CLIST(userinfo->link));
+
+  temp = lookup_widget(global.win, "checkbutton44");
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp)) &&
+      (opennap_version(0, 39) || !opennap_version(0, 0))) {
     browse_direct(nick);
   } else {
     send_command(CMD_CLIENT_BROWSE, command);
   }
   g_free(command);
-  hotlist_update_user();
 }
 
-user_t* browse_list_search(char* nick) {
-  user_t* result;
+/////
+user_t* browse_search_nick(char* nick) {
   GList* dlist;
-  
+  user_t* user;
+
   for (dlist = global.browses; dlist; dlist = dlist->next) {
-    result = (user_t*)(dlist->data);
-    if (!strcasecmp(nick, result->username)) return result;
+    user = (user_t*)(dlist->data);
+    if (!strcasecmp(nick, user->username)) return user;
   }
   return NULL;
 }
 
-void browse_clear(user_t* userinfo) {
-  GtkCTree* ctree;
-  user_t* browse;
-
-  ctree = GTK_CTREE(lookup_widget(global.win, "ctree3"));
-  browse = (user_t*)gtk_object_get_data(GTK_OBJECT(ctree), "userinfo");
-
-  if (userinfo && (userinfo != browse)) return;
-  
-  gtk_object_set_data(GTK_OBJECT(ctree), "userinfo", NULL);
-  gtk_clist_clear(GTK_CLIST(ctree));
-  
-  hotlist_update_user();
-}
-
-void browse_show(user_t* userinfo) {
+void browse_clear_files(user_t* userinfo) {
+  GtkWidget* temp;
+  GtkCList* clist;
+  user_t* stats;
   GList* dlist;
   file_t* file;
-  GtkCList* clist;
-  user_t* browse;
-
-  hotlist_update_user();
-
-  clist = GTK_CLIST(lookup_widget(global.win, "ctree3"));
-  browse = (user_t*)gtk_object_get_data(GTK_OBJECT(clist), "userinfo");
-  if ((browse == userinfo) || (userinfo->files == NULL)) return;
-  browse_clear(NULL);
   
-  if (userinfo->files) {
-    userinfo->timeout = 0;
-    gtk_clist_freeze(clist);
-    for (dlist = userinfo->files; dlist; dlist = dlist->next) {
-      file = (file_t*)(dlist->data);
-      file_insert_browse_real(file);
-    }
-    gtk_clist_thaw(clist);
+  if (!userinfo) return;
+
+  // first updating browse stats
+  temp = lookup_widget(global.win, "notebook6");
+  stats = (user_t*)gtk_object_get_data(GTK_OBJECT(temp), "browse_stats");
+  stats->shared -= userinfo->shared;
+  stats->bytes -= userinfo->bytes;
+  browse_update_stats(stats);
+
+  // updating user stats
+  userinfo->shared = 0;
+  userinfo->bytes = 0;
+  browse_update_user(userinfo);
+
+  // releasing file list
+  for (dlist = (GList*)(userinfo->files); dlist; dlist = dlist->next) {
+    file = (file_t*)(dlist->data);
+    destroy_file_row(file);
   }
-  gtk_object_set_data(GTK_OBJECT(clist), "userinfo", userinfo);
+  g_list_free(userinfo->files);
+  userinfo->files = NULL;
 
-  gtk_ctree_sort_recursive(GTK_CTREE(clist), NULL);
-  
-  hotlist_update_user();
+  // clearing the ctree
+  if (userinfo->link) {
+    clist = GTK_CLIST(userinfo->link);
+    gtk_clist_clear(clist);
+  }
 }
 
-void on_browse_user(GtkMenuItem     *menuitem,
-		    gpointer         user_data) {
-  browse_show(user_data);
-}
-
-GtkWidget* create_browse_user_popup (void) {
-  GtkWidget *popup;
-  GtkAccelGroup *popup_accels;
-  GtkWidget *mode;
-  GtkCList* clist;
-  int row;
+user_t* browse_create_new(char* nick) {
   user_t* userinfo;
-  char* temp_str;
-  int cnt = 0;
+
+  userinfo = (user_t*)malloc(sizeof(user_t));
+  userinfo->files = NULL;
+  userinfo->timeout = 0;
+  userinfo->bytes = 0;
+  userinfo->shared = 0;
+  strcpy(userinfo->username, nick);
+  global.browses = g_list_append(global.browses, userinfo);
+
+  userinfo->link = browse_create_page(nick);
+  gtk_object_set_data(GTK_OBJECT(userinfo->link), "userinfo", userinfo);
+  browse_update_user(userinfo);
+
+  return userinfo;
+}
+
+void on_expand_clicked                   (GtkButton       *button,
+					  gpointer         user_data)
+{
+  GtkCTree* ctree;
+
+  ctree = GTK_CTREE(user_data);
+  gtk_ctree_expand_recursive(ctree, NULL);
+}
+
+void on_collapse_clicked                   (GtkButton       *button,
+					    gpointer         user_data)
+{
+  GtkCTree* ctree;
+
+  ctree = GTK_CTREE(user_data);
+  gtk_ctree_collapse_recursive(ctree, NULL);
+}
+
+GtkWidget* browse_create_page(char* nick) {
+  GtkWidget *vbox;
+  GtkWidget *scrolledwindow;
+  GtkWidget *ctree;
+  GtkWidget *label;
+  GtkWidget *frame;
+  GtkWidget *hbox;
+  GtkWidget *hbox2;
+  GtkWidget *button;
+  GtkNotebook* notebook;
+  GtkTooltips* tips;
+
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_widget_show (vbox);
+
+  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
+  gtk_widget_show (scrolledwindow);
+  gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+
+  ctree = gtk_ctree_new (5, 0);
+  gtk_widget_show (ctree);
+  gtk_widget_set_events (ctree, GDK_POINTER_MOTION_MASK | GDK_LEAVE_NOTIFY_MASK);
+  gtk_container_add (GTK_CONTAINER (scrolledwindow), ctree);
+  gtk_clist_set_column_width (GTK_CLIST (ctree), 0, 
+			      global.browse_width[0]);
+  gtk_clist_set_column_visibility(GTK_CLIST (ctree), 0,
+				  global.browse_show[0]);
+  gtk_clist_set_column_width (GTK_CLIST (ctree), 1,
+			      global.browse_width[1]);
+  gtk_clist_set_column_visibility(GTK_CLIST (ctree), 1,
+				  global.browse_show[1]);
+  gtk_clist_set_column_width (GTK_CLIST (ctree), 2,
+			      global.browse_width[2]);
+  gtk_clist_set_column_visibility(GTK_CLIST (ctree), 2,
+				  global.browse_show[2]);
+  gtk_clist_set_column_width (GTK_CLIST (ctree), 3,
+			      global.browse_width[3]);
+  gtk_clist_set_column_visibility(GTK_CLIST (ctree), 3,
+				  global.browse_show[3]);
+  gtk_clist_set_column_width (GTK_CLIST (ctree), 4,
+			      global.browse_width[4]);
+  gtk_clist_set_column_visibility(GTK_CLIST (ctree), 4,
+				  global.browse_show[4]);
+
+
+  gtk_clist_set_selection_mode (GTK_CLIST (ctree), GTK_SELECTION_EXTENDED);
+  gtk_clist_column_titles_show (GTK_CLIST (ctree));
+  gtk_clist_set_compare_func(GTK_CLIST(ctree), browse_compare);
+
+  label = gtk_label_new (_("Filename"));
+  gtk_widget_show (label);
+  gtk_clist_set_column_widget (GTK_CLIST (ctree), 0, label);
+
+  label = gtk_label_new (_("Size"));
+  gtk_widget_show (label);
+  gtk_clist_set_column_widget (GTK_CLIST (ctree), 1, label);
+
+  label = gtk_label_new (_("Bitrate"));
+  gtk_widget_show (label);
+  gtk_clist_set_column_widget (GTK_CLIST (ctree), 2, label);
+
+  label = gtk_label_new (_("Frequency"));
+  gtk_widget_show (label);
+  gtk_clist_set_column_widget (GTK_CLIST (ctree), 3, label);
+
+  label = gtk_label_new (_("Length"));
+  gtk_widget_show (label);
+  gtk_clist_set_column_widget (GTK_CLIST (ctree), 4, label);
+
+  hbox2 = gtk_hbox_new (FALSE, 0);
+  gtk_widget_show (hbox2);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
+
+  hbox = gtk_hbox_new (FALSE, 3);
+  gtk_widget_show (hbox);
+  gtk_box_pack_end (GTK_BOX (hbox2), hbox, FALSE, FALSE, 0);
+
+  ///////
+
+  tips = (GtkTooltips*)gtk_object_get_data (GTK_OBJECT (global.win), "tooltips");
+  button = gtk_button_new_with_label (_("Remove Browse"));
+  gtk_widget_show (button);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_widget_set_usize (button, -2, 15);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+                      GTK_SIGNAL_FUNC (on_remove_browse_clicked),
+                      (gpointer)ctree);
+
+  button = gtk_button_new_with_label (_("  X  "));
+  gtk_widget_show (button);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_widget_set_usize (button, -2, 15);
+  gtk_tooltips_set_tip (tips, button, _("Expand whole tree"), NULL);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+                      GTK_SIGNAL_FUNC (on_expand_clicked),
+                      (gpointer)ctree);
+
+  button = gtk_button_new_with_label (_("  C  "));
+  gtk_widget_show (button);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_widget_set_usize (button, -2, 15);
+  gtk_tooltips_set_tip (tips, button, _("Collapse whole tree"), NULL);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+                      GTK_SIGNAL_FUNC (on_collapse_clicked),
+                      (gpointer)ctree);
+
+  //////
+
+  label = gtk_label_new (_("Files"));
+  gtk_widget_show (label);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+  frame = gtk_frame_new (NULL);
+  gtk_widget_show (frame);
+  gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
+  gtk_widget_set_usize (frame, 50, -2);
+  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
+
+  label = gtk_label_new ("");
+  gtk_widget_show (label);
+  gtk_container_add (GTK_CONTAINER (frame), label);
+  gtk_object_set_data(GTK_OBJECT(ctree), "label2", label);
+
+  label = gtk_label_new (_("MB"));
+  gtk_widget_show (label);
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_misc_set_padding (GTK_MISC (label), 3, 0);
+
+  frame = gtk_frame_new (NULL);
+  gtk_widget_show (frame);
+  gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
+  gtk_widget_set_usize (frame, 55, -2);
+  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
+
+  label = gtk_label_new ("");
+  gtk_widget_show (label);
+  gtk_container_add (GTK_CONTAINER (frame), label);
+  gtk_object_set_data(GTK_OBJECT(ctree), "label3", label);
+  gtk_object_set_data(GTK_OBJECT(ctree), "child", vbox);
+  gtk_object_set_data(GTK_OBJECT(vbox), "clist", ctree);
+
+  gtk_signal_connect (GTK_OBJECT (ctree), "click_column",
+                      GTK_SIGNAL_FUNC (on_list_click_column),
+                      NULL);
+  gtk_signal_connect (GTK_OBJECT (ctree), "button_press_event",
+                      GTK_SIGNAL_FUNC (on_browse_button_press_event),
+                      NULL);
+  gtk_signal_connect (GTK_OBJECT (ctree), "motion_notify_event",
+                      GTK_SIGNAL_FUNC (on_browse_motion_notify_event),
+                      NULL);
+  gtk_signal_connect (GTK_OBJECT (ctree), "leave_notify_event",
+                      GTK_SIGNAL_FUNC (on_list_leave_notify_event),
+                      NULL);
+
+  label = gtk_label_new (nick);
+  
+  notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook6"));
+  gtk_notebook_append_page(notebook, vbox, label);
+  gtk_notebook_set_page(notebook, gtk_notebook_page_num(GTK_NOTEBOOK(notebook), 
+							scrolledwindow));
+
+  return ctree;
+}
+
+void browse_update_user(user_t* userinfo) {
+  GtkObject* ctree;
+  GtkWidget* temp;
+  char str[1024];
+
+  if (!userinfo) return;
+  if (userinfo->link) {
+    ctree = GTK_OBJECT(userinfo->link);
+    if (ctree) {
+      temp = gtk_object_get_data(ctree, "label2");
+      sprintf(str, "%d", userinfo->shared);
+      gtk_label_set_text(GTK_LABEL(temp), str);
+      temp = gtk_object_get_data(ctree, "label3");
+      sprintf(str, "%.0f", userinfo->bytes/1048576.);
+      gtk_label_set_text(GTK_LABEL(temp), str);
+    }
+  }
+}
+
+void browse_update_stats(user_t* stats) {
+  GtkWidget* temp;
+  char str[1024];
+
+  temp = lookup_widget(global.win, "label229");
+  sprintf(str, "%d", stats->shared);
+  gtk_label_set_text(GTK_LABEL(temp), str);
+  temp = lookup_widget(global.win, "label231");
+  sprintf(str, "%.0f", stats->bytes/1048576.);
+  gtk_label_set_text(GTK_LABEL(temp), str);
+}
+
+void browse_remove(user_t* userinfo) {
+  GtkObject* ctree;
+  GtkWidget* child;
+  GtkNotebook* notebook;
+  int i1;
+
+  if (!userinfo) return;
+
+  // clearing file list
+  browse_clear_files(userinfo);
+
+  // removing browse page
+  ctree = GTK_OBJECT(userinfo->link);
+  if (ctree) {
+    notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook6"));
+    child = GTK_WIDGET(gtk_object_get_data(ctree, "child"));
+    i1 = gtk_notebook_page_num(notebook, child);
+    gtk_notebook_remove_page(notebook, i1);
+  }
+  free(userinfo);
+
+  global.browses = g_list_remove(global.browses, userinfo);
+}
+
+file_t* browse_search_file(user_t* info, char* winname) {
+  GList* dlist;
+  file_t* file;
+  
+  for (dlist = info->files; dlist; dlist = dlist->next) {
+    file = (file_t*)(dlist->data);
+    if (!strcmp(winname, file->winname)) return file;
+  }
+  return NULL;
+}
+
+///////////
+void search_browse(user_t* userinfo, search_pattern_t* search, int *cnt) {
+  file_t* file;
   GList* dlist;
 
-  popup = gtk_menu_new ();
-  gtk_object_set_data (GTK_OBJECT (popup), "popup", popup);
-  popup_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (popup));
+  for (dlist = userinfo->files; dlist; dlist = dlist->next) {
+    file = (file_t*)(dlist->data);
+    if (search_pattern_fits_file(search, file, 1)) {
+      file = file_dup(file);
+      file_insert_search(file, DEST_HOTLIST);
+      (*cnt)++;
+    }
+    if (*cnt >= search->max_results) return;
+  }
+}
+
+void browse_do_search(search_pattern_t* search) {
+  GtkCList* clist;
+  user_t* userinfo;
+  GList* dlist;
+  int cnt = 0;
+
+  if (search->max_results == 0) return;
 
   clist = GTK_CLIST(lookup_widget(global.win, "clist16"));
-  for (row = 0; row < clist->rows; row++) {
-    userinfo = gtk_clist_get_row_data(clist, row);
-    if (!userinfo || (!userinfo->files)) continue;
-    temp_str = g_strdup_printf("%s (%d)", userinfo->username,
-			       userinfo->shared);
-    mode = gtk_menu_item_new_with_label (temp_str);
-    g_free(temp_str);
-    gtk_widget_ref (mode);
-    gtk_widget_show (mode);
-    gtk_container_add (GTK_CONTAINER (popup), mode);
-    gtk_signal_connect (GTK_OBJECT (mode), "activate",
-			GTK_SIGNAL_FUNC (on_browse_user),
-			userinfo);
-    cnt++;
-  }
-
-  if ((cnt > 0) && global.browses) {
-    mode = gtk_menu_item_new ();
-    gtk_widget_set_sensitive (mode, FALSE);
-    gtk_widget_ref (mode);
-    gtk_widget_show (mode);
-    gtk_container_add (GTK_CONTAINER (popup), mode);
-  }
-
   for (dlist = global.browses; dlist; dlist = dlist->next) {
     userinfo = (user_t*)(dlist->data);
-    if (!userinfo) continue;
-    temp_str = g_strdup_printf("%s (%d)", userinfo->username,
-			       userinfo->shared);
-    mode = gtk_menu_item_new_with_label (temp_str);
-    g_free(temp_str);
-    gtk_widget_ref (mode);
-    gtk_widget_show (mode);
-    gtk_container_add (GTK_CONTAINER (popup), mode);
-    gtk_signal_connect (GTK_OBJECT (mode), "activate",
-			GTK_SIGNAL_FUNC (on_browse_user),
-			userinfo);
-    cnt++;
+    search_browse(userinfo, search, &cnt);
+    if (cnt >= search->max_results) return;
   }
-
-  if (cnt == 0) {
-    mode = gtk_menu_item_new_with_label ("empty");
-    gtk_widget_set_sensitive(mode, FALSE);
-    gtk_widget_ref (mode);
-    gtk_widget_show (mode);
-    gtk_container_add (GTK_CONTAINER (popup), mode);
-    gtk_signal_connect (GTK_OBJECT (mode), "activate",
-			NULL, NULL);
-  }
-  return popup;
 }
+
