@@ -233,6 +233,9 @@ on_delete_transfer_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
+on_allow_upload_activate             (GtkMenuItem     *menuitem,
+				      gpointer         user_data);
+void
 on_retry_download_activate             (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 void
@@ -350,6 +353,9 @@ on_refresh_users_files_activate        (GtkMenuItem     *menuitem,
 void
 on_remove_user_from_hotlist_activate   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+void
+on_remove_from_hotlist_activate   (GtkMenuItem     *menuitem,
+				   gpointer         user_data);
 
 void
 on_download_folder_activate         (GtkMenuItem     *menuitem,
@@ -358,6 +364,18 @@ void
 on_download2_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
+void
+on_download3_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_remove_browse_activate                  (GtkMenuItem     *menuitem,
+					    gpointer         user_data);
+
+void
+on_remove_browse_clicked                   (GtkButton       *button,
+					    gpointer         user_data);
+
 void
 on_download_selected1_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -506,14 +524,6 @@ void
 on_button_users_clicked                 (GtkButton       *button,
                                         gpointer         user_data);
 void
-on_entry45_activate                    (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_entry46_activate                    (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
 on_checkbutton7_toggled                (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
@@ -562,14 +572,6 @@ void
 on_button99_clicked                    (GtkButton       *button,
                                         gpointer         user_data);
 
-void
-on_button123_clicked                   (GtkButton       *button,
-                                        gpointer         user_data);
-
-void
-on_button122_clicked                   (GtkButton       *button,
-                                        gpointer         user_data);
-
 void
 on_checkbutton9_toggled                (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
@@ -609,10 +611,6 @@ void
 on_NULL_button1_clicked                (GtkButton       *button,
                                         gpointer         user_data);
 
-void
-on_button124_clicked                   (GtkButton       *button,
-                                        gpointer         user_data);
-
 void
 on_button125_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
@@ -621,10 +619,6 @@ void
 on_checkbutton11_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
-void
-on_entry51_activate                    (GtkEditable     *editable,
-                                        gpointer         user_data);
-
 void
 on_entry50_activate                    (GtkEditable     *editable,
                                         gpointer         user_data);
@@ -638,28 +632,11 @@ void
 on_entry50_changed                     (GtkEditable     *editable,
                                         gpointer         user_data);
 
-void
-on_entry45_changed                     (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_entry46_changed                     (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_entry51_changed                     (GtkEditable     *editable,
-                                        gpointer         user_data);
-
 gboolean
 on_entry46_key_press_event             (GtkWidget       *widget,
                                         GdkEventKey     *event,
                                         gpointer         user_data);
 
-gboolean
-on_entry51_key_press_event             (GtkWidget       *widget,
-                                        GdkEventKey     *event,
-                                        gpointer         user_data);
-
 void
 on_entry17_activate                    (GtkEditable     *editable,
                                         gpointer         user_data);
@@ -677,9 +654,8 @@ on_private_mode_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_checkbutton14_toggled               (GtkToggleButton *togglebutton,
-                                        gpointer         user_data);
-
+on_whisper_activate               (GtkMenuItem     *menuitem,
+				   gpointer         user_data);
 void
 on_button127_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
@@ -754,11 +730,6 @@ void
 on_play_file2_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_list_click_column                   (GtkCList        *clist,
-                                        gint             column,
-                                        gpointer         user_data);
-
 gboolean
 on_search_list_button_press_event      (GtkWidget       *widget,
                                         GdkEventButton  *event,
@@ -768,10 +739,6 @@ void
 on_search_artist_activate              (GtkEditable     *editable,
                                         gpointer         user_data);
 
-void
-on_combo_entry18_changed               (GtkEditable     *editable,
-                                        gpointer         user_data);
-
 void
 on_join_channel1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -957,7 +924,12 @@ on_resume_tree_button_press_event      (GtkWidget       *widget,
 void
 on_resume_search_activate               (GtkMenuItem     *menuitem,
 					 gpointer         user_data);
-
+void
+on_resume_freeze_activate(GtkMenuItem     *menuitem,
+			  gpointer         user_data);
+void
+on_resume_thaw_activate(GtkMenuItem     *menuitem,
+			gpointer         user_data);
 void
 on_resume_delete_activate(GtkMenuItem     *menuitem,
 			  gpointer         user_data);
@@ -1136,17 +1108,10 @@ on_checkbutton33_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
 gboolean
-on_ctree3_button_press_event           (GtkWidget       *widget,
+on_browse_button_press_event           (GtkWidget       *widget,
                                         GdkEventButton  *event,
                                         gpointer         user_data);
 
-void
-on_clist16_select_row                  (GtkCList        *clist,
-                                        gint             row,
-                                        gint             column,
-                                        GdkEvent        *event,
-                                        gpointer         user_data);
-
 gboolean
 on_clist16_button_press_event          (GtkWidget       *widget,
                                         GdkEventButton  *event,
@@ -1164,15 +1129,6 @@ void
 on_entry79_activate                    (GtkEditable     *editable,
                                         gpointer         user_data);
 
-void
-on_entry79_changed                     (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-gboolean
-on_entry79_focus_in_event              (GtkWidget       *widget,
-                                        GdkEventFocus   *event,
-                                        gpointer         user_data);
-
 void
 on_button177_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
@@ -1197,13 +1153,6 @@ on_clist17_select_row                  (GtkCList        *clist,
                                         GdkEvent        *event,
                                         gpointer         user_data);
 
-void
-on_clist17_select_row                  (GtkCList        *clist,
-                                        gint             row,
-                                        gint             column,
-                                        GdkEvent        *event,
-                                        gpointer         user_data);
-
 void
 on_button183_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
@@ -1265,10 +1214,6 @@ void
 on_button187_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
 
-void
-on_button188_clicked                   (GtkButton       *button,
-                                        gpointer         user_data);
-
 void
 on_checkbutton39_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
@@ -1295,6 +1240,9 @@ void
 on_locate_activate               (GtkMenuItem     *menuitem,
 				  gpointer         user_data);
 void
+on_send_file_activate               (GtkMenuItem     *menuitem,
+				     gpointer         user_data);
+void
 on_nuke_activate               (GtkMenuItem     *menuitem,
 				gpointer         user_data);
 void
@@ -1362,10 +1310,6 @@ void
 on_spinbutton35_changed                (GtkEditable     *editable,
                                         gpointer         user_data);
 
-void
-on_combo_entry22_changed               (GtkEditable     *editable,
-                                        gpointer         user_data);
-
 void
 on_checkbutton42_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
@@ -1378,20 +1322,6 @@ void
 on_spinbutton36_changed                (GtkEditable     *editable,
                                         gpointer         user_data);
 
-void
-on_combo_entry23_changed               (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-gboolean
-on_drawingarea1_expose_event           (GtkWidget       *widget,
-                                        GdkEventExpose  *event,
-                                        gpointer         user_data);
-
-gboolean
-on_drawingarea2_expose_event           (GtkWidget       *widget,
-                                        GdkEventExpose  *event,
-                                        gpointer         user_data);
-
 gboolean
 on_drawingarea1_expose_event           (GtkWidget       *widget,
                                         GdkEventExpose  *event,
@@ -1578,3 +1508,355 @@ on_unshare_subtree_activate            (GtkMenuItem     *menuitem,
 void
 on_share_subtree_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_button236_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_spinbutton42_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+void
+on_search_select_row            (GtkCList        *clist,
+				 gint             row,
+				 gint             column,
+				 GdkEvent        *event,
+				 gpointer         user_data);
+
+void
+on_button237_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button238_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button243_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button244_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button241_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button242_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button245_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_list_click_column                   (GtkCList        *clist,
+                                        gint             column,
+                                        gpointer         user_data);
+
+void
+on_checkbutton46_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_spinbutton48_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_spinbutton49_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_transfer_up_select_row              (GtkCList        *clist,
+                                        gint             row,
+                                        gint             column,
+                                        GdkEvent        *event,
+                                        gpointer         user_data);
+
+void
+on_checkbutton47_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton48_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton49_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton45_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_spinbutton53_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_spinbutton54_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_combo_entry22_changed               (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_combo_entry23_changed               (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_spinbutton55_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_combo_entry25_changed               (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_wizard_destroy                      (GtkObject       *object,
+                                        gpointer         user_data);
+
+void
+on_button_browse_clicked               (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button262_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button263_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button247_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button246_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button264_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button272_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button273_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+void
+on_combo_entry26_changed               (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_detach_statistic_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_button274_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea5_motion_notify_event    (GtkWidget       *widget,
+                                        GdkEventMotion  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea6_motion_notify_event    (GtkWidget       *widget,
+                                        GdkEventMotion  *event,
+                                        gpointer         user_data);
+
+void
+on_clist16_select_row                  (GtkCList        *clist,
+                                        gint             row,
+                                        gint             column,
+                                        GdkEvent        *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea3_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea4_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+void
+on_button275_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button276_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_resume_configure(GtkMenuItem     *menuitem,
+		    gpointer         user_data);
+
+void
+on_checkbutton51_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton52_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton53_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+
+void
+on_combo_entry13_changed               (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_entry80_changed                     (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_combo_entry20_changed               (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea10_expose_event          (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_lib_tree_motion_notify_event        (GtkWidget       *widget,
+                                        GdkEventMotion  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_search_motion_notify_event        (GtkWidget       *widget,
+				      GdkEventMotion  *event,
+				      gpointer         user_data);
+gboolean
+on_browse_motion_notify_event        (GtkWidget       *widget,
+				      GdkEventMotion  *event,
+				      gpointer         user_data);
+
+gboolean
+on_list_leave_notify_event             (GtkWidget       *widget,
+                                        GdkEventCrossing *event,
+                                        gpointer         user_data);
+
+void
+on_remove_all_activate                 (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_unshare_all_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_share_all_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_checkbutton59_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_spinbutton61_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_checkbutton63_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_checkbutton64_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_spinbutton62_changed                (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+gboolean
+on_ctree3_button_press_event           (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+gboolean
+on_drawingarea7_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea8_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea9_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea5_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea6_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea5_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_drawingarea6_expose_event           (GtkWidget       *widget,
+                                        GdkEventExpose  *event,
+                                        gpointer         user_data);
+
+void
+on_latest_version_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_button308_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_checkbutton66_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_button309_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button310_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button311_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button312_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button313_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button314_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_checkbutton67_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
