@@ -4,6 +4,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/time.h>
+#include <time.h>
 
 #include <gtk/gtk.h>
 
@@ -20,6 +21,7 @@
 #include "commands.h"
 #include "log.h"
 #include "handler.h"
+#include "hotlist.h"
 
 #define MAX_LINE_LENGTH  150
 
@@ -29,17 +31,20 @@ char* get_word(GtkText *text, int deselect) {
   GtkEditable *editable;
   char* str;
   static char result[1024];
-  int cnt = 0;
   int cursor;
+  char left, right;
 
   editable = GTK_EDITABLE (text);
   
+  left = right = 0;
   cursor = global.popup_row;
   while (1) {
     if (cursor < 1) break;
     if (GTK_TEXT_INDEX(text, cursor-1) == '<') {
-      cnt++;
-      break;
+      left = 2;
+    }
+    if (GTK_TEXT_INDEX(text, cursor-1) == '[') {
+      left = 1;
     }
     if (GTK_TEXT_INDEX(text, cursor-1) == '\n')
       break;
@@ -51,8 +56,10 @@ char* get_word(GtkText *text, int deselect) {
   while (1) {
     if (cursor >= gtk_text_get_length(text)) break;
     if (GTK_TEXT_INDEX(text, cursor) == '>') {
-      cnt++;
-      break;
+      right = 2;
+    }
+    if (GTK_TEXT_INDEX(text, cursor) == ']') {
+      right = 1;
     }
     if (GTK_TEXT_INDEX(text, cursor) == '\n')
       break;
@@ -62,7 +69,18 @@ char* get_word(GtkText *text, int deselect) {
   }
   end_pos = cursor;
 
-  if (!cnt) return NULL;
+  if (!left && !right) return NULL;
+  if ((left == 1) && (right == 1)) return NULL;
+  if (left == right) {
+    start_pos++;
+    end_pos--;
+  } else {
+    if (left > right) {
+      start_pos++;
+    } else {
+      end_pos--;
+    }
+  }
 
   str = gtk_editable_get_chars(GTK_EDITABLE(text),
 			       start_pos, end_pos);
@@ -87,6 +105,9 @@ char* get_popup_user(int mode) {
   guint8 space;
   static char result[1024];
   socket_t* socket;
+  user_t* userinfo;
+  resume_t* resume;
+  access_t* access;
 
   if ((mode == M_ONLINE) || (mode == M_HOTLIST)) {
     gtk_clist_get_pixtext (GTK_CLIST(global.popup_list), 
@@ -114,6 +135,17 @@ char* get_popup_user(int mode) {
   } else if (mode == M_IGNORE) {
     gtk_clist_get_text(GTK_CLIST(global.popup_list),
 		       global.popup_row, 0, &user);
+  } else if (mode == M_BROWSE) {
+    userinfo = gtk_object_get_data(GTK_OBJECT(global.popup_list), "userinfo");
+    user = userinfo->username;
+  } else if (mode == M_RESUME) {
+    resume = (resume_t*)gtk_clist_get_row_data(GTK_CLIST(global.popup_list),
+					       global.popup_row);
+    user = resume->user;
+  } else if (mode == M_ACCESS) {
+    access = (access_t*)gtk_clist_get_row_data(GTK_CLIST(global.popup_list),
+					       global.popup_row);
+    user = access->name;
   }
   
   strcpy(result, user);
@@ -242,33 +274,79 @@ void snap_to_bottom(GtkWidget *text) {
 			   - vertical_adjustment->page_size);
 }
 
+int chat_page_thaw(gpointer data) {
+  chat_page_t* page = (chat_page_t*)data;
+
+  if (page->header && (page->type == P_PUBLIC)) 
+    gtk_text_thaw(GTK_TEXT(page->header));
+  if (page->wallop) 
+    gtk_text_thaw(GTK_TEXT(page->wallop));
+  if (page->main) 
+    gtk_text_thaw(GTK_TEXT(page->main));
+
+  if (page->timeout >= 0) {
+    gtk_timeout_remove(page->timeout);
+    page->timeout = -1;
+  }
+
+  if (page->snap & 1) snap_to_bottom(page->header);
+  if (page->snap & 2) snap_to_bottom(page->wallop);
+  if (page->snap & 4) snap_to_bottom(page->main);
+
+  page->snap = 0;
+  page->modified = 0;
+  
+
+  return 1;
+}
+
+void chat_page_freeze(chat_page_t* page, int message_type) {
+
+  if ((message_type == M_TOPIC) && ((page->modified & 1) == 0) &&
+      scroll_mode(page->header))
+    page->snap |= 1;
+  if ((message_type == M_OP) && ((page->modified & 2) ==0) &&
+      scroll_mode(page->wallop))
+    page->snap |= 2;
+  if ((message_type == M_PUBLIC) && ((page->modified & 4) == 0) &&
+      scroll_mode(page->main))
+    page->snap |= 4;
+
+  if (page->timeout >= 0) {  // update freeze
+    gtk_timeout_remove(page->timeout);
+  } else {                   // first freeze
+    if (page->header && (page->type == P_PUBLIC)) 
+      gtk_text_freeze(GTK_TEXT(page->header));
+    if (page->wallop) 
+      gtk_text_freeze(GTK_TEXT(page->wallop));
+    if (page->main) 
+      gtk_text_freeze(GTK_TEXT(page->main));
+  }
+
+  page->timeout = gtk_timeout_add(100, chat_page_thaw, page);
+}
+
 void chat_print_channel(chat_page_t* page, int message_type,
 			char* color, char* text) {
-  char t[1024];
   style_t* style;
   GtkWidget* temp;
-  int snap;
 
-  if (!strcmp(page->name, "Channels"))
-    page = chat_page_search("Console", P_FIX);
+  if (page->type == P_FIX) {
+    page = chat_page_search("MOTD", P_OTHER);
+    if (!page) page = create_other_page("MOTD", "MOTD");
+  }
   if (!page) return;
 
-  if (message_type == M_TOPIC)
-    sprintf(t, "%s_topic", page->name);
-  else if (message_type == M_OP)
-    sprintf(t, "%s_text2", page->name);
-  else sprintf(t, "%s_text", page->name);
+  if (message_type == M_TOPIC) temp = page->header;
+  else if (message_type == M_OP) temp = page->wallop;
+  else temp = page->main;
 
-  temp = lookup_widget(global.win, t);
   style = style_get(global.scheme, color);
 
-  gtk_text_freeze(GTK_TEXT(temp));
-  snap = scroll_mode(temp);
+  chat_page_freeze(page, message_type);
   gtk_text_insert(GTK_TEXT(temp), style->font, 
 		  style->fore, style->back, 
 		  text, strlen(text));
-  gtk_text_thaw(GTK_TEXT(temp));
-  if (snap) snap_to_bottom(temp);
   
   highlight(page, 1);
   
@@ -278,24 +356,21 @@ void chat_print_channel(chat_page_t* page, int message_type,
 void chat_print_channel_color(chat_page_t* page, int message_type,
 			      char* color, GdkColor* fore, 
 			      GdkColor* back, char *text) {
-  char t[1024];
   style_t* style;
   GtkWidget* temp;
   GdkColor* f;
   GdkColor* b;
-  int snap;
 
-  if (!strcmp(page->name, "Channels"))
-    page = chat_page_search("Console", P_FIX);
+  if (page->type == P_FIX) {
+    page = chat_page_search("MOTD", P_OTHER);
+    if (!page) page = create_other_page("MOTD", "MOTD");
+  }
   if (!page) return;
   
-  if (message_type == M_TOPIC)
-    sprintf(t, "%s_topic", page->name);
-  else if (message_type == M_OP)
-    sprintf(t, "%s_text2", page->name);
-  else sprintf(t, "%s_text", page->name);
+  if (message_type == M_TOPIC) temp = page->header;
+  else if (message_type == M_OP) temp = page->wallop;
+  else temp = page->main;
 
-  temp = lookup_widget(global.win, t);
   style = style_get(global.scheme, color);
 
   if (fore) f = fore;
@@ -303,12 +378,9 @@ void chat_print_channel_color(chat_page_t* page, int message_type,
   if (back) b = back;
   else b = style->back;
   
-  gtk_text_freeze(GTK_TEXT(temp));
-  snap = scroll_mode(temp);
+  chat_page_freeze(page, message_type);
   gtk_text_insert(GTK_TEXT(temp), style->font, 
 		  f, b, text, strlen(text));
-  gtk_text_thaw(GTK_TEXT(temp));
-  if (snap) snap_to_bottom(temp);
 
   highlight(page, 1);
   
@@ -369,14 +441,12 @@ void join_channel(char* name) {
 
 void on_hide_show_clicked(GtkButton *button, gpointer user_data) {
   GtkWidget* temp;
-  char str[1024];
   GtkPaned* paned;
   int pos;
 
   temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "con_widget"));
 
-  sprintf(str, "%s_channel", global.current_page->name);
-  paned = GTK_PANED(lookup_widget(global.win, str));
+  paned = GTK_PANED(global.current_page->toplevel);
 
   if (GTK_WIDGET_VISIBLE(temp)) {
     pos = (int)gtk_object_get_data(GTK_OBJECT(button), "pan_pos");
@@ -384,8 +454,6 @@ void on_hide_show_clicked(GtkButton *button, gpointer user_data) {
   } else {
     gtk_object_set_data(GTK_OBJECT(button), "pan_pos", 
 			(gpointer)(paned->child1_size));
-    sprintf(str, "%s_vbox", global.current_page->name);
-    temp = lookup_widget(global.win, str);
     gtk_paned_set_position (paned, 
 			    paned->child1_size + temp->allocation.width);
   }
@@ -395,38 +463,18 @@ void
 create_private_page (char* name) {
   GtkWidget *scrolledwindow27;
   GtkWidget *text4;
-  GtkWidget *vbox22;
   GtkWidget *tab_label;
-  GtkWidget *entry19;
-  GtkWidget *temp;
   GtkNotebook* notebook;
-  char str[100];
   chat_page_t* page;
+  hot_t* hot;
 
-  temp = lookup_widget(global.win, "notebook1");
-  gtk_notebook_set_page(GTK_NOTEBOOK(temp), 1);
+  switch_to_page(1);
   
-  vbox22 = gtk_vbox_new (FALSE, 0);
-  sprintf(str, "%s_vbox22", name);
-  gtk_object_set_data (GTK_OBJECT (global.win), str, vbox22);
-  gtk_widget_show (vbox22);
-
-  entry19 = gtk_entry_new ();
-  sprintf(str, "%s_topic", name);
-  gtk_object_set_data (GTK_OBJECT (global.win), str, entry19);
-  gtk_widget_show (entry19);
-  gtk_box_pack_start (GTK_BOX (vbox22), entry19, FALSE, FALSE, 0);
-
   scrolledwindow27 = gtk_scrolled_window_new (NULL, NULL);
-  sprintf(str, "%s_scrolled", name);
-  gtk_object_set_data (GTK_OBJECT (global.win), str, scrolledwindow27);
   gtk_widget_show (scrolledwindow27);
-  gtk_box_pack_start (GTK_BOX (vbox22), scrolledwindow27, TRUE, TRUE, 0);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow27), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   text4 = gtk_text_new (NULL, NULL);
-  sprintf(str, "%s_text", name);
-  gtk_object_set_data (GTK_OBJECT (global.win), str, text4);
   gtk_signal_connect_after (GTK_OBJECT (text4), "button_press_event",
                             GTK_SIGNAL_FUNC (on_text_button_press_event),
                             NULL);
@@ -438,15 +486,36 @@ create_private_page (char* name) {
   gtk_container_add (GTK_CONTAINER (scrolledwindow27), text4);
 
   tab_label = gtk_label_new(name);
-  sprintf(str, "%s_tab", name);
-  gtk_object_set_data (GTK_OBJECT (global.win), str, tab_label);
+  gtk_widget_ref (tab_label);
   gtk_widget_show(tab_label);
 
   notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook3"));
-  gtk_notebook_append_page(notebook, vbox22, tab_label);
+  gtk_notebook_append_page(notebook, scrolledwindow27, tab_label);
   page = chat_page_new(name, P_PRIVATE);
+  page->header = NULL;
+  page->main = text4;
+  page->wallop = NULL;
+  page->label = tab_label;
+  page->toplevel = scrolledwindow27;
+  gtk_object_set_data(GTK_OBJECT(tab_label), "page", page);
   gtk_notebook_set_page(GTK_NOTEBOOK(notebook),
-			gtk_notebook_page_num(GTK_NOTEBOOK(notebook), vbox22));
+			gtk_notebook_page_num(GTK_NOTEBOOK(notebook), scrolledwindow27));
+
+  hot = hotlist_search_user(name);
+  if (hot) {
+    if (hot->online == HOT_OFFLINE) chat_page_set_active(page, 0);
+  } else {
+    //    chat_page_set_active(page, 0);    
+    hotlist_add(name, NULL, 0);
+  }
+}
+
+void chat_print_time_stamp(chat_page_t* page, int message_type) {
+  char stime[200] = "";
+
+  if (global.options.timestamps == TIME_NONE) return;
+  chat_print_channel(page, message_type, "message", 
+		     current_time(stime, global.options.timestamps));
 }
 
 void cmd_public_message_no_log(char* data, int message_type) {
@@ -454,7 +523,8 @@ void cmd_public_message_no_log(char* data, int message_type) {
   char* user;
   chat_page_t* page;
   char* message;
-
+  user_timestamp_t* stamp;
+  
   channel = arg(data, 0);
   if (!channel) return;
   user = arg(NULL, 0);
@@ -465,11 +535,19 @@ void cmd_public_message_no_log(char* data, int message_type) {
     if (is_string_in_list(global.ignored_users, user)) return;
   }
 
+  stamp = timestamp_search(global.appearance, user);
+  if (!stamp) {
+    stamp = timestamp_new(user);
+    global.appearance = g_list_append(global.appearance, stamp);
+  } else timestamp_touch(stamp);
+
   // searching any type, as this function is also used for
   // private pages
   page = chat_page_search(channel, -1);
   if (!page) return;
 
+  chat_print_time_stamp(page, message_type);
+
   chat_print_channel(page, message_type, "user", "<");
   if (!strcasecmp(SERVER->nick, user))
     chat_print_channel(page, message_type, "yourself", user);
@@ -508,15 +586,15 @@ void send_afk(char* user, afk_t* afk) {
   tim /= 60;            // hours
   h = tim%24;           // %hours
   d = tim/24;           // days
-  if (d) sprintf(s1, _("%d days "), d);
+  if (d) sprintf(s1, "%d days ", d);
   else *s1 = 0;
-  if (h || d) sprintf(s2, _("%d hours "), h);
+  if (h || d) sprintf(s2, "%d hours ", h);
   else *s2 = 0;
-  if (m || h || d) sprintf(s3, _(" %d minutes"), m);
+  if (m || h || d) sprintf(s3, " %d minutes", m);
   else sprintf(s3, "%d minutes", 0);
   message = g_strdup_printf("AFK: %s [%s%s%s]",
 			    afk->message, s1, s2, s3);
-  send_private(user, message);
+  send_private(user, message, 1);
   g_free(message);
 }
 
@@ -542,15 +620,19 @@ int check_channel_wallop(char* data) {
   page = chat_page_search(channel, P_PUBLIC);
   if (!page) return 1;
   
-  if (global.options.sep_chwallop) {
+  if (global.options.piping & PIPE_CHWALLOP) {
     command = g_strdup_printf("%s %s %s", channel, nick, message);
     cmd_public_message_no_log(command, M_OP);
     g_free(command);
   } else {
+    int old_stamp = global.options.timestamps;
+    chat_print_time_stamp(page, M_PUBLIC);
     chat_print_channel(page, M_PUBLIC, "message", _("(Channel op) "));
+    global.options.timestamps = TIME_NONE;
     command = g_strdup_printf("%s %s %s", channel, nick, message);
     cmd_public_message_no_log(command, M_PUBLIC);
     g_free(command);
+    global.options.timestamps = old_stamp;
   }
   log(channel, LOG_CHANNEL, "<%s> [Wallop] %s\n", nick, message);
   
@@ -603,7 +685,7 @@ int check_server_pong(char* data) {
   char* ping;
   link_t* link;
   
-  if (strncmp(_("Pong from server"), data, 16)) return 0;
+  if (strncmp("Pong from server", data, 16)) return 0;
   
   temp = strdup(data+17);
   server = arg(temp, 0);
@@ -640,24 +722,23 @@ void check_get_motd(char* data) { // for macachu
 }
 */
 
-// Doofus opped you on channel #Lopster
 void check_op(char* data) {
-  char str[1024];
   char* pos;
-  GtkWidget* paned;
+  chat_page_t* page;
 
   pos = strchr(data, ' ');
   if (!pos) return;
+  pos = strchr(pos+1, ' ');
+  if (!pos) return;
   pos++;
-  if (!strncmp(pos, "opped you on channel ", 21)) {
-    pos += 21;
+  if (!strncmp(pos, "set you as operator on channel ", 31)) {
+    pos += 31;
     if (is_string_in_list(global.opchannel, pos)) return;
     global.opchannel = g_list_append(global.opchannel, strdup(pos));
     
-    sprintf(str, "%s_channel2", pos);
-    paned = lookup_widget(global.win, str);
-    if (!paned) return;
-    gtk_paned_set_position (GTK_PANED(paned), global.paned_pos2);
+    page = chat_page_search(pos, P_PUBLIC);
+    if (!page || !page->paned) return;
+    gtk_paned_set_position (GTK_PANED(page->paned), global.paned_pos2);
     return;
   }
 }
@@ -723,6 +804,8 @@ void send_emote(char* text) {
 
   if (!in_channel()) return;
 
+  while ((pos = strchr(text, '\"'))) *pos = '\'';
+  
   while ((pos = strchr(text, '\n')) != NULL) {
     *pos = 0;
     if (global.options.parse_color) text2 = cparse(text);
@@ -743,7 +826,7 @@ void send_emote(char* text) {
   g_free(command);
 }
 
-void send_private(char* nick, char* text) {
+void send_private(char* nick, char* text, int show) {
   char* text2;
   char* pos;
   char* command;
@@ -759,18 +842,21 @@ void send_private(char* nick, char* text) {
     send_command(CMD_CLIENT_PRIVMSG, command);
     g_free(command);
 
-    log(nick, LOG_PRIVATE, "<%s> %s\n", SERVER->nick, text2);
-    if (strcasecmp(nick, global.current_page->name)) {
-      chat_print("whisper", _("<to: "));
-      chat_print("whisper", nick);
-      chat_print("whisper", "> ");
-      chat_print_ln("whisper", text2);
-    } else {
-      command = g_strdup_printf("%s %s %s", 
-				global.current_page->name, 
-				SERVER->nick, text2);
-      cmd_public_message_no_log(command, M_PUBLIC);
-      g_free(command);
+    if (show) {
+      log(nick, LOG_PRIVATE, "<%s> %s\n", SERVER->nick, text2);
+      if (strcasecmp(nick, global.current_page->name)) {
+	chat_print_time_stamp(global.current_page, M_PUBLIC);
+	chat_print("whisper", _("<to: "));
+	chat_print("whisper", nick);
+	chat_print("whisper", "> ");
+	chat_print_ln("whisper", text2);
+      } else {
+	command = g_strdup_printf("%s %s %s", 
+				  global.current_page->name, 
+				  SERVER->nick, text2);
+	cmd_public_message_no_log(command, M_PUBLIC);
+	g_free(command);
+      }
     }
     text = pos+1;
   }
@@ -781,18 +867,21 @@ void send_private(char* nick, char* text) {
   send_command(CMD_CLIENT_PRIVMSG, command);
   g_free(command);
 
-  log(nick, LOG_PRIVATE, "<%s> %s\n", SERVER->nick, text2);
-  if (strcasecmp(nick, global.current_page->name)) {
-    chat_print("whisper", _("<to: "));
-    chat_print("whisper", nick);
-    chat_print("whisper", "> ");
-    chat_print_ln("whisper", text2);
-  } else {
-    command = 
-      g_strdup_printf("%s %s %s", global.current_page->name, 
-		      SERVER->nick, text2);
-    cmd_public_message_no_log(command, M_PUBLIC);
-    g_free(command);
+  if (show) {
+    log(nick, LOG_PRIVATE, "<%s> %s\n", SERVER->nick, text2);
+    if (strcasecmp(nick, global.current_page->name)) {
+      chat_print_time_stamp(global.current_page, M_PUBLIC);
+      chat_print("whisper", _("<to: "));
+      chat_print("whisper", nick);
+      chat_print("whisper", "> ");
+      chat_print_ln("whisper", text2);
+    } else {
+      command = 
+	g_strdup_printf("%s %s %s", global.current_page->name, 
+			SERVER->nick, text2);
+      cmd_public_message_no_log(command, M_PUBLIC);
+      g_free(command);
+    }
   }
 }
 
@@ -857,10 +946,9 @@ void mark_user(char* user, GdkPixmap *pix, GdkBitmap *bit) {
   for (dlist = global.chat_pages; dlist; dlist = dlist->next) {
     page = (chat_page_t*)(dlist->data);
     if (page->type != P_PUBLIC) continue;
-    sprintf(t, "%s_online", page->name);
-    //    printf("searching in %s\n", t);
+    temp = page->online;
+    if (!temp) continue;
 
-    temp = lookup_widget(global.win, t);
     for (i2 = 0; i2 < GTK_CLIST(temp)->rows; i2++) {
       gtk_clist_get_pixtext (GTK_CLIST(temp), i2, 0,
 			     &text, &space,
@@ -958,8 +1046,7 @@ GList* user_search(char* user) {
 
   if (!in_channel()) return NULL;
 
-  sprintf(t, "%s_online", global.current_page->name);
-  temp = GTK_CLIST(lookup_widget(global.win, t));
+  temp = GTK_CLIST(global.current_page->online);
   if (!temp) return NULL;
   for (i1 = 0; i1 < temp->rows; i1++) {
     gtk_clist_get_pixtext (temp, i1, 0,
@@ -985,8 +1072,7 @@ int user_search_full(char* user) {
 
   if (!in_channel()) return 0;
 
-  sprintf(t, "%s_online", global.current_page->name);
-  temp = GTK_CLIST(lookup_widget(global.win, t));
+  temp = GTK_CLIST(global.current_page->online);
   if (!temp) return 0;
   for (i1 = 0; i1 < temp->rows; i1++) {
     gtk_clist_get_pixtext (temp, i1, 0,
@@ -1072,8 +1158,6 @@ user_compare (GtkCList      *clist,
 }
 
 void highlight(chat_page_t* page, int high) {
-  char t[1024];
-  GtkWidget* temp;
   chat_page_t* page2;
 
   if (page->window) return;
@@ -1081,10 +1165,7 @@ void highlight(chat_page_t* page, int high) {
   page2 = chat_page_get_current_main();
   if (page2 == page) return;
 
-  sprintf(t, "%s_tab", page->name);
-  temp = lookup_widget(global.win, t);
-
-  gtk_widget_set_style(temp, global.style[high]);  
+  gtk_widget_set_style(page->label, global.style[high]);  
 }
 
 void set_afk(char* message) {
@@ -1108,26 +1189,50 @@ void set_afk(char* message) {
 }
 
 
+void chat_page_down(chat_page_t* page) {
+  GtkAdjustment* vertical_adjustment;
+  int value;
+
+  if (!page) return;
+  if (!page->main) return;
+  
+  vertical_adjustment = GTK_ADJUSTMENT(GTK_TEXT(page->main)->vadj);
+  value = vertical_adjustment->value + vertical_adjustment->page_size;
+  if (value > vertical_adjustment->upper - vertical_adjustment->lower - vertical_adjustment->page_size)
+    value = vertical_adjustment->upper - vertical_adjustment->lower - vertical_adjustment->page_size;
+
+  gtk_adjustment_set_value(vertical_adjustment, value);
+}
+
+void chat_page_up(chat_page_t* page) {
+  GtkAdjustment* vertical_adjustment;
+  int value;
+
+  if (!page) return;
+  if (!page->main) return;
+  
+  vertical_adjustment = GTK_ADJUSTMENT(GTK_TEXT(page->main)->vadj);
+  value = vertical_adjustment->value - vertical_adjustment->page_size;
+  if (value < 0) value = 0;
+
+  gtk_adjustment_set_value(vertical_adjustment, value);
+}
 
 chat_page_t* chat_page_get_current_main() {
   GtkNotebook* notebook;
   int page;
   GtkWidget* temp;
-  char* name;
+  chat_page_t* page1;
 
+  //  return global.current_page;
   notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook3"));
   page = gtk_notebook_get_current_page(notebook);
   if (page < 0) return NULL;
   temp = gtk_notebook_get_nth_page(notebook, page);
   temp = gtk_notebook_get_tab_label(notebook, temp);
-  name = GTK_LABEL(temp)->label;
   
-  if (!strcmp(name, _("Console")))
-    return chat_page_search("Console", -1);
-  else if (!strcmp(name, _("Channels")))
-    return chat_page_search("Channels", -1);
-  else
-    return chat_page_search(name, -1);
+  page1 = (chat_page_t*)gtk_object_get_data(GTK_OBJECT(temp), "page");
+  return page1;
 }
 
 chat_page_t* chat_page_new(char* name, int type) {
@@ -1137,36 +1242,29 @@ chat_page_t* chat_page_new(char* name, int type) {
   page->name = strdup(name);
   page->window = NULL;
   page->type = type;
+  page->header = NULL;
+  page->wallop = NULL;
+  page->main = NULL;
+  page->label = NULL;
+  page->online = NULL;
+  page->no = NULL;
+  page->toplevel = NULL;
+  page->timeout = -1;
+  page->snap = 0;
+  page->modified = 7;
+  page->active = 1;
 
   global.chat_pages = g_list_append(global.chat_pages, page);
   return page;
 }
 
-GtkWidget* chat_page_get_widget(chat_page_t* page) {
-  char str[1024];
-  GtkWidget* channel_widget;
-
-  if (page->type == P_PRIVATE)
-    sprintf(str, "%s_vbox22", page->name);
-  else if (page->type == P_PUBLIC)
-    sprintf(str, "%s_channel", page->name);
-  else if (!strcmp(page->name, "Console"))
-    sprintf(str, "hbox147");
-  else if (!strcmp(page->name, "Channels"))
-    sprintf(str, "scrolledwindow50");
-  else return NULL;
-  
-  channel_widget = lookup_widget(global.win, str);
-  return channel_widget;
-}
-
 void chat_page_destroy(chat_page_t* page) {
   GtkWidget* channel_widget;
-  
+
   if (page->type == P_FIX) return;
   global.chat_pages = g_list_remove(global.chat_pages, page);
 
-  channel_widget = chat_page_get_widget(page);
+  channel_widget = page->toplevel;
   if (!channel_widget) return;
 
   gtk_container_remove(GTK_CONTAINER(channel_widget->parent), 
@@ -1181,36 +1279,27 @@ void chat_page_destroy(chat_page_t* page) {
     global.current_page = chat_page_get_current_main();
   }
 
+  if (page->type == P_PRIVATE) hotlist_remove(page->name, 0);
+
   if (page->name) free(page->name);
   free(page);
 }
 
 void chat_page_attach_to_main(chat_page_t* page) {
-  GtkWidget* tab_label;
   GtkNotebook* notebook;
   GtkWidget* channel_widget;
-  char str[1024];
 
   if (page->window == NULL) return;
   
-  channel_widget = chat_page_get_widget(page);
+  channel_widget = page->toplevel;
   if (!channel_widget) return;
 
   gtk_widget_ref(channel_widget);
   gtk_container_remove(GTK_CONTAINER(channel_widget->parent), 
 		       channel_widget);
 
-  if (!strcmp(page->name, "Console"))
-    tab_label = gtk_label_new(_("Console"));
-  else if (!strcmp(page->name, "Channels"))
-    tab_label = gtk_label_new(_("Channels"));
-  else tab_label = gtk_label_new(page->name);
-  gtk_widget_show(tab_label);
-  sprintf(str, "%s_tab", page->name);
-  gtk_object_set_data (GTK_OBJECT (global.win), str, tab_label);
-  
   notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook3"));
-  gtk_notebook_append_page(notebook, channel_widget, tab_label);
+  gtk_notebook_append_page(notebook, channel_widget, page->label);
   gtk_widget_unref(channel_widget);
   gtk_notebook_set_page(GTK_NOTEBOOK(notebook),
 			gtk_notebook_page_num(GTK_NOTEBOOK(notebook),
@@ -1219,7 +1308,7 @@ void chat_page_attach_to_main(chat_page_t* page) {
 }
 
 void chat_page_attach_to_new(chat_page_t* page) {
-  char str[1024];
+  char str[2048];
   GtkWidget* channel;
   GtkWidget* window;
   GtkWidget* vbox;
@@ -1229,18 +1318,19 @@ void chat_page_attach_to_new(chat_page_t* page) {
   GtkWidget* entry;
 
   if (page->window) return;
-  channel = chat_page_get_widget(page);
+  channel = page->toplevel;
   if (!channel) return;
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size(GTK_WINDOW (window), 
 			      channel->allocation.width,
 			      channel->allocation.height+60);
-  if (page->type == P_PRIVATE)
-    sprintf(str, _("Private: %s"), page->name);
-  else if (page->type == P_PUBLIC)
-    sprintf(str, _("Channel: %s"), page->name);
-  else strcpy(str, _(page->name));
+  if (page->type == P_PRIVATE) sprintf(str, _("Private: "));
+  else if (page->type == P_PUBLIC) sprintf(str, _("Channel: "));
+  else *str = 0;
+  strcat(str, GTK_LABEL(page->label)->label);
+  if (!page->active) strcat(str, _(" [inactive]"));
+
   gtk_window_set_title (GTK_WINDOW (window), str);
   gtk_widget_show(window);
   gtk_object_set_data(GTK_OBJECT(window), "channel", page);
@@ -1310,11 +1400,13 @@ void chat_page_attach_to_new(chat_page_t* page) {
 }
 
 void chat_page_leave(chat_page_t* page) {
+  if (page->type == P_FIX) return;
+
   if (page->type == P_PUBLIC) {
-    if (global.status.connection > 1)
+    if ((global.status.connection > 1) && (page->active))
       send_command(CMD_CLIENT_PART, page->name);
     else chat_page_destroy(page);
-  } else if (page->type == P_PRIVATE) {
+  } else {
     chat_page_destroy(page);
   }
 }
@@ -1334,6 +1426,19 @@ chat_page_t* chat_page_search(char* name, int type) {
   return NULL;
 }
 
+void chat_page_clear(chat_page_t* page, int bottom, int top) {
+  if (top && page->wallop) {
+    gtk_text_set_point(GTK_TEXT(page->wallop), 0);
+    gtk_text_forward_delete(GTK_TEXT(page->wallop),
+			    gtk_text_get_length(GTK_TEXT(page->wallop)));
+  }
+  if (bottom && page->main) {
+    gtk_text_set_point(GTK_TEXT(page->main), 0);
+    gtk_text_forward_delete(GTK_TEXT(page->main),
+			    gtk_text_get_length(GTK_TEXT(page->main)));
+  }
+}
+
 int in_private() {
   return (global.current_page->type == P_PRIVATE);
 }
@@ -1342,6 +1447,46 @@ int in_channel() {
   return (global.current_page->type == P_PUBLIC);
 }
 
+void chat_page_set_active(chat_page_t* page, int active) {
+  char str[1024];
+
+  if (!page) return;
+  if (page->active == active) return;
+  if (page->type == P_FIX) return;
+
+  page->active = active;
+  // redraw label
+
+  if (!page->label) return;
+  if (active) gtk_widget_set_sensitive(page->label, TRUE);
+  else gtk_widget_set_sensitive(page->label, FALSE);
+  
+  if ((page->type == P_PUBLIC) && (page->active)) {
+    chat_print_channel(page, M_PUBLIC, "message", "================================\n");
+    chat_print_channel(page, M_PUBLIC, "message", " Server: ");
+    chat_print_channel(page, M_PUBLIC, "error", (SERVER->description)?(SERVER->description):(SERVER->address));
+    chat_print_channel(page, M_PUBLIC, "message", "\n");
+    chat_print_channel(page, M_PUBLIC, "message", "================================\n");
+    gtk_clist_clear(GTK_CLIST(page->online));
+  }
+  if (!page->window) return;
+
+  if (page->active) {
+    if (page->type == P_PRIVATE)
+      sprintf(str, _("Private: %s"), page->name);
+    else if (page->type == P_PUBLIC)
+      sprintf(str, _("Channel: %s"), page->name);
+    else strcpy(str, _(page->name));
+  } else {
+    if (page->type == P_PRIVATE)
+      sprintf(str, _("Private: %s [inactive]"), page->name);
+    else if (page->type == P_PUBLIC)
+      sprintf(str, _("Channel: %s [inactive]"), page->name);
+    else strcpy(str, _(page->name));
+  }
+  gtk_window_set_title (GTK_WINDOW (page->window), str);
+}
+
 chat_page_t* chat_page_find_unfixed() {
   GList* dlist;
   chat_page_t* page;
@@ -1353,3 +1498,80 @@ chat_page_t* chat_page_find_unfixed() {
   
   return NULL;
 }
+
+user_timestamp_t* timestamp_new(char* user) {
+  user_timestamp_t* result;
+  
+  result = (user_timestamp_t*)malloc(sizeof(user_timestamp_t*));
+  result->user = strdup(user);
+  timestamp_touch(result);
+
+  return result;
+}
+
+user_timestamp_t* timestamp_search(GList* stamps, char* user) {
+  GList* dlist;
+  user_timestamp_t* result;
+
+  for (dlist = stamps; dlist; dlist = dlist->next) {
+    result = (user_timestamp_t*)(dlist->data);
+    if (!strcasecmp(result->user, user)) return result;
+  }
+
+  return NULL;
+}
+
+void timestamp_touch(user_timestamp_t* stamp) {
+  gettimeofday(&(stamp->tv), 0);
+}
+
+int timestamp_difference(user_timestamp_t* stamp) {
+  struct timeval tv;
+  int result;
+
+  gettimeofday(&(tv), 0);
+
+  result = (tv.tv_sec-stamp->tv.tv_sec)*1000 + 
+    (tv.tv_usec-stamp->tv.tv_usec)/1000;
+  return (result);
+}
+
+chat_page_t* create_other_page(char* name, char* title) {
+  GtkWidget *scrolledwindow27;
+  GtkWidget *text4;
+  GtkWidget *tab_label;
+  GtkWidget *entry19 = NULL;
+  GtkNotebook* notebook;
+  chat_page_t* page;
+
+  scrolledwindow27 = gtk_scrolled_window_new (NULL, NULL);
+  gtk_widget_show (scrolledwindow27);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow27), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+
+  text4 = gtk_text_new (NULL, NULL);
+  gtk_signal_connect_after (GTK_OBJECT (text4), "button_press_event",
+                            GTK_SIGNAL_FUNC (on_text_button_press_event),
+                            NULL);
+
+  gtk_widget_set_style(text4, global.style[2]);  
+
+  gtk_widget_show (text4);
+  gtk_text_set_word_wrap(GTK_TEXT(text4), 1);
+  gtk_container_add (GTK_CONTAINER (scrolledwindow27), text4);
+
+  tab_label = gtk_label_new(title);
+  gtk_widget_ref (tab_label);
+  gtk_widget_show(tab_label);
+
+  notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook3"));
+  gtk_notebook_append_page(notebook, scrolledwindow27, tab_label);
+  page = chat_page_new(name, P_OTHER);
+  page->header = entry19;
+  page->main = text4;
+  page->wallop = NULL;
+  page->label = tab_label;
+  page->toplevel = scrolledwindow27;
+  gtk_object_set_data(GTK_OBJECT(tab_label), "page", page);
+
+  return page;
+}
