@@ -7,8 +7,12 @@ char* get_popup_user(int mode);
 GtkWidget* create_online_popup (void);
 void refresh_channels();
 
+void chat_page_down(chat_page_t* page);
+void chat_page_up(chat_page_t* page);
 void chat_print(char* color, char *text);
 void chat_print_ln(char* color, char* text);
+void chat_page_set_active(chat_page_t* page, int active);
+void chat_page_clear(chat_page_t* page, int bottom, int top);
 
 void chat_print_channel(chat_page_t* page, int message_type, 
 			char* color, char *text);
@@ -21,13 +25,14 @@ void chat_print_colored(chat_page_t* page, int message_type,
 void join_channel(char* name);
 void create_private_page (char* name);
 
+void chat_print_time_stamp(chat_page_t* page, int message_type);
 void cmd_public_message_no_log(char* data, int message_type);
 
 void send_global(char* text);
 void send_wallop(char* text);
 void send_chwallop(char* channel, char* text);
 void send_emote(char* text);
-void send_private(char* nick, char* text);
+void send_private(char* nick, char* text, int show);
 void send_public(char* channel, char* text);
 void send_afk(char* user, afk_t* afk);
 
@@ -58,7 +63,6 @@ void set_afk(char* message);
 
 chat_page_t* chat_page_get_current_main();
 chat_page_t* chat_page_new(char* name, int type);
-GtkWidget* chat_page_get_widget(chat_page_t* page);
 void chat_page_destroy(chat_page_t* page);
 void chat_page_attach_to_main(chat_page_t* page);
 void chat_page_attach_to_new(chat_page_t* page);
@@ -67,3 +71,10 @@ chat_page_t* chat_page_search(char* name, int type);
 int in_private();
 int in_channel();
 chat_page_t* chat_page_find_unfixed();
+
+user_timestamp_t* timestamp_new(char* user);
+user_timestamp_t* timestamp_search(GList* stamps, char* user);
+void timestamp_touch(user_timestamp_t* stamp);
+int timestamp_difference(user_timestamp_t* stamp);
+
+chat_page_t* create_other_page(char* name, char* title);
