@@ -15,6 +15,7 @@
 #include <dirent.h>
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <time.h>
 #include <netdb.h>
 #include <sys/utsname.h>
 
@@ -40,6 +41,8 @@
 #include "preferences.h"
 #include "log.h"
 #include "exec.h"
+#include "wizard.h"
+#include "whois.h"
 
 char* FolderNames(int no) {
   switch (no) {
@@ -180,8 +183,7 @@ int command_make_alias(char* iname, char* alias) {
   new_alias = (command_t*)malloc(sizeof(command_t));
   new_alias->id = comm->id;
   new_alias->name = strdup(alias);
-  sprintf(temp, _("see %s"), iname);
-  new_alias->syntax = strdup(temp);
+  new_alias->syntax = g_strdup_printf("see %s", iname);
   strcpy(temp, _("This is an alias for <"));
   strcat(temp, iname);
   strcat(temp, ">");
@@ -281,7 +283,7 @@ COMMAND(lopster_whisper) {
     client_message(_("Error"), _("Whisper what?"));
     return;
   }
-  send_private(nick, message);
+  send_private(nick, message, 1);
 }
 
 COMMAND(lopster_whois) {
@@ -292,11 +294,11 @@ COMMAND(lopster_whois) {
   nick = arg(data, 0);
   if (!nick) {
     if (global.reply)
-      send_command(CMD_CLIENT_WHOIS, global.reply);
+      whois_request(global.reply, WHOIS_SHOW);
     else
-      send_command(CMD_CLIENT_WHOIS, SERVER->nick);
+      whois_request(SERVER->nick, WHOIS_SHOW);
   } else {
-    send_command(CMD_CLIENT_WHOIS, nick);
+    whois_request(nick, WHOIS_SHOW);
   }
 }
 
@@ -353,7 +355,8 @@ COMMAND(lopster_help) {
   if (!com) {
     chat_print("message", _("[Available Commands]\n"));
     for (i2 = 0; i2 < NO_FOLDERS; i2++) {
-      if ((i2 == F_OP) && !global.opchannel) continue;
+      if ((i2 == F_OP) && !global.opchannel &&
+	  ((int)global.user.level < (int)L_MOD)) continue;
       if ((i2 == F_MOD) && 
 	  ((int)global.user.level < (int)L_MOD)) continue;
       if ((i2 == F_ADMIN) && 
@@ -393,7 +396,12 @@ COMMAND(lopster_help) {
       chat_print_ln("message", command->name);
       client_message(_("Syntax"), command->syntax);
       client_message(_("Description"), command->description);
-      if ((int)command->level > L_MOD)
+      if ((int)command->level > L_ADMIN)
+	if (global.user.level > L_ADMIN)
+	  client_message(_("Access"), _("Elite (and you are!)"));
+	else
+	  client_message(_("Access"), _("Elite (not you!)"));
+      else if ((int)command->level > L_MOD)
 	if (global.user.level > L_MOD)
 	  client_message(_("Access"), _("Admin or higher (and you are!)"));
 	else
@@ -468,54 +476,30 @@ COMMAND(lopster_chlevel) {
 }
 
 COMMAND(lopster_clear) {
-  GtkWidget* temp;
-  char* channel;
   char* where;
+  chat_page_t* page;
 
-  if (!strcasecmp("Channels", global.current_page->name)) return;
+  page = global.current_page;
+  if (page->type == P_FIX) return;
   
   where = arg(data, 0);
   if (where) {
     if (!strcasecmp(where, "TOP") && in_channel()) {
-      channel = g_strdup_printf("%s_text2", global.current_page->name);
-      temp = lookup_widget(global.win, channel);
-      gtk_text_set_point(GTK_TEXT(temp), 0);
-      gtk_text_forward_delete(GTK_TEXT(temp),
-			      gtk_text_get_length(GTK_TEXT(temp)));
-      g_free(channel);
+      chat_page_clear(page, 0, 1);
     } else if (!strcasecmp(where, "BOTTOM")) {
-      channel = g_strdup_printf("%s_text", global.current_page->name);
-      temp = lookup_widget(global.win, channel);
-      gtk_text_set_point(GTK_TEXT(temp), 0);
-      gtk_text_forward_delete(GTK_TEXT(temp),
-			      gtk_text_get_length(GTK_TEXT(temp)));
-      g_free(channel);
+      chat_page_clear(page, 1, 0);
     } else {
       client_message(_("Error"), _("Clear what?"));
     }
   } else {
-    channel = g_strdup_printf("%s_text", global.current_page->name);
-    temp = lookup_widget(global.win, channel);
-    gtk_text_set_point(GTK_TEXT(temp), 0);
-    gtk_text_forward_delete(GTK_TEXT(temp),
-			    gtk_text_get_length(GTK_TEXT(temp)));
-    g_free(channel);
-    if (in_channel()) {
-      channel = g_strdup_printf("%s_text2", global.current_page->name);
-      temp = lookup_widget(global.win, channel);
-      gtk_text_set_point(GTK_TEXT(temp), 0);
-      gtk_text_forward_delete(GTK_TEXT(temp),
-			      gtk_text_get_length(GTK_TEXT(temp)));
-      g_free(channel);
-    }
+    chat_page_clear(page, 1, 1);
   }
 }
 
 COMMAND(lopster_cloak) {
   send_command(CMD_CLIENT_CLOAK, "");
   if (global.status.connection > 1) {
-    global.status.whois_hide++;
-    send_command(CMD_CLIENT_WHOIS, SERVER->nick);
+    whois_request(SERVER->nick, WHOIS_NONE);
   }
 }
 
@@ -577,40 +561,35 @@ COMMAND(lopster_kick) {
   char* command;
 
   arg1 = arg(data, 0);
-  arg2 = arg(NULL, 0);
-  arg3 = arg(NULL, 0);
 
   if (!arg1) {
-    client_message(_("Error"), _("Please specify a channel or a user switch to a channel"));
+    client_message(_("Error"), _("Please specify a channel or a user or switch to a channel"));
     return;
   }
-  if (!arg2) {
-    command = g_strdup_printf("%s %s", global.current_page->name, arg1);
-    send_command(CMD_CLIENT_KICK, command);
-    g_free(command);
-    return;
-  }
-  if (!arg3) {
-    if (user_search_full(arg1)) {
-      command = g_strdup_printf("%s %s \"%s\"", 
-				global.current_page->name, arg1, arg2);
-    } else if (!strcmp(arg1, ".")) {
-      command = g_strdup_printf("%s %s", global.current_page->name, arg2);
-    } else {
-      command = g_strdup_printf("%s %s", arg1, arg2);
-    }
-    send_command(CMD_CLIENT_KICK, command);
-    g_free(command);
-    return;
-  } else {
-    if (!strcmp(arg1, "."))
-      command = g_strdup_printf("%s %s \"%s\"", global.current_page->name,
-				arg2, arg3);
+
+  if (user_search_full(arg1)) {
+    arg2 = arg(NULL, 1);
+    if (arg2)
+      command = g_strdup_printf("%s %s \"%s\"", global.current_page->name, arg1, arg2);
     else
+      command = g_strdup_printf("%s %s", global.current_page->name, arg1);
+    send_command(CMD_CLIENT_KICK, command);
+    g_free(command);
+  } else {
+    arg2 = arg(NULL, 0);
+    arg3 = arg(NULL, 1);
+    if (!arg2) {
+      client_message(_("Error"), _("You have to specify a user"));
+      return;
+    }
+    if (arg3) 
       command = g_strdup_printf("%s %s \"%s\"", arg1, arg2, arg3);
+    else
+      command = g_strdup_printf("%s %s", arg1, arg2);
     send_command(CMD_CLIENT_KICK, command);
     g_free(command);
   }
+
 }
 
 COMMAND(lopster_kill) {
@@ -697,7 +676,7 @@ COMMAND(lopster_wallop) {
 }
 
 COMMAND(lopster_part) {
-  on_button24_clicked(NULL, NULL);
+  chat_page_leave(global.current_page);
 }
 
 COMMAND(lopster_password) {
@@ -725,18 +704,21 @@ COMMAND(lopster_password) {
 
 COMMAND(lopster_ping) {
   char* nick;
-  time_t tim;
-  char* t;
+  user_timestamp_t* stamp;
 
   nick = arg(data, 0);
   if (!nick) {
     client_message(_("Error"), _("Please specify a nickname!"));
     return;
   }
-  time(&tim);
-  t = strdup(ctime(&tim));
-  t[strlen(t)-1] = 0;
-  client_message(_("Message"), _("pinging %s [%s]"), nick, t);
+
+  stamp = timestamp_search(global.userstamp, nick);
+  if (!stamp) {
+    stamp = timestamp_new(nick);
+    global.userstamp = g_list_append(global.userstamp, stamp);
+  } else timestamp_touch(stamp);
+  
+  client_message(_("Message"), _("Pinging %s"), nick);
   send_command(CMD_CLIENT_PING, nick);
 }
 
@@ -856,7 +838,7 @@ COMMAND(lopster_scheme) {
  
   scheme = arg(data, 1);
   if (!scheme) {
-    command = g_strdup_printf(_("Current scheme is [%s]"), global.scheme);
+    command = g_strdup_printf(_("Current scheme is [%s]"), global.scheme->name);
     client_message(_("Message"), command);
     g_free(command);
   } else {
@@ -925,7 +907,7 @@ COMMAND(lopster_reply) {
     client_message(_("Error"), _("Want to reply _nothing_?"));
     return;
   }
-  send_private(global.reply, message);
+  send_private(global.reply, message, 1);
 }
 
 COMMAND(lopster_private) {
@@ -986,7 +968,6 @@ COMMAND(lopster_chban) {
   char* reason;
   char* command;
   GtkCList* clist;
-  char t[1024];
   int user;
 
   channel = arg(data, 0);
@@ -1001,8 +982,7 @@ COMMAND(lopster_chban) {
   }
 
   if (in_channel()) {
-    sprintf(t, "%s_online", global.current_page->name);
-    clist = GTK_CLIST(lookup_widget(global.win, t));
+    clist = GTK_CLIST(global.current_page->online);
     user = search_user_in_list(clist, channel);
     if (user >= 0) nick = channel;
     else nick = arg(NULL, 0);
@@ -1129,7 +1109,7 @@ COMMAND(lopster_cversion) {
   if (in_channel())
     send_public(global.current_page->name, t2);
   else if (in_private())
-    send_private(global.current_page->name, t2);
+    send_private(global.current_page->name, t2, 1);
   else
     client_message(_("Message"), t2);
 }
@@ -1183,20 +1163,22 @@ COMMAND(lopster_chlimit) {
 
 COMMAND(lopster_sping) {
   char* server;
-  time_t tim;
-  char* t;
+  user_timestamp_t* stamp;
 
   server = arg(data, 0);
   if (!server) {
     client_message(_("Error"), _("Pinging, yes, but what server?"));
     return;
   }
-  time(&tim);
-  t = strdup(ctime(&tim));
-  t[strlen(t)-1] = 0;
-  client_message(_("Ping"), _("pinging server %s [%s]"), server, t);
+
+  stamp = timestamp_search(global.userstamp, server);
+  if (!stamp) {
+    stamp = timestamp_new(server);
+    global.userstamp = g_list_append(global.userstamp, stamp);
+  } else timestamp_touch(stamp);
+  
+  client_message(_("Message"), _("Pinging server %s"), server);
   send_command(CMD_CLIENT_PING_SERVER, server);
-  free(t);
 }
 
 COMMAND(lopster_friend) {
@@ -1223,13 +1205,17 @@ COMMAND(lopster_enemy) {
 
 COMMAND(lopster_hotadd) {
   char* nick;
+  char* des;
 
   nick = arg(data, 0);
   if (!nick) {
     client_message(_("Error"), _("You have to specify a user!"));
     return;
   }
-  add_to_hotlist(nick);
+
+  des = arg(NULL, 1);
+
+  hotlist_add(nick, des, 1);
 }
 
 COMMAND(lopster_hotrem) {
@@ -1240,7 +1226,7 @@ COMMAND(lopster_hotrem) {
     client_message(_("Error"), _("You have to specify a user!"));
     return;
   }
-  remove_from_hotlist(nick);
+  hotlist_remove(nick, 1);
 }
 
 COMMAND(lopster_clientinfo) {
@@ -1304,7 +1290,7 @@ COMMAND(lopster_clones) {
 
   which = arg(data, 0);
   if (!which) {
-    client_message(_("Error"), _("Print which clones?"));
+    client_message(_("Error"), _("Print what clones?"));
     return;
   }
   pos = arg(NULL, 0);
@@ -1322,7 +1308,7 @@ COMMAND(lopster_clones) {
       client_message(t, clone->ip);
       for (dlist2 = clone->nicks; dlist2; dlist2 = dlist2->next) {
 	pos = (char*)(dlist2->data);
-	client_message("--->", pos);
+	client_message("--->", "<%s>", pos);
 	cnt++;
       }
       cnt2++;
@@ -1352,12 +1338,11 @@ COMMAND(lopster_clones) {
       for (dlist2 = clone->nicks; dlist2; dlist2 = dlist2->next) {
 	pos = (char*)(dlist2->data);
 	if (user_search_full(pos)) {
-	  sprintf(t, "(C) %s", pos);
+	  client_message("--->", "(C) <%s>", pos);
 	  cnt3++;
 	} else {
-	  sprintf(t, "( ) %s", pos);
+	  client_message("--->", "( ) <%s>", pos);
 	}
-	client_message("--->", t);
 	cnt2++;
       }
     }
@@ -1374,8 +1359,7 @@ COMMAND(lopster_clones) {
       client_message(t, clone->ip);
       for (dlist2 = clone->nicks; dlist2; dlist2 = dlist2->next) {
 	pos = (char*)(dlist2->data);
-	sprintf(t, "%s", pos);
-	client_message("--->", t);
+	client_message("--->", "<%s>", pos);
       }
       break;
     }
@@ -1582,14 +1566,20 @@ COMMAND(lopster_chmuzzle) {
       client_message(_("Error"), _("Please specify a user switch to a channel"));
     return;
   }
+  if (!in_channel()) {
+    client_message(_("Error"), _("Please specify a user switch to a channel"));
+    return;
+  }
+
   reason = arg(NULL, 1);
   if (!reason) {
-    send_command(CMD_CLIENT_CHANNEL_MUZZLE, nick);
+    command = g_strdup_printf("%s %s", global.current_page->name, nick);
+    send_command(CMD_CLIENT_CHANNEL_MUZZLE, command);
   } else {
-    command = g_strdup_printf("%s \"%s\"", nick, reason);
+    command = g_strdup_printf("%s %s \"%s\"", global.current_page->name, nick, reason);
     send_command(CMD_CLIENT_CHANNEL_MUZZLE, command);
-    g_free(command);
   }
+  g_free(command);
 }
 
 COMMAND(lopster_chunmuzzle) {
@@ -1600,7 +1590,7 @@ COMMAND(lopster_chunmuzzle) {
 
   arg1 = arg(data, 0);
   arg2 = arg(NULL, 0);
-  arg3 = arg(NULL, 0);
+  arg3 = arg(NULL, 1);
 
   if (!arg1) {
     client_message(_("Error"), _("Please specify a channel or a user switch to a channel"));
@@ -1723,7 +1713,7 @@ COMMAND(lopster_killghost) {
     command = g_strdup_printf("ghost %s %s", ghost, passwd);
   }
 
-  send_private("NickServ", command);
+  send_private("NickServ", command, 1);
   g_free(command);
 }
 
@@ -1761,7 +1751,7 @@ COMMAND(lopster_whowas) {
     client_message(_("Error"), _("Whowas who?"));
     return;
   }
-  send_command(10121, nick);
+  send_command(CMD_CLIENT_WHO_WAS, nick);
 }
 
 COMMAND(lopster_rcolors) {
@@ -1857,7 +1847,7 @@ COMMAND(lopster_rainbow) {
   if (in_channel())
     send_public(global.current_page->name, command);
   else
-    send_private(global.current_page->name, command);
+    send_private(global.current_page->name, command, 1);
 }
 
 COMMAND(lopster_browse) {
@@ -1886,54 +1876,20 @@ COMMAND(lopster_browse) {
 COMMAND(lopster_send) {
   char* nick;
   char* file;
-  char* command;
-  struct stat st;
-  transfer_t* new_trans;
-  struct sockaddr_in localaddr;
-  int len = sizeof(struct sockaddr_in);
-  socket_t* socket;
 
   nick = arg(data, 0);
   file = arg(NULL, 1);
 
-  if (!nick || !file) {
-    client_message(_("Error"), _("Wrong format in command send!"));
+  if (global.network.port == 0) {
+    client_message(_("Error"), _("You cannot send files at the moment if you are firewalled"));
     return;
   }
-  if (stat(file, &st) < 0) {
-    client_message(_("Error"), _("File does not exist!"));
+  if (!nick) {
+    client_message(_("Error"), _("Missing nickname!"));
     return;
   }
-
-  new_trans = transfer_new();
-  new_trans->longname = strdup(file);
-  new_trans->winname = strdup(file);
-  convert_to_win(new_trans->winname);
-  new_trans->shortname =
-    strdup(extract_short_name(new_trans->longname));
-  new_trans->user = strdup(nick);
-  new_trans->type = T_UPLOAD;
-  new_trans->is_dcc = TRUE;
-  new_trans->mime_type = get_mimetype(file);
-  
-  send_command(CMD_CLIENT_USERSPEED, new_trans->user);
-  new_trans->size = st.st_size;
-  socket = socket_new(S_TRANSFER);
-  socket->data = new_trans;
-  
-  transfer_insert(socket);
-  upload_start(socket);
-  
-  getsockname(global.napster->fd,
-	      (struct sockaddr *)&localaddr, &len);
-  command = 
-    g_strdup_printf("%s %cSEND %s %lu %d \"%s\" %lu %s %d%c", 
-		    nick, '\001', SERVER->nick, 
-		    (unsigned long)BSWAP32(localaddr.sin_addr.s_addr), 
-		    global.network.port, new_trans->winname, 
-		    (unsigned long)st.st_size, "checksum", 0, '\001');
-  send_command(CMD_CLIENT_PRIVMSG, command);
-
+  if (file) dcc_send_file(nick, file);
+  else dcc_select_file(nick);
 }
 
 COMMAND(lopster_slink) {
@@ -2046,6 +2002,212 @@ COMMAND(lopster_masskill) {
     send_command(CMD_CLIENT_MASS_KILL, ip);
   }
 }
-COMMAND(lopster_test) {
-  //  ping(data);
+
+COMMAND(lopster_shistogram) {
+  client_message(NULL, _("Requesting server histogram..."));
+  send_command(CMD_CLIENT_HISTOGRAM, "");
 }
+
+COMMAND(lopster_register) {
+  char* user;
+  char* passwd;
+  char* email;
+  char* level;
+  char* command;
+
+  if (global.status.connection < 2) return;
+  user = arg(data, 0);
+  passwd = arg(NULL, 0);
+  email = arg(NULL, 0);
+  level = arg(NULL, 0);
+
+  if (!user) {
+    command = g_strdup_printf("%s %s anon@anon",
+			      SERVER->nick, SERVER->passwd);
+    send_command(CMD_CLIENT_REGISTER_USER, command);
+    free(command);
+    return;
+  }
+  if (!passwd || !email) {
+    client_message(_("Error"), _("Specify password and E-mail."));
+    return;
+  }
+  if (level)
+    command = g_strdup_printf("%s %s %s %s",
+			      user, passwd, email, level);
+  else
+    command = g_strdup_printf("%s %s %s",
+			      user, passwd, email);
+  send_command(CMD_CLIENT_REGISTER_USER, command);
+  free(command);
+}
+
+COMMAND(lopster_redirect) {
+  char* user;
+  char* server;
+  char* port;
+  char* command;
+
+  user = arg(data, 0);
+  server = arg(NULL, 0);
+  port = arg(NULL, 0);
+  
+  if (!user || !server || !port) {
+    client_message(_("Error"), _("Wrong number of arguments."));
+    return;
+  }
+  command = g_strdup_printf("%s %s %s", user, server, port);
+  send_command(CMD_CLIENT_REDIRECT, command);
+  free(command);
+}
+
+COMMAND(lopster_cycle) {
+  char* user;
+  char* server;
+  char* command;
+
+  user = arg(data, 0);
+  server = arg(NULL, 0);
+  
+  if (!user || !server) {
+    client_message(_("Error"), _("Wrong number of arguments."));
+    return;
+  }
+  command = g_strdup_printf("%s %s", user, server);
+  send_command(CMD_CLIENT_REDIRECT, command);
+  free(command);
+}
+
+COMMAND(lopster_limit) {
+  char* tag;
+  char* ip;
+  int add;
+
+  tag = arg(data, 0);
+  if (!tag || !strcasecmp(tag, "list")) {
+    client_message(_("Message"), _("Requesting limits"));
+    add = 2;
+  } else if (!strcasecmp(tag, "add")) {
+    add = 0;
+  } else if (!strcasecmp(tag, "remove")) {
+    add = 1;
+  } else {
+    client_message(_("Error"), _("Unknown action"));
+    return;
+  }
+
+  ip = arg(NULL, 1);
+  if (!ip && (add != 2)) {
+    client_message(_("Error"), _("No ip specified"));
+    return;
+  }
+
+  if (ip) send_command(CMD_CLIENT_CLASS_ADD+add, ip);
+  else send_command(CMD_CLIENT_CLASS_ADD+add, "");
+}
+
+COMMAND(lopster_iline) {
+  char* tag;
+  char* ip;
+  int add;
+
+  tag = arg(data, 0);
+  if (!tag || !strcasecmp(tag, "list")) {
+    client_message(_("Message"), _("Requesting ilines"));
+    add = 2;
+  } else if (!strcasecmp(tag, "add")) {
+    add = 0;
+  } else if (!strcasecmp(tag, "remove")) {
+    add = 1;
+  } else {
+    client_message(_("Error"), _("Unknown action"));
+    return;
+  }
+
+  ip = arg(NULL, 1);
+  if (!ip && (add != 2)) {
+    client_message(_("Error"), _("No ip specified"));
+    return;
+  }
+
+  if (ip) send_command(CMD_CLIENT_ILINE_ADD+add, ip);
+  else send_command(CMD_CLIENT_ILINE_ADD+add, "");
+}
+
+COMMAND(lopster_dline) {
+  char* tag;
+  char* ip;
+  int add;
+
+  tag = arg(data, 0);
+  if (!tag || !strcasecmp(tag, "list")) {
+    client_message(_("Message"), _("Requesting dlines"));
+    add = 2;
+  } else if (!strcasecmp(tag, "add")) {
+    add = 0;
+  } else if (!strcasecmp(tag, "remove")) {
+    add = 1;
+  } else {
+    client_message(_("Error"), _("Unknown action"));
+    return;
+  }
+
+  ip = arg(NULL, 1);
+  if (!ip && (add != 2)) {
+    client_message(_("Error"), _("No ip specified"));
+    return;
+  }
+
+  if (ip) send_command(CMD_CLIENT_DLINE_ADD+add, ip);
+  else send_command(CMD_CLIENT_DLINE_ADD+add, "");
+}
+
+COMMAND(lopster_eline) {
+  char* tag;
+  char* ip;
+  int add;
+
+  tag = arg(data, 0);
+  if (!tag || !strcasecmp(tag, "list")) {
+    client_message(_("Message"), _("Requesting elines"));
+    add = 2;
+  } else if (!strcasecmp(tag, "add")) {
+    add = 0;
+  } else if (!strcasecmp(tag, "remove")) {
+    add = 1;
+  } else {
+    client_message(_("Error"), _("Unknown action"));
+    return;
+  }
+
+  ip = arg(NULL, 1);
+  if (!ip && (add != 2)) {
+    client_message(_("Error"), _("No ip specified"));
+    return;
+  }
+
+  if (ip) send_command(CMD_CLIENT_ELINE_ADD+add, ip);
+  else send_command(CMD_CLIENT_ELINE_ADD+add, "");
+}
+
+#ifdef PROTOCOL_DEBUG
+COMMAND(lopster_test) {
+  
+  /*
+  GList* list;
+  char* pos;
+
+  list = create_allowed_ports(data);
+  pos = create_string_from_ports(list);
+  printf("ports: [%s]\n", pos);
+  GList* dlist;
+  hot_t* hot;
+
+  for (dlist = global.hotlist; dlist; dlist = dlist->next) {
+    hot = (hot_t*)(dlist->data);
+    client_message("Hotlist", "%d %s", hot->visible, hot->user);
+  }
+  */
+  return;
+}
+#endif
