@@ -63,12 +63,16 @@
 #define C_RAINBOW           63
 #define C_RAW               64
 #define C_RCOLORS           65
+#define C_REDIRECT          66
+#define C_CYCLE             67
 
-#define C_OP               100
-#define C_DEOP             101
-#define C_CHVOICE          102
-#define C_CHUNVOICE        103
-#define C_LINKS            104
+#ifdef PROTOCOL_DEBUG
+ #define C_TEST            100
+#endif
+#define C_OP               101
+#define C_DEOP             102
+#define C_CHVOICE          103
+#define C_CHUNVOICE        104
 #define C_INVITE           106
 #define C_CHMUZZLE         107
 #define C_CHUNMUZZLE       108
@@ -86,7 +90,13 @@
 #define C_SKILL            120
 #define C_SRELOAD          121
 #define C_MASSKILL         122
-#define C_TEST             123
+#define C_LINKS            123
+#define C_SHISTOGRAM       124
+#define C_REGISTER         125
+#define C_LIMIT            126
+#define C_ILINE            127
+#define C_DLINE            128
+#define C_ELINE            129
 
 #define COMMAND(f) void f (char *data)
 
@@ -212,5 +222,14 @@ COMMAND(lopster_sremove);
 COMMAND(lopster_skill);
 COMMAND(lopster_sreload);
 COMMAND(lopster_masskill);
+COMMAND(lopster_shistogram);
+COMMAND(lopster_register);
+COMMAND(lopster_redirect);
+COMMAND(lopster_cycle);
+COMMAND(lopster_limit);
+COMMAND(lopster_iline);
+COMMAND(lopster_dline);
+COMMAND(lopster_eline);
+#ifdef PROTOCOL_DEBUG
 COMMAND(lopster_test);
-COMMAND(lopster_prefix);
+#endif
