@@ -20,8 +20,8 @@
 
 #include "lopster.h"
 #include "support.h"
-#include "connection.h"
 #include "global.h"
+#include "connection.h"
 #include "commands.h"
 #include "search.h"
 #include "transfer.h"
@@ -35,8 +35,11 @@
 #include "resume.h"
 #include "chat.h"
 #include "dialog.h"
+#include "whois.h"
 
-#define NOT_DOTTED_QUAD ((u_long)-1)
+#ifndef INADDR_NONE
+#define INADDR_NONE ((u_long)-1)
+#endif
 
 int server_login(gpointer data, gint source, GdkInputCondition condition);
 int connection_timeout(gpointer data);
@@ -102,7 +105,7 @@ socket_t* socket_new(int type) {
     sock->max_cnt = global.network.transfer_timeout;
     break;
   case S_SERVER:
-    sock->max_cnt = 300;
+    sock->max_cnt = SERVER_TIMEOUT;
     break;
   case S_HTTP:
     sock->max_cnt = 20;
@@ -110,6 +113,9 @@ socket_t* socket_new(int type) {
   case S_DATA:
     sock->max_cnt = 0;
     break;
+  case S_PING:
+    sock->max_cnt = 5;
+    break;
   default:
     sock->max_cnt = 100;
     break;
@@ -120,11 +126,12 @@ socket_t* socket_new(int type) {
   return sock;
 }
 
-void socket_end(socket_t* socket, int data) {
+void socket_end(socket_t* socket, void* data) {
   transfer_t* transfer;
   server_t* server;
   int tim;
   GtkWidget* temp;
+  int* int_data;
 
 #ifdef CONNECTION_DEBUG
   printf("ending socket [%d]\n", socket->fd);
@@ -139,10 +146,11 @@ void socket_end(socket_t* socket, int data) {
   
   switch (socket->type) {
   case S_TRANSFER:
+    int_data = (int*)data;
     transfer = (transfer_t*)(socket->data);
     if (transfer) {
       if (transfer->type == T_DOWNLOAD) {
-	if ((tim = transfer_to_retry(data)) >= 0) {
+	if ((tim = transfer_to_retry(*int_data)) >= 0) {
 #ifdef CONNECTION_DEBUG
 	  printf("adding timeout\n");
 #endif
@@ -150,11 +158,29 @@ void socket_end(socket_t* socket, int data) {
  	    gtk_timeout_add(tim, transfer_remote_timeout, socket);
 	}
       }
-      transfer_end(socket->data, data);
-      transfer_status_set(socket, data);
+      transfer_end(socket, *int_data);
+      //      transfer_status_set(socket, data);
       
-      if (transfer_to_destroy(socket, data))
+#ifdef CONNECTION_DEBUG
+      printf("data %d\n", *int_data);
+#endif
+      if (transfer_to_destroy(socket, *int_data)) {
+#ifdef CONNECTION_DEBUG
+	printf("destroy\n");
+#endif
 	transfer_clist_remove(socket);
+#ifdef WINMX_QUEUE_BEHAVIER
+      } else if (transfer && (*int_data == S_REMOTE)) {
+	user_info_t* userinfo = search_user_info(transfer->user, 1);
+	if (userinfo) {
+	  if (userinfo->client == C_UNKNOWN) {
+	    whois_request(transfer->user, WHOIS_NONE);
+	  } else if (userinfo->client == C_WINMX) {
+	    send_private(transfer->user, "//WantQueue", 0);
+	  }
+	}
+#endif
+      }
     }
     break;
   case S_HTTP:
@@ -163,21 +189,30 @@ void socket_end(socket_t* socket, int data) {
   case S_SERVER:
     server = (server_t*)(socket->data);
     if (server) {
-      global.status.connection = 0;
       connect_progress((char*)data);
       napster_reset();
+      global.status.connection = 0;
       if (global.connect_win) {
 	temp = lookup_widget(global.connect_win, "connection_ok");
 	gtk_widget_set_sensitive(temp, TRUE);
       }
     }
+    break;
+  case S_BROWSE:
+    if (!data && (socket->data)) {
+#ifdef CONNECTION_DEBUG
+      printf("p2p browse failed, starting old one\n");
+#endif
+      send_command(CMD_CLIENT_BROWSE, (char*)(socket->data));
+    }
+    break;
   default:
     break;
   }
 
 }
 
-void socket_destroy(socket_t* socket, int data) {
+void socket_destroy(socket_t* socket, void* data) {
   
   if (!socket) return;
   
@@ -190,12 +225,10 @@ void socket_destroy(socket_t* socket, int data) {
 
   switch (socket->type) {
   case S_TRANSFER:
-    if (socket->data)
-      transfer_destroy(socket->data);
+    if (socket->data) transfer_destroy(socket->data);
     break;
   case S_BROWSE:
-    if (socket->data)
-      free(socket->data);
+    if (socket->data) free(socket->data);
     break;
   case S_SERVER:
     if (socket->data) {
@@ -319,13 +352,13 @@ int connect_socket(socket_t* s, char* proto, int type) {
 
   protocol = getprotobyname(proto);
   if (!protocol) {
-    socket_destroy(s, (int)_("Could not get protocol"));
+    socket_destroy(s, (void*)_("Could not get protocol"));
     return 0;
   }
   sock = socket(AF_INET, type, protocol->p_proto);
   
   if (sock < 0) {
-    socket_destroy(s, (int)_("Could not create socket"));
+    socket_destroy(s, (void*)_("Could not create socket"));
     return 0;
   }
   s->fd = sock;
@@ -336,7 +369,7 @@ int connect_socket(socket_t* s, char* proto, int type) {
   server.sin_port = s->port;
   
   if (server.sin_addr.s_addr == -1) {
-    socket_destroy(s, (int)_("Invalid Address"));
+    socket_destroy(s, (void*)_("Invalid Address"));
     return 0;
   }
   
@@ -349,13 +382,13 @@ int connect_socket(socket_t* s, char* proto, int type) {
 
   if ((res == 0) || (errno == EINPROGRESS)) return 1;
   else {
-    socket_destroy(s, (int)_("Could not connect socket"));
+    socket_destroy(s, (void*)_("Could not connect socket"));
     return 0;
   }
 }
 
 int get_best_server(socket_t* socket) {
-  connect_progress(_("Get best server"));
+  connect_progress(_("Getting best server"));
   if (!connect_socket(socket, "TCP", SOCK_STREAM)) {
     return 0;
   }
@@ -393,6 +426,8 @@ gint send_command_real(napster_command_t* command) {
   msg[length+4] = 0;
 
   global.queue = g_list_remove(global.queue, command);
+  free(command->data);
+  free(command);
 
   while(sendt < (length+4)) {
     res = send(global.napster->fd, msg+sendt, length+4-sendt, 0);
@@ -466,6 +501,7 @@ int server_get_input(gpointer data, gint source, GdkInputCondition condition) {
   char *mess;
   gint16 type;
   socket_t* socket = (socket_t*)data;
+  GtkWidget* temp;
 
   if (condition != GDK_INPUT_READ) {
     napster_disconnect(_("Server read error"));
@@ -479,6 +515,10 @@ int server_get_input(gpointer data, gint source, GdkInputCondition condition) {
 #ifdef PROTOCOL_DEBUG
   log("protocol", LOG_OTHER, "R: %5d L: %4d [%s]\n", type, strlen(mess), mess);
 #endif
+  if (socket->cnt > 0) {
+    temp = lookup_widget(global.win, "progressbar1");
+    gtk_progress_bar_update(GTK_PROGRESS_BAR(temp), 0);
+  }
   socket->cnt = 0;
   handle_command(type, mess);
   
@@ -565,10 +605,10 @@ void napster_reset() {
   char* t;
   GList* dlist;
   resume_t* resume;
-  GtkCList* clist;
-  int row;
-  user_t* userinfo;
-  
+  user_timestamp_t* stamp;
+  chat_page_t* page;
+  hot_t* hot;
+
   // delete ignored users
   dlist = g_list_first(global.ignored_users);
   while (dlist) {
@@ -584,34 +624,63 @@ void napster_reset() {
     gtk_clist_clear(GTK_CLIST(temp));
   }
   
-  // reset hotlist
-  clist = GTK_CLIST(lookup_widget(global.win, "clist16"));
-  for (row = 0; row < clist->rows; row++) {
-    userinfo = gtk_clist_get_row_data(clist, row);
-    hotlist_remove_user_files(userinfo);
+  // clear recent list
+  for (dlist = global.appearance; dlist; dlist = dlist->next) {
+    stamp = (user_timestamp_t*)(dlist->data);
+    if (stamp->user) free(stamp->user);
+    free(stamp);
   }
+  g_list_free(global.appearance);
+  global.appearance = NULL;
+
+  for (dlist = global.userstamp; dlist; dlist = dlist->next) {
+    stamp = (user_timestamp_t*)(dlist->data);
+    if (stamp->user) free(stamp->user);
+    free(stamp);
+  }
+  g_list_free(global.userstamp);
+  global.userstamp = NULL;
+
+  // reset hotlist
+  for (dlist = global.hotlist; dlist; dlist = dlist->next) {
+    hot = (hot_t*)(dlist->data);
+    hotlist_user_unknown(hot);
+  }
+  /*
   while (global.browses) {
-    hotlist_remove_user_files((user_t*)(global.browses->data));
+    browse_clear_files((user_t*)(global.browses->data));
     free(global.browses->data);
     global.browses = 
       g_list_remove(global.browses, global.browses->data);
   }
+  */  
 
   // resetting incomplete files
+  temp = lookup_widget(global.win, "resume_tree");
+  gtk_clist_freeze(GTK_CLIST(temp));
   for (dlist = global.incomplete; dlist; dlist = dlist->next) {
     resume = (resume_t*)(dlist->data);
     if (resume->status < R_QUEUED) {
       resume->status = R_NONE;
-      resume_list_update(resume);
+      resume_list_update(resume, 0);
     }
     resume_remove_search(resume);
   }
+  gtk_clist_thaw(GTK_CLIST(temp));
+
+  while (global.whois_requests) {
+    whois_remove(global.whois_requests->data);
+  }
 
   // reset left searches
   while (global.searches) {
     search_remove((search_t*)(global.searches->data));
   }
 
+  global.status.searching = 0;
+  search_update_counter();
+  setup_sensitive(-1);
+
   clones_clear();
 
   // deleting operator channels
@@ -632,16 +701,17 @@ void napster_reset() {
   global.share_queue = NULL;
 
   global.status.searching = 0;
-  global.status.whois_hide = 0;
   
   global.user.status = STATUS_INACTIVE;
   global.user.level = L_USER;
   
-  update_user_stats();
+  update_user_stats(0);
   
   // unsharing library
-  lib_unshare_flag(FLAG_SHARED, FLAG_TO_SHARE, NULL);
-  
+  lib_del_flag(FLAG_SHARED, NULL);
+  global.user.unshared = global.user.shared;
+  //  lib_unshare_flag(FLAG_SHARED, FLAG_TO_SHARE, NULL);
+
   // clear channel list
   temp = lookup_widget(global.win, "channel_list");
   gtk_clist_clear(GTK_CLIST(temp));
@@ -653,18 +723,33 @@ void napster_reset() {
   setup_sensitive(0);
   if (global.napster && SERVER) SERVER->network = N_UNKNOWN;
 
+  // inactivate chat pages
+  if (global.status.connection == 2) {
+    for (dlist = global.chat_pages; dlist; dlist = dlist->next) {
+      page = (chat_page_t*)(dlist->data);
+      if (page->type != P_FIX) {
+	chat_page_set_active(page, 0);
+	chat_print_channel(page, M_PUBLIC, "message", "================================\n");
+	chat_print_channel(page, M_PUBLIC, "message", " Leaving Server......\n");
+	chat_print_channel(page, M_PUBLIC, "message", "================================\n");
+      }
+    }
+  }
+  
+  temp = lookup_widget(global.win, "progressbar1");
+  gtk_progress_bar_update(GTK_PROGRESS_BAR(temp), 0);
+
   // leaving sockets untouched
-  // leaving chat pages untouched
 }
 
 void napster_disconnect(char* message) {
   // message == NULL is user disconnect
   
   if (!global.napster) return;
-  socket_end(global.napster, (int)message);
+  socket_end(global.napster, (void*)message);
   if ((global.napster->timeout == -1) && 
       global.network.auto_reconnect && message) {
-    connect_progress(_("Activate reconnect timer"));
+    //    connect_progress(_("Activate reconnect timer"));
     global.napster->timeout = 
       gtk_timeout_add(global.network.auto_reconnect*1000,
 		      auto_reconnect, global.napster);
@@ -840,13 +925,13 @@ gint get_con_type(gpointer data, gint source,
   }
   
 #ifdef TRANSFER_DEBUG
-  printf("in [%s]\n", buffer);
+  printf("in [%s] %d\n", buffer, cnt);
 #endif
   if (!strncmp(buffer, "GETLIST", 7)) {
     recv(source, buffer, 7, 0);
     //    client_message(NULL, _("Someone is browsing your files"));
     socket->type = S_SHARE;
-    socket->max_cnt = 50000;
+    socket->max_cnt = 30;
     socket->input = 
       gdk_input_add(socket->fd, GDK_INPUT_WRITE, 
 		    GTK_SIGNAL_FUNC(send_browse_nick), socket);
@@ -854,7 +939,7 @@ gint get_con_type(gpointer data, gint source,
     recv(source, buffer, 8, 0);
     //    client_message(NULL, _("Someone is sending his shared list"));
     socket->type = S_BROWSE;
-    socket->max_cnt = 20000;
+    socket->max_cnt = 30;
     socket->input = 
       gdk_input_add(socket->fd, GDK_INPUT_READ, 
 		    GTK_SIGNAL_FUNC(get_browse_nick), socket);
@@ -882,6 +967,7 @@ gint get_con_type(gpointer data, gint source,
 #endif
     send(source, "INVALID REQUEST", 
 	 strlen("INVALID REQUEST"), 0);
+    socket_destroy(socket, 0);
   }
   return 1;
 }
@@ -902,6 +988,7 @@ int handle_upload_request(gpointer data, gint source,
   
   socket->ip_long = from.sin_addr.s_addr;
   socket->port = from.sin_port;
+  //  printf("got ip %lu\n", socket->ip_long);
 
   send(socket->fd, "1", 1, 0);
   
@@ -921,10 +1008,10 @@ void create_upload_port(int port, int send) {
   int protocol;
   struct sockaddr_in server;
   char str[2048];
-  GList* dlist;
+  int save;
 
   if (global.upload_socket)
-    socket_destroy(global.upload_socket, 1);
+    socket_destroy(global.upload_socket, NULL);
   global.upload_socket = NULL;
 
   if (send) {
@@ -955,7 +1042,7 @@ void create_upload_port(int port, int send) {
   setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &len, sizeof(len));
   setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char *)&len, sizeof(len));
   
-  if (g_list_find(global.allowed_ports, (gpointer)port)) {
+  if (port_is_allowed(global.allowed_ports, port)) {
     global.upload_socket->port = htons(port);
     server.sin_family = AF_INET;
     server.sin_addr.s_addr = global.upload_socket->ip_long;
@@ -966,33 +1053,37 @@ void create_upload_port(int port, int send) {
   } else {
     client_message(_("Error"), _("Port %d is not in the list of allowed ports"), port);
   }
+  client_message(_("Message"), _("Searching for port...."), port);
 
   send = 1;
 
-  for (dlist = global.allowed_ports; dlist; dlist = dlist->next) {
-    global.upload_socket->port = htons((int)(dlist->data));
+  port = get_next_port(global.allowed_ports, port);
+  save = port;
+  while (1) {
+    global.upload_socket->port = htons(port);
     server.sin_port = global.upload_socket->port;
 
     if (bind(sock, (struct sockaddr *)&server, sizeof(server)) >= 0)
       goto success;
+    port = get_next_port(global.allowed_ports, port);
+    if (port == save) goto failure;
   }    
   goto failure;
   
  success:
-  global.network.port = ntohs(global.upload_socket->port);
-  client_message(_("Information"), _("Data Port was set to %d"), 
-		 global.network.port);
+  global.network.port = port;
+  client_message(_("Information"), _("Data Port was set to %d"), global.network.port);
   client_message(_("Information"), _("At most %d incoming connections are allowed at a time."), 
 		 10 + global.limit.max_uploads + global.limit.max_downloads);
 
   listen(sock, 10 + global.limit.max_uploads + global.limit.max_downloads);
 #ifdef PROTOCOL_DEBUG
-  log("protocol", LOG_OTHER, _("Listen for incoming connections on port [%d]\n"), global.upload_socket->port);
+  log("protocol", LOG_OTHER, _("Listen for incoming connections on port [%d]\n"), global.network.port);
 #endif
   setup_preferences(P_GENERAL);
 
-  if (send) {
-    sprintf(str, "%d", htons(global.upload_socket->port));
+  if (send && global.status.connection == 2) {
+    sprintf(str, "%d", global.network.port);
     send_command(CMD_CLIENT_CHANGE_DATA_PORT, str);
   }
 
@@ -1182,10 +1273,10 @@ void socket_show_clist(socket_t* socket) {
       copy_string(tstr[1], (transfer->is_dcc)?"Yes":"No");
       gtk_clist_append(clist, list);
       if (transfer->user_info) {
-	copy_string(tstr[0], _("Userinfo (cur/max/refs)"));
-	sprintf(tstr[1], "%d/%d/%d", transfer->user_info->cur,
-		transfer->user_info->max,
-		transfer->user_info->cnt);
+	copy_string(tstr[0], _("Userinfo"));
+	sprintf(tstr[1], "[%d running][%d allowed]", 
+		transfer->user_info->cur,
+		transfer->user_info->max);
 	gtk_clist_append(clist, list);
       }
       break;
