@@ -5,8 +5,8 @@ void connect_progress(char* text);
 void connect_success();
 
 socket_t* socket_new(int type);
-void socket_destroy(socket_t* socket, int data);
-void socket_end(socket_t* socket, int data);
+void socket_destroy(socket_t* socket, void* data);
+void socket_end(socket_t* socket, void* data);
 char* socket_get_name(socket_t* socket);
 
 int napster_check(gpointer data);
