@@ -209,7 +209,8 @@ GtkWidget *create_dir_browser(gchar * title, gchar * current_path, GtkSelectionM
   gchar *currentdir, *pos, *tpath, *tpathnew;
   gboolean leaf;
  
-  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  win = gtk_window_new (GTK_WINDOW_DIALOG);
+  //  gtk_window_set_modal (GTK_WINDOW (win), TRUE);
   gtk_object_set_data (GTK_OBJECT (win), "win", win);
   gtk_widget_set_usize (win, 230, 320);
   gtk_window_set_title (GTK_WINDOW (win), title);
