@@ -6,6 +6,8 @@ extern const char LineSpeedShort[11][20];
 extern char tstr[10][512];
 extern char* list[10];
 extern char Network[3][20];
+extern char* Browser[BROWSER_NO];
+extern int SocketStatus[S_NUMBER];
 
 extern char **environ;
 
@@ -17,20 +19,34 @@ extern char **environ;
 #define BSWAP32(x) (x)
 #endif
 
+#ifndef HAVE_STRCASECMP
+#define strcasecmp g_strcasecmp
+#endif
+
 extern global_t global;
 
 int ColorTable(int i1, int i2);
 int ColorTable2(int index);
 
+char* browser_to_string(int browser);
+int browser_to_int(char* browser);
+void browse_url(char* url);
+int version_is_up_to_date(char* version);
+int verify_speed(int speed);
+int get_beats(time_t tim);
+int guess_mime(char* suffix);
+int find_valid_bitrate(int bitrate);
+char* current_time(char* stime, int with_secs);
 char* ntoa (unsigned int ip);
 int directory_exists(char* dir);
 void create_dir(char* dir);
 char* MimeNames(int no);
 char* MimeNames_(int no);
 char* LineSpeed(int no);
+char* UserStatus(int status);
 
-void update_user_stats();
-char* strcasestr(char* str, const char* sub);
+void update_user_stats(int highlight);
+char* strcasestr(char* str, char* sub);
 
 int speed2int(char* linespeed);
 int level2int(char* level);
@@ -48,16 +64,18 @@ void write_rc();
 void setup_sensitive(int section);
 void setup_commands();
 void create_pixmaps();
+void setup_main_tab();
 void setup_widgets();
 void setup_buttons();
 void setup_mime();
+void setup_global_vars();
 
 void global_init();
 void global_exit();
 
 char* extract_short_name(char* longname);
 char* convert_to_win(char* name);
-char* convert_to_unix(char* name);
+char* convert_to_unix(char* name, int change);
 
 void destroy_file_row(gpointer data);
 
@@ -66,10 +84,25 @@ void destroy_file_row(gpointer data);
 gint channel_compare (GtkCList      *clist,
 		      gconstpointer  ptr1,
 		      gconstpointer  ptr2);
+gint transfer_compare (GtkCList      *clist,
+		       gconstpointer  ptr1,
+		       gconstpointer  ptr2);
+gint browse_compare (GtkCList      *clist,
+		     gconstpointer  ptr1,
+		     gconstpointer  ptr2);
+gint hotlist_compare (GtkCList      *clist,
+		      gconstpointer  ptr1,
+		      gconstpointer  ptr2);
+gint access_compare (GtkCList      *clist,
+		     gconstpointer  ptr1,
+		     gconstpointer  ptr2);
 gint
 client_compare (GtkCList      *clist,
 		gconstpointer  ptr1,
 		gconstpointer  ptr2);
+gint resume_compare (GtkCList      *clist,
+		     gconstpointer  ptr1,
+		     gconstpointer  ptr2);
 
 void send_search_request();
 void cmd_eval_whois(char* data);
@@ -92,6 +125,9 @@ void set_button_state(GtkWidget* button, int open);
 int toggle_button_state(GtkWidget* button);
 void set_up_button(GtkWidget* button, 
 		   char* pix1, char* pix2, char *wid);
+void set_up_button2(GtkWidget* button, 
+		    GtkWidget* pix1, GtkWidget* pix2, 
+		    GtkWidget* wid);
 void show_bans(char* channel);
 void cmd_banlist_entry(char* data);
 char* cparse(char *input);
@@ -122,7 +158,11 @@ int get_mimetype(char* name);
 suffix_t* get_suffix(char* name);
 GtkWidget* create_mode_popup (void);
 GtkWidget* create_user_popup(int mode);
-void print_speed(char* str, long bytes, int withdot);
+char* print_speed(char* str, long bytes, int withdot);
+char* print_time_average(char* str, int secs);
+char* print_time(char* str, int secs);
+char* print_time_short(char* str, int secs);
+char* print_size(char* str, double bytes);
 void draw_band_width(bandwidth_t* bandwidth, int full);
 
 void extra_win_get_geometry();
@@ -134,10 +174,18 @@ int get_version(char* data);
 void get_server(char* data);
 void clones_clear();
 
-GList* create_ports_from_string(char* string);
-char* create_string_from_ports(GList* ports);
+GList* create_allowed_ports(char* string);
+int get_next_port(GList* list, int last);
+char* create_string_from_ports(GList* list);
+void destroy_allowed_ports(GList* list);
+int port_is_allowed(GList* list, int port);
 
 void gtk_my_widget_show(GtkWidget* widget);
 void gtk_my_widget_show_all();
 
+void switch_to_page(int no);
+int notebook_page_visible(int no);
+
+void print_topright_corner();
+
 #endif
