@@ -164,6 +164,8 @@
 #define CMD_CLIENT_PING_ALL_SERVERS     10120
 #define CMD_CLIENT_WHO_WAS              10121
 #define CMD_CLIENT_MASS_KILL            10122
+#define CMD_CLIENT_HISTOGRAM		10123
+#define CMD_SERVER_HISTOGRAM		10124
 #define CMD_CLIENT_REGISTER_USER	10200
 #define CMD_CLIENT_CHANNEL_LEVEL	10201	/* deprecated, use 823 instead*/
 #define CMD_CLIENT_KICK_USER		10202	/* deprecated, use 829 instead*/
@@ -178,6 +180,18 @@
 #define CMD_CLIENT_CHANNEL_UNVOICE	10212
 #define CMD_CLIENT_CHANNEL_MUZZLE	10213
 #define CMD_CLIENT_CHANNEL_UNMUZZLE	10214
+#define CMD_CLIENT_CLASS_ADD            10250
+#define CMD_CLIENT_CLASS_DEL            10251
+#define CMD_CLIENT_CLASS_LIST           10252
+#define CMD_CLIENT_DLINE_ADD            10253
+#define CMD_CLIENT_DLINE_DEL            10254
+#define CMD_CLIENT_DLINE_LIST           10255
+#define CMD_CLIENT_ILINE_ADD            10256
+#define CMD_CLIENT_ILINE_DEL            10257
+#define CMD_CLIENT_ILINE_LIST           10258
+#define CMD_CLIENT_ELINE_ADD            10259
+#define CMD_CLIENT_ELINE_DEL            10260
+#define CMD_CLIENT_ELINE_LIST           10261
 #define CMD_CLIENT_SHARE_FILE		10300	/* generic media type */
 #define CMD_CLIENT_BROWSE_NEW		10301
 #define CMD_SERVER_BROWSE_RESULT_NEW	10302
@@ -259,3 +273,8 @@ HANDLER(cmd_usage_stats);
 HANDLER(cmd_version_stats);
 HANDLER(cmd_user_mode);
 HANDLER(cmd_browse_new);
+HANDLER(cmd_redirect);
+HANDLER(cmd_cycle);
+HANDLER(cmd_histogram);
+HANDLER(cmd_histogram_end);
+HANDLER(cmd_acl_list);
