@@ -46,7 +46,6 @@ void set_options_channels() {
 }
 
 void setup_preferences(unsigned long flags) {
-  
   GtkWidget* win;
   GtkWidget* temp;
   GList* dlist;
@@ -71,57 +70,73 @@ void setup_preferences(unsigned long flags) {
 				 NULL, NULL, NULL, NULL,
 				 FALSE, TRUE);
     gtk_ctree_node_set_row_data(ctree, node, (void*)1);
-    strcpy(tstr[0], _("Transfers"));
+    strcpy(tstr[0], _("Search"));
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  TRUE, FALSE);
     gtk_ctree_node_set_row_data(ctree, node2, (void*)2);
-    strcpy(tstr[0], _("Mimetypes"));
+    strcpy(tstr[0], _("Transfers"));
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  TRUE, FALSE);
     gtk_ctree_node_set_row_data(ctree, node2, (void*)3);
+    strcpy(tstr[0], _("Mimetypes"));
+    node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
+				  NULL, NULL, NULL, NULL,
+				  TRUE, FALSE);
+    gtk_ctree_node_set_row_data(ctree, node2, (void*)4);
     strcpy(tstr[0], _("Paths"));
     node = gtk_ctree_insert_node(ctree, NULL, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  FALSE, TRUE);
-    gtk_ctree_node_set_row_data(ctree, node, (void*)4);
+    gtk_ctree_node_set_row_data(ctree, node, (void*)5);
     strcpy(tstr[0], _("Download"));
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void*)5);
+    gtk_ctree_node_set_row_data(ctree, node2, (void*)6);
     strcpy(tstr[0], _("Shares"));
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void*)6);
+    gtk_ctree_node_set_row_data(ctree, node2, (void*)7);
     strcpy(tstr[0], _("Appearance"));
     node = gtk_ctree_insert_node(ctree, NULL, NULL, list, 15,
 				 NULL, NULL, NULL, NULL,
 				 FALSE, TRUE);
-    gtk_ctree_node_set_row_data(ctree, node, (void*)7);
+    gtk_ctree_node_set_row_data(ctree, node, (void*)8);
+    strcpy(tstr[0], _("Chat"));
+    node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
+				  NULL, NULL, NULL, NULL,
+				  TRUE, FALSE);
+    gtk_ctree_node_set_row_data(ctree, node2, (void*)9);
     strcpy(tstr[0], _("Schemes"));
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void*)8);
+    gtk_ctree_node_set_row_data(ctree, node2, (void*)10);
     strcpy(tstr[0], _("Advanced"));
     node = gtk_ctree_insert_node(ctree, NULL, NULL, list, 15,
 				 NULL, NULL, NULL, NULL,
 				 FALSE, TRUE);
-    gtk_ctree_node_set_row_data(ctree, node, (void*)9);
+    gtk_ctree_node_set_row_data(ctree, node, (void*)11);
     strcpy(tstr[0], _("Aliases"));
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void*)10);
+    gtk_ctree_node_set_row_data(ctree, node2, (void*)12);
     strcpy(tstr[0], _("Channels"));
     node2 = gtk_ctree_insert_node(ctree, node, NULL, list, 15,
 				  NULL, NULL, NULL, NULL,
 				  TRUE, FALSE);
-    gtk_ctree_node_set_row_data(ctree, node2, (void*)11);
+    gtk_ctree_node_set_row_data(ctree, node2, (void*)13);
 
+    temp = lookup_widget(win, "frame266");
+    gtk_widget_set_style(temp, global.style[3]);
+    temp = lookup_widget(win, "frame267");
+    gtk_widget_set_style(temp, global.style[3]);
+    temp = lookup_widget(win, "frame268");
+    gtk_widget_set_style(temp, global.style[3]);
     temp = lookup_widget(win, "frame69");
     gtk_widget_set_style(temp, global.style[3]);
     temp = lookup_widget(win, "frame37");
@@ -164,7 +179,18 @@ void setup_preferences(unsigned long flags) {
     gtk_widget_set_style(temp, global.style[3]);
     temp = lookup_widget(win, "frame240");
     gtk_widget_set_style(temp, global.style[3]);
-
+    temp = lookup_widget(win, "label780");
+    gtk_widget_set_style(temp, global.style[STYLE_PREF]);
+    temp = lookup_widget(win, "frame306");
+    gtk_widget_set_style(temp, global.style[3]);
+    temp = lookup_widget(win, "frame307");
+    gtk_widget_set_style(temp, global.style[3]);
+    temp = lookup_widget(win, "frame308");
+    gtk_widget_set_style(temp, global.style[3]);
+    temp = lookup_widget(win, "frame318");
+    gtk_widget_set_style(temp, global.style[3]);
+    temp = lookup_widget(win, "frame321");
+    gtk_widget_set_style(temp, global.style[3]);
   }
   if (flags & P_GENERAL) {
     temp = lookup_widget(win, "entry4");
@@ -184,7 +210,46 @@ void setup_preferences(unsigned long flags) {
     temp = lookup_widget(win, "checkbutton4");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.network.firewall);
     temp = lookup_widget(win, "entry83");
-    gtk_entry_set_text(GTK_ENTRY(temp), global.allowed_ports_str);
+    pos = create_string_from_ports(global.allowed_ports);
+    if (pos) gtk_entry_set_text(GTK_ENTRY(temp), pos);
+    g_free(pos);
+  }
+
+  if (flags & P_NSEARCH) {
+    temp = lookup_widget(win, "spinbutton50");
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 
+			      (gfloat)global.limit.max_searches);
+  }
+  if (flags & P_RSEARCH) {
+    temp = lookup_widget(win, "spinbutton51");
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), global.resume_timeout);
+    if (global.resume_timer >= 0) {
+      gtk_widget_set_sensitive(temp, TRUE);
+      temp = lookup_widget(win, "checkbutton48");
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
+    } else {
+      gtk_widget_set_sensitive(temp, FALSE);
+      temp = lookup_widget(win, "checkbutton48");
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
+    }
+  }
+  if (flags & P_PINGS) {
+    temp = lookup_widget(global.win, "checkbutton45");
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
+      temp = lookup_widget(win, "checkbutton49");
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
+      temp = lookup_widget(win, "entry85");
+      if (global.ping_command)
+	gtk_entry_set_text(GTK_ENTRY(temp), global.ping_command);
+      gtk_widget_set_sensitive(temp, TRUE);
+    } else {
+      temp = lookup_widget(win, "checkbutton49");
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
+      temp = lookup_widget(win, "entry85");
+      if (global.ping_command)
+	gtk_entry_set_text(GTK_ENTRY(temp), global.ping_command);
+      gtk_widget_set_sensitive(temp, FALSE);
+    }
   }
 
   if (flags & P_D_LIMIT) {
@@ -210,6 +275,8 @@ void setup_preferences(unsigned long flags) {
   if (flags & P_REST1) {
     temp = lookup_widget(win, "spinbutton40");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), (gfloat)(global.network.transfer_timeout));
+    temp = lookup_widget(win, "checkbutton62");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.confirm_delete);
   }
 
   if (flags & P_MIMETYPES) {
@@ -223,6 +290,8 @@ void setup_preferences(unsigned long flags) {
 	strcpy(tstr[0], suffix->suffix);
 	strcpy(tstr[1], MimeNames(i1));
 	strcpy(tstr[2], suffix->application);
+	if (suffix->as_mp3) strcpy(tstr[3], _("Yes"));
+	else strcpy(tstr[3], _("No"));
 	gtk_clist_append(GTK_CLIST(temp), list);
       }
     }
@@ -253,7 +322,7 @@ void setup_preferences(unsigned long flags) {
     GList* dlist;
 
     temp = lookup_widget(win, "clist19");
-    for (i1 = 1; i1 < MIME_SIZE; i1++) {
+    for (i1 = 0; i1 < MIME_SIZE; i1++) {
       for (dlist = global.mimetype[i1].shared; dlist; dlist = dlist->next) {
 	strcpy(tstr[0], MimeNames(i1));
 	strcpy(tstr[1], dlist->data);
@@ -281,20 +350,67 @@ void setup_preferences(unsigned long flags) {
     temp = lookup_widget(win, "checkbutton12");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.public_ignore);
   }
-  if (flags & P_SHOW_JOIN) {
-    temp = lookup_widget(win, "checkbutton11");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.show_joins);
-  }
-  if (flags & P_SEND_IGNORE) {
-    temp = lookup_widget(win, "checkbutton15");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.send_ignore);
-  }
   if (flags & P_PARSE_COLOR) {
     temp = lookup_widget(win, "checkbutton22");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.parse_color);
   }
-  if (flags & P_REST2) {
-    // nothing to do.
+  if (flags & P_TAB_STYLE) {
+    temp = lookup_widget(win, "checkbutton57");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), 
+				 global.options.tab_style&TAB_ICON);
+    
+    temp = lookup_widget(win, "checkbutton58");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), 
+				 global.options.tab_style&TAB_TEXT);
+    
+  }
+  if (flags & P_SHOW_JOIN) {
+    temp = lookup_widget(win, "combo_entry25");
+    switch (global.options.show_parts) {
+    case 0:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("None"));
+      break;
+    case 1:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("Recent Users"));
+      break;
+    case 2:
+    default:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("All"));
+      break;
+    }
+
+    temp = lookup_widget(win, "spinbutton56");
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), global.options.recent_timeout);
+
+    temp = lookup_widget(win, "combo_entry24");
+    switch (global.options.show_joins) {
+    case 0:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("None"));
+      break;
+    case 1:
+    default:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("All"));
+      break;
+    }
+  }
+
+  if (flags & P_TIMESTAMPS) {
+    temp = lookup_widget(win, "entry108");
+    switch (global.options.timestamps) {
+    case TIME_NONE:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("None"));
+      break;
+    case TIME_BEATTIME:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("BeatTime"));
+      break;
+    case TIME_HOUR_MIN:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("Hour:Minute"));
+      break;
+    case TIME_HOUR_MIN_SEC:
+    default:
+      gtk_entry_set_text(GTK_ENTRY(temp), _("Hour:Minute:Second"));
+      break;
+    }
   }
 
   if (flags & P_SCHEMES) {
@@ -305,12 +421,17 @@ void setup_preferences(unsigned long flags) {
   }
 
   if (flags & P_ADVANCED) {
+    temp = lookup_widget(win, "checkbutton65");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.check_version);
     temp = lookup_widget(win, "checkbutton13");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.network.auto_connect);
     temp = lookup_widget(win, "checkbutton38");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.network.allow_dcc);
-    temp = lookup_widget(win, "checkbutton37");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.sep_chwallop);
+    temp = lookup_widget(win, "checkbutton54");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.dummy_md5);
+
+    temp = lookup_widget(win, "button312");
+    gtk_object_set_data(GTK_OBJECT(temp), "value", (void*)global.options.piping);
 
     temp = lookup_widget(win, "checkbutton33");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.network.auto_reconnect);
@@ -330,6 +451,10 @@ void setup_preferences(unsigned long flags) {
     default: temp = lookup_widget(win, "radiobutton1"); break;
     }
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
+
+    temp = lookup_widget(win, "combo_entry27");
+    gtk_entry_set_text(GTK_ENTRY(temp), browser_to_string(global.browser));
+
     temp = lookup_widget(win, "spinbutton41");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), (gfloat)global.network.transfer_delay);
   }
@@ -350,7 +475,7 @@ void setup_preferences(unsigned long flags) {
     }
   }
 
-  if (flags & P_CHANNELS) {
+  if (flags & P_CHANNEL) {
     temp = lookup_widget(win, "clist6");
     if (global.channels) {
       for (row = 0; row < g_list_length(global.channels); row++) {
@@ -360,6 +485,10 @@ void setup_preferences(unsigned long flags) {
     }
     set_options_channels();
   }  
+  if (flags & P_REJOIN) {
+    temp = lookup_widget(win, "checkbutton60");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), global.options.channel_rejoin);
+  }
 }
 
 int search_shared_folder(char* folder, int mime) {
@@ -424,6 +553,7 @@ void apply_preferences() {
   int i1;
   int i2;
   char* pos;
+  char* pos2;
   int do_refresh;
   scheme_t* scheme;
 
@@ -452,13 +582,13 @@ void apply_preferences() {
 
   temp = lookup_widget(win, "entry83");
   pos = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
-  g_list_free(global.allowed_ports);
-  global.allowed_ports = create_ports_from_string(pos);
+  destroy_allowed_ports(global.allowed_ports);
+  global.allowed_ports = create_allowed_ports(pos);
   free(pos);
   pos = create_string_from_ports(global.allowed_ports);
-  gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  global.allowed_ports_str = strdup(pos);
-  
+  if (pos) gtk_entry_set_text(GTK_ENTRY(temp), pos);
+  g_free(pos);
+
   temp = lookup_widget(win, "spinbutton2");
   i1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
   temp = lookup_widget(win, "checkbutton4");
@@ -469,6 +599,44 @@ void apply_preferences() {
     create_upload_port(i1, TRUE);
   }
 
+  // search
+  temp = lookup_widget(win, "spinbutton50");
+  global.limit.max_searches = 
+    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
+  temp = lookup_widget(global.win, "spinbutton42");
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
+			    global.limit.max_searches);
+
+  temp = lookup_widget(win, "spinbutton51");
+  global.resume_timeout = 
+    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
+  temp = lookup_widget(win, "checkbutton48");
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
+    temp = lookup_widget(global.win, "checkbutton46");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
+  } else {
+    temp = lookup_widget(global.win, "checkbutton46");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
+  }
+  temp = lookup_widget(global.win, "spinbutton48");
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
+			    global.resume_timeout);
+  
+  temp = lookup_widget(win, "entry85");
+  pos = gtk_entry_get_text(GTK_ENTRY(temp));
+  free(global.ping_command);
+  global.ping_command = strdup(pos);
+
+  temp = lookup_widget(win, "checkbutton49");
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
+    temp = lookup_widget(global.win, "checkbutton45");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
+  } else {
+    temp = lookup_widget(global.win, "checkbutton45");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
+  }
+  
+
   // Transfers
   temp = lookup_widget(win, "spinbutton38");
   global.limit.max_downloads = 
@@ -508,6 +676,11 @@ void apply_preferences() {
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
 			       global.network.auto_resume);
 
+  temp = lookup_widget(win, "checkbutton62");
+  global.options.confirm_delete =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+
+
   // mimetypes
   clear_suffixes();
   temp = lookup_widget(win, "clist17");
@@ -515,15 +688,20 @@ void apply_preferences() {
     int mime;
     char* suf;
     char* app;
+    char* as_mp3;
     suffix_t* suffix;
 
     gtk_clist_get_text (GTK_CLIST(temp), i1, 1, &pos);
     mime = mime2int(pos);
     gtk_clist_get_text (GTK_CLIST(temp), i1, 0, &suf);
     gtk_clist_get_text (GTK_CLIST(temp), i1, 2, &app);
+    gtk_clist_get_text (GTK_CLIST(temp), i1, 3, &as_mp3);
     suffix = (suffix_t*)malloc(sizeof(suffix_t));
     suffix->suffix = strdup(suf);
     suffix->application = strdup(app);
+    if (!strcasecmp(as_mp3, _("Yes"))) suffix->as_mp3 = 1;
+    else suffix->as_mp3 = 0;
+    if (mime == MIME_MP3) suffix->as_mp3 = 1;
     global.mimetype[mime].suffixes = 
       g_list_append(global.mimetype[mime].suffixes, suffix);
   }
@@ -573,16 +751,15 @@ void apply_preferences() {
   }
 
   // Appearance
+  temp = lookup_widget(global.options_win, "button312");
+  global.options.piping = (int)gtk_object_get_data(GTK_OBJECT(temp), "value");
+
   temp = lookup_widget(win, "entry42");
   pos = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
   make_list_from_string(&global.highlight, pos, " ");
   free(pos);
   pos = make_string_from_list(global.highlight, " ");
   gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  temp = lookup_widget(global.win, "entry45");
-  gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  temp = lookup_widget(global.win, "button122");
-  gtk_widget_set_sensitive(temp, FALSE);
 
   temp = lookup_widget(win, "entry44");
   pos = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
@@ -590,10 +767,6 @@ void apply_preferences() {
   free(pos);
   pos = make_string_from_list(global.frienduser, " ");
   gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  temp = lookup_widget(global.win, "entry46");
-  gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  temp = lookup_widget(global.win, "button123");
-  gtk_widget_set_sensitive(temp, FALSE);
 
   temp = lookup_widget(win, "entry52");
   pos = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
@@ -601,18 +774,23 @@ void apply_preferences() {
   free(pos);
   pos = make_string_from_list(global.enemyuser, " ");
   gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  temp = lookup_widget(global.win, "entry51");
-  gtk_entry_set_text(GTK_ENTRY(temp), pos);
-  temp = lookup_widget(global.win, "button124");
-  gtk_widget_set_sensitive(temp, FALSE);
 
-  temp = lookup_widget(win, "checkbutton11");
-  global.options.show_joins =
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
-  temp = lookup_widget(global.win, "checkbutton7");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
-			       global.options.show_joins);
-  
+  temp = lookup_widget(win, "combo_entry24");
+  pos = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  temp = lookup_widget(win, "combo_entry25");
+  pos2 = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  temp = lookup_widget(win, "spinbutton56");
+  i1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
+
+  temp = lookup_widget(global.win, "combo_entry23");
+  gtk_entry_set_text(GTK_ENTRY(temp), pos);
+  free(pos);
+  temp = lookup_widget(global.win, "combo_entry22");
+  gtk_entry_set_text(GTK_ENTRY(temp), pos2);
+  free(pos2);
+  temp = lookup_widget(global.win, "spinbutton55");
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), i1);
+
   temp = lookup_widget(win, "checkbutton12");
   global.options.public_ignore =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
@@ -620,20 +798,35 @@ void apply_preferences() {
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
 			       global.options.public_ignore);
 
-  temp = lookup_widget(win, "checkbutton15");
-  global.options.send_ignore =
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
-  temp = lookup_widget(global.win, "checkbutton14");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
-			       global.options.send_ignore);
-
   temp = lookup_widget(win, "checkbutton22");
   global.options.parse_color =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
   temp = lookup_widget(global.win, "checkbutton21");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
 			       global.options.parse_color);
+  temp = lookup_widget(win, "checkbutton57");
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp)))
+    global.options.tab_style |= TAB_ICON;
+  else global.options.tab_style &= (0x03^TAB_ICON);
+  temp = lookup_widget(win, "checkbutton58");
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp)))
+    global.options.tab_style |= TAB_TEXT;
+  else global.options.tab_style &= (0x03^TAB_TEXT);
 
+  temp = lookup_widget(win, "checkbutton60");
+  global.options.channel_rejoin =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
+  temp = lookup_widget(global.win, "checkbutton59");
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
+			       global.options.channel_rejoin);
+
+  temp = lookup_widget(win, "entry108");
+  pos = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
+  temp = lookup_widget(global.win, "combo_entry26");
+  gtk_entry_set_text(GTK_ENTRY(temp), pos);
+  free(pos);
+
+  setup_main_tab();
 
   // saving color scheme
   //  on_button100_clicked(NULL, NULL);
@@ -652,6 +845,14 @@ void apply_preferences() {
   }
 
   // Advanced
+  temp = lookup_widget(win, "checkbutton65");
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
+    if (!global.options.check_version)
+      global.options.check_version = 1;
+  } else {
+    global.options.check_version = 0;
+  }
+
   temp = lookup_widget(win, "checkbutton13");
   global.network.auto_connect =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
@@ -660,8 +861,8 @@ void apply_preferences() {
   global.network.allow_dcc =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
 
-  temp = lookup_widget(win, "checkbutton37");
-  global.options.sep_chwallop =
+  temp = lookup_widget(win, "checkbutton54");
+  global.options.dummy_md5 =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));
 
   temp = lookup_widget(win, "checkbutton33");
@@ -684,6 +885,10 @@ void apply_preferences() {
   global.network.transfer_delay = 
     gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
   
+  temp = lookup_widget(win, "combo_entry27");
+  pos = gtk_entry_get_text(GTK_ENTRY(temp));
+  global.browser = browser_to_int(pos);
+
   temp = lookup_widget(win, "radiobutton1");
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp)))
     global.options.popup_create = 1;
@@ -710,3 +915,85 @@ void apply_preferences() {
 
   if (do_refresh) lib_refresh();
 }
+
+void preferences_goto(int section) {
+  GtkCList* clist;
+
+  if (!global.options_win) {
+    on_preferences_activate(NULL, NULL);
+  }
+  if (!global.options_win) return;
+
+  clist = GTK_CLIST(lookup_widget(global.options_win, "ctree2"));
+  gtk_clist_select_row(clist, section, 0);
+}
+
+void on_toggle_pipe(GtkMenuItem     *menuitem,
+		    gpointer         user_data) {
+  int value;
+  GtkWidget* temp;
+  int no = (int)user_data;
+
+  temp = lookup_widget(global.options_win, "button312");
+  value = (int)gtk_object_get_data(GTK_OBJECT(temp), "value");
+  value = value ^ no;
+  gtk_object_set_data(GTK_OBJECT(temp), "value", (gpointer)value);
+}
+
+GtkWidget* create_pipe_popup (int val) {
+  GtkWidget *mode_popup;
+  GtkAccelGroup *mode_popup_accels;
+  GtkWidget *mode;
+
+  mode_popup = gtk_menu_new ();
+  gtk_object_set_data (GTK_OBJECT (mode_popup), "mode_popup", mode_popup);
+  mode_popup_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (mode_popup));
+
+  mode = gtk_check_menu_item_new_with_label (_("Away Logging"));
+  gtk_widget_show (mode);
+  gtk_container_add (GTK_CONTAINER (mode_popup), mode);
+  if (val & PIPE_AWAY_LOG)
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mode), TRUE);
+  gtk_signal_connect (GTK_OBJECT (mode), "activate",
+		      (GtkSignalFunc)on_toggle_pipe,
+		      (gpointer)PIPE_AWAY_LOG);
+
+  mode = gtk_check_menu_item_new_with_label (_("Channel Operator Message"));
+  gtk_widget_show (mode);
+  gtk_container_add (GTK_CONTAINER (mode_popup), mode);
+  if (val & PIPE_CHWALLOP)
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mode), TRUE);
+  gtk_signal_connect (GTK_OBJECT (mode), "activate",
+		      (GtkSignalFunc)on_toggle_pipe,
+		      (gpointer)PIPE_CHWALLOP);
+
+  mode = gtk_check_menu_item_new_with_label (_("Operator Messages"));
+  gtk_widget_show (mode);
+  gtk_container_add (GTK_CONTAINER (mode_popup), mode);
+  if (val & PIPE_WALLOP)
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mode), TRUE);
+  gtk_signal_connect (GTK_OBJECT (mode), "activate",
+		      (GtkSignalFunc)on_toggle_pipe,
+		      (gpointer)PIPE_WALLOP);
+
+  mode = gtk_check_menu_item_new_with_label (_("Global Messages"));
+  gtk_widget_show (mode);
+  gtk_container_add (GTK_CONTAINER (mode_popup), mode);
+  if (val & PIPE_GLOBAL)
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mode), TRUE);
+  gtk_signal_connect (GTK_OBJECT (mode), "activate",
+		      (GtkSignalFunc)on_toggle_pipe,
+		      (gpointer)PIPE_GLOBAL);
+
+  mode = gtk_check_menu_item_new_with_label (_("Server Messages"));
+  gtk_widget_show (mode);
+  gtk_container_add (GTK_CONTAINER (mode_popup), mode);
+  if (val & PIPE_SERVER)
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mode), TRUE);
+  gtk_signal_connect (GTK_OBJECT (mode), "activate",
+		      (GtkSignalFunc)on_toggle_pipe,
+		      (gpointer)PIPE_SERVER);
+
+  return mode_popup;
+}
+
