@@ -15,7 +15,7 @@
 #include "support.h"
 #include "chat.h"
 
-scheme_t* scheme_new() {
+scheme_t* scheme_new(void) {
   scheme_t* scheme;
 
   scheme = (scheme_t*)malloc(sizeof(scheme_t));
@@ -27,7 +27,7 @@ scheme_t* scheme_new() {
   return scheme;
 }
 
-scheme_t* scheme_create_default() {
+scheme_t* scheme_create_default(void) {
   scheme_t* scheme = NULL;
 
   scheme = scheme_new();
@@ -54,6 +54,16 @@ scheme_t* scheme_create_default() {
 				 style_new("message", "#b2ccff", "NULL", "-b&h-lucidatypewriter-medium-r-normal-*-*-120-*-*-m-*-iso8859-1"));
   scheme->colors = g_list_append(scheme->colors, 
 				 style_new("highlight", "#ded305", "NULL", "-b&h-lucidatypewriter-bold-r-normal-*-*-120-*-*-m-*-iso8859-1"));
+  scheme->colors = g_list_append(scheme->colors, 
+				 style_new("transfer_running", "#ff0011", "#ffd484", "NULL"));
+  scheme->colors = g_list_append(scheme->colors, 
+				 style_new("transfer_waiting", "#002173", "#94bfdd", "NULL"));
+  scheme->colors = g_list_append(scheme->colors, 
+				 style_new("transfer_bar1", "#ff0300", "#ffa393", "NULL"));
+  scheme->colors = g_list_append(scheme->colors, 
+				 style_new("transfer_bar2", "#1700ff", "#77aee5", "NULL"));
+  scheme->colors = g_list_append(scheme->colors, 
+				 style_new("library_unshared", "#000000", "#eeeeee", "NULL"));
 
   scheme->client_prefix = strdup("%K=%c=%C= ");
   scheme->server_prefix = strdup("%K%r%R ");
@@ -182,7 +192,27 @@ scheme_t* scheme_load(char* name) {
     scheme->client_prefix = strdup("%K=%c=%C= ");
   if (!scheme->server_prefix)
     scheme->server_prefix = strdup("%K%r%R ");
-  
+
+  if (!style_search(scheme, "transfer_running")) {
+    scheme->colors = g_list_append(scheme->colors, 
+				   style_new("transfer_running", "#ff0011", "#ffd484", "NULL"));
+  }
+  if (!style_search(scheme, "transfer_waiting")) {
+    scheme->colors = g_list_append(scheme->colors, 
+				   style_new("transfer_waiting", "#002173", "#94bfdd", "NULL"));
+  }
+  if (!style_search(scheme, "transfer_bar1")) {
+    scheme->colors = g_list_append(scheme->colors, 
+				   style_new("transfer_bar1", "#ff0300", "#ffa393", "NULL"));
+  }
+  if (!style_search(scheme, "transfer_bar2")) {
+    scheme->colors = g_list_append(scheme->colors, 
+				   style_new("transfer_bar2", "#1700ff", "#77aee5", "NULL"));
+  }
+  if (!style_search(scheme, "library_unshared")) {
+    scheme->colors = g_list_append(scheme->colors, 
+				   style_new("library_unshared", "#000000", "#eeeeee", "NULL"));
+  }
   return scheme;
 }
 
@@ -355,7 +385,7 @@ void scheme_load_global(char* scheme, int force) {
 
   global.scheme = scheme_load("default");
   if (!global.scheme) {
-    global.scheme = scheme_create_default("default");
+    global.scheme = scheme_create_default();
     client_message(_("Message"), _("Scheme [%s] created!"), 
 		   global.scheme->name);
   } else {
