@@ -1,5 +1,5 @@
-scheme_t* scheme_new();
-scheme_t* scheme_create_default();
+scheme_t* scheme_new(void);
+scheme_t* scheme_create_default(void);
 void scheme_destroy(scheme_t* scheme);
 scheme_t* scheme_load(char* name);
 void scheme_save(scheme_t* scheme);
