@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 #include <ctype.h>
 #include <strings.h>
 #include <string.h>
@@ -12,11 +15,13 @@
 #include "global.h"
 #include "search.h"
 #include "share.h"
-#include "hotlist.h"
+#include "browse.h"
 #include "handler.h"
 #include "support.h"
 #include "transfer.h"
 #include "resume.h"
+#include "whois.h"
+#include "ping.h"
 
 typedef int (*IntFunc)(char* str);
 
@@ -126,11 +131,12 @@ void search_create_page(search_t* search) {
 
   search->link = (void*)scrolledwindow;
   
-  search_list = gtk_clist_new (6);
+  search_list = gtk_clist_new (7);
   gtk_widget_show (search_list);
+  gtk_widget_set_events (search_list, GDK_POINTER_MOTION_MASK | GDK_LEAVE_NOTIFY_MASK);
 
   gtk_clist_set_sort_column(GTK_CLIST(search_list), 0);
-  gtk_clist_set_auto_sort(GTK_CLIST(search_list), TRUE);
+  //  gtk_clist_set_auto_sort(GTK_CLIST(search_list), TRUE);
   gtk_clist_set_compare_func(GTK_CLIST(search_list), search_compare);
 
   gtk_container_add (GTK_CONTAINER (scrolledwindow), search_list);
@@ -185,12 +191,25 @@ void search_create_page(search_t* search) {
   gtk_widget_show (label);
   gtk_clist_set_column_widget (GTK_CLIST (search_list), 5, label);
 
+  label = gtk_label_new (_("Ping"));
+  gtk_widget_show (label);
+  gtk_clist_set_column_widget (GTK_CLIST (search_list), 6, label);
+
   gtk_signal_connect (GTK_OBJECT (search_list), "click_column",
                       GTK_SIGNAL_FUNC (on_list_click_column),
                       NULL);
   gtk_signal_connect (GTK_OBJECT (search_list), "button_press_event",
                       GTK_SIGNAL_FUNC (on_search_list_button_press_event),
                       NULL);
+  gtk_signal_connect (GTK_OBJECT (search_list), "select_row",
+                      GTK_SIGNAL_FUNC (on_search_select_row),
+                      NULL);
+  gtk_signal_connect (GTK_OBJECT (search_list), "motion_notify_event",
+                      GTK_SIGNAL_FUNC (on_search_motion_notify_event),
+                      NULL);
+  gtk_signal_connect (GTK_OBJECT (search_list), "leave_notify_event",
+                      GTK_SIGNAL_FUNC (on_list_leave_notify_event),
+                      NULL);
 
   str = g_strdup_printf("%s: %s\n%s", 
 			Destination(pattern->destination),
@@ -267,26 +286,20 @@ GtkWidget* create_search_list_popup(file_t* file) {
 			  GTK_SIGNAL_FUNC (on_download_selected_activate),
 			  (void*)1);
     }
-    
+
     trennlinie17 = gtk_menu_item_new ();
     gtk_widget_show (trennlinie17);
     gtk_container_add (GTK_CONTAINER (popup), trennlinie17);
     gtk_widget_set_sensitive (trennlinie17, FALSE);
+  }
 
-    if (!file->local) {
-      item = gtk_menu_item_new_with_label (_("User Menu"));
-      gtk_widget_show (item);
-      gtk_container_add (GTK_CONTAINER (popup), item);
-      
-      user_popup = create_user_popup(M_SEARCH);
-      gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), user_popup);
-
-      trennlinie17 = gtk_menu_item_new ();
-      gtk_widget_show (trennlinie17);
-      gtk_container_add (GTK_CONTAINER (popup), trennlinie17);
-      gtk_widget_set_sensitive (trennlinie17, FALSE);
-    }
-      
+  clear_all = gtk_menu_item_new_with_label (_("Remove Search"));
+  gtk_widget_show (clear_all);
+  gtk_container_add (GTK_CONTAINER (popup), clear_all);
+  gtk_signal_connect (GTK_OBJECT (clear_all), "activate",
+		      GTK_SIGNAL_FUNC (on_clear_all_activate),
+		      NULL);
+  if (file) {    
     sprintf(item_str, _("Remove Selected (%d)"), item_num);
     clear_selected = gtk_menu_item_new_with_label (item_str);
     gtk_widget_show (clear_selected);
@@ -305,13 +318,6 @@ GtkWidget* create_search_list_popup(file_t* file) {
       gtk_widget_set_sensitive(clear_users_files, FALSE);
   }
 
-  clear_all = gtk_menu_item_new_with_label (_("Remove Search"));
-  gtk_widget_show (clear_all);
-  gtk_container_add (GTK_CONTAINER (popup), clear_all);
-  gtk_signal_connect (GTK_OBJECT (clear_all), "activate",
-		      GTK_SIGNAL_FUNC (on_clear_all_activate),
-		      NULL);
-    
   trennlinie13 = gtk_menu_item_new ();
   gtk_widget_show (trennlinie13);
   gtk_container_add (GTK_CONTAINER (popup), trennlinie13);
@@ -323,11 +329,26 @@ GtkWidget* create_search_list_popup(file_t* file) {
   gtk_signal_connect (GTK_OBJECT (clear_all), "activate",
 		      GTK_SIGNAL_FUNC (on_search_again_activate),
 		      NULL);
+  
   trennlinie17 = gtk_menu_item_new ();
   gtk_widget_show (trennlinie17);
   gtk_container_add (GTK_CONTAINER (popup), trennlinie17);
   gtk_widget_set_sensitive (trennlinie17, FALSE);
 
+  if (file && !file->local) {
+    item = gtk_menu_item_new_with_label (_("User Menu"));
+    gtk_widget_show (item);
+    gtk_container_add (GTK_CONTAINER (popup), item);
+    
+    user_popup = create_user_popup(M_SEARCH);
+    gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), user_popup);
+    
+    trennlinie17 = gtk_menu_item_new ();
+    gtk_widget_show (trennlinie17);
+    gtk_container_add (GTK_CONTAINER (popup), trennlinie17);
+    gtk_widget_set_sensitive (trennlinie17, FALSE);
+  }
+
   customize_list3 = gtk_menu_item_new_with_label (_("Customize List"));
   gtk_widget_show (customize_list3);
   gtk_container_add (GTK_CONTAINER (popup), customize_list3);
@@ -373,8 +394,14 @@ gint search_compare (GtkCList      *clist,
     if (file1->linespeed < file2->linespeed) return -1;
     if (file1->linespeed > file2->linespeed) return +1;
     return 0;
+  } else if (clist->sort_column == 6) {
+    if (!file1->ping) return (file2->ping!=0);
+    if (!file2->ping) return -1;
+    if (file1->ping < file2->ping) return -1;
+    if (file1->ping > file2->ping) return 1;
+    return 0;
   } else {
-    return strcasecmp (file1->shortname, file2->shortname);
+    return 0;
   }
 }
 
@@ -388,27 +415,27 @@ file_t* file_create_from_search_response(char* data) {
   new_file->md5 = strdup(arg(NULL, 0));
 
   pos1 = arg(NULL, 0);
-  sscanf(pos1, "%ld", &(new_file->size));
+  new_file->size = strtoul(pos1, NULL, 10);
 
   pos1 = arg(NULL, 0);
-  sscanf(pos1, "%d", &(new_file->bitrate));
+  new_file->bitrate = atoi(pos1);
 
   pos1 = arg(NULL, 0);
-  sscanf(pos1, "%d", &(new_file->frequency));
+  new_file->frequency = atoi(pos1);
 
   pos1 = arg(NULL, 0);
-  sscanf(pos1, "%d", &(new_file->duration));
+  new_file->duration = atoi(pos1);
 
   new_file->user = strdup(arg(NULL, 0));
 
   pos1 = arg(NULL, 0);
-  sscanf(pos1, "%lu", &(new_file->ip));
+  new_file->ip = BSWAP32(strtoul(pos1, NULL, 10));
 
   pos1 = arg(NULL, 0);
-  sscanf(pos1, "%d", &(new_file->linespeed));
+  new_file->linespeed = atoi(pos1);
   
   new_file->longname = strdup(new_file->winname);
-  convert_to_unix(new_file->longname);
+  convert_to_unix(new_file->longname, 1);
   pos1 = extract_short_name(new_file->longname);
   new_file->shortname = strdup(pos1);
   new_file->mime_type = get_mimetype(new_file->shortname);
@@ -416,6 +443,7 @@ file_t* file_create_from_search_response(char* data) {
   new_file->flags = 0;
   new_file->status = FILE_NONE;
   new_file->local = 0;            //remote file
+  new_file->ping = 0;
 
   return new_file;
 }
@@ -497,7 +525,7 @@ void append_field2(search_pattern_t* pattern, char* data, int field) {
       sprintf(str, " %s \"AT LEAST\" \"%ld\"", t, s1);
       strcat(data, str);
     }
-    if (s1 > 0) {
+    if (s2 > 0) {
       sprintf(str, " %s \"AT BEST\" \"%ld\"", t, s2);
       strcat(data, str);
     }
@@ -514,6 +542,8 @@ void napster_search(search_pattern_t* pattern, int local) {
     g_warning("not connected!");
     return;
   }
+  if (strlen(pattern->include) == 0) return;
+
   global.status.searching++;
   search_update_counter();
   setup_sensitive(-1);
@@ -522,7 +552,6 @@ void napster_search(search_pattern_t* pattern, int local) {
   else opennap = 0;
   
   *data = 0;
-  if (strlen(pattern->include) == 0) return;
   if (opennap) {
     sprintf(str, "FILENAME CONTAINS \"%s\"", pattern->include);
     strcat(data, str);
@@ -550,16 +579,17 @@ void napster_search(search_pattern_t* pattern, int local) {
   append_field(pattern, data, 1);
   append_field(pattern, data, 2);
 
-  append_field2(pattern, data, 0);
-  append_field2(pattern, data, 1);
+  if (opennap) {
+    append_field2(pattern, data, 0);
+    append_field2(pattern, data, 1);
+    if (pattern->media_type &&
+	strcmp(pattern->media_type, "mp3")) {
+      sprintf(str, " TYPE %s", pattern->media_type);
+      strcat(data, str);
+    }
   
-  if (pattern->media_type &&
-      strcmp(pattern->media_type, "mp3")) {
-    sprintf(str, " TYPE %s", pattern->media_type);
-    strcat(data, str);
+    if (local) strcat(data, " LOCAL_ONLY");
   }
-  
-  if (local) strcat(data, " LOCAL_ONLY");
   send_command(CMD_CLIENT_SEARCH, data);
 }
 
@@ -600,21 +630,17 @@ void send_search_request(gboolean local) {
   search = search_new();
   sprintf(str, "%d %s", search->id, pattern->include);
   search->pattern = search_pattern_create(str);
+  search->local = local;
   search_create_page(search);
   /////
 
   pattern = search->pattern;
-  if (pattern->destination == DEST_LIBRARY) {
-    lib_search(pattern);
-  } else if (pattern->destination == DEST_HOTLIST) {
-    hotlist_search(pattern);
-  } else if (pattern->destination == DEST_NAPSTER) {
-    napster_search(pattern, local);
-  } else if (pattern->destination == DEST_SEARCH) {
-    search_search(pattern);
-  } else {
-    printf("unknown search destination");
+  if (pattern->destination == DEST_NAPSTER) {
+    search_queue(search);
+    return;
   }
+
+  search_do(search);
 }
 
 void search_insert_file(GtkWidget* child, file_t *file) {
@@ -625,21 +651,34 @@ void search_insert_file(GtkWidget* child, file_t *file) {
   GtkWidget* temp;
   GtkNotebook* notebook;
   search_t* search;
-
+  ping_t* p;
+  
   clist = gtk_object_get_data(GTK_OBJECT(child), "clist");
   
   strcpy(tstr[0], file->shortname);
   sprintf(tstr[1], "%.2f MB", (double)(file->size)/1024/1024);
   sprintf(tstr[2], "%d", file->bitrate);
-  sprintf(tstr[3], "%d:%s%d", file->duration/60, 
-	  (file->duration%60)<10?"0":"", file->duration%60);
+  print_time_short(tstr[3], file->duration);
   strcpy(tstr[4], file->user);
   if (file->local)
     strcpy(tstr[5], _("Local File"));
   else
     strcpy(tstr[5], LineSpeed(file->linespeed));
     
-  strcpy(tstr[6], _("unknown"));
+  temp = lookup_widget(global.win, "checkbutton45");
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
+    if ((p = ping_search(file->ip))) {
+      file->ping = p->time;
+      if (p->pid >= 0) sprintf(tstr[6], "eval..");
+      else if (file->ping) sprintf(tstr[6], "%d ms", file->ping);
+      else sprintf(tstr[6], "n/a");
+    } else {
+      ping(file->ip);
+      strcpy(tstr[6], "eval..");
+    }
+  } else {
+    strcpy(tstr[6], "");
+  }
 
   row = gtk_clist_append(clist, &list[0]);
   gtk_clist_set_row_data (clist, row, (gpointer)file);
@@ -658,25 +697,69 @@ void search_insert_file(GtkWidget* child, file_t *file) {
   }
 }
 
+
+int file_cmp(file_t* file1, file_t* file2) {
+  if (file1->size != file2->size) return 1;
+  if (strcmp(file1->winname, file2->winname)) return 2;
+  if (strcmp(file1->user, file2->user)) return 3;
+  //  if (file1->bitrate != file2->bitrate) return 4;
+  //  if (file1->frequency != file2->frequency) return 5;
+  //  if (file1->duration != file2->duration) return 6;
+
+  return 0;
+}
+
+file_t* search_find_file(search_t* search, file_t* file) {
+  GList* dlist;
+  file_t* file2;
+
+  if (!search) return NULL;
+  if (!file) return NULL;
+
+  for (dlist = search->results; dlist; dlist = dlist->next) {
+    file2 = (file_t*)(dlist->data);
+    if (!file_cmp(file, file2)) return file2;
+  }
+
+  return NULL;
+}
+
 void file_insert_search(file_t *file, int destination) {
   search_t* search;
   GList* dlist;
   int cnt = 0;
   file_t* file2;
+  resume_t* resume;
 
   for (dlist = global.searches; dlist; dlist = dlist->next) {
     search = (search_t*)(dlist->data);
-    if ((search->pattern->destination == destination) &&
-	search_pattern_fits_file(search->pattern, file, 0)) {
-      cnt++;
-      file2 = file_dup(file);
-      search->results = g_list_append(search->results, file2);
-      if (search->resume) 
-	resume_insert_file((resume_t*)(search->link), file2);
-      else 
-	search_insert_file(GTK_WIDGET(search->link), file2);
+    if (search->queued) continue;
+    if (!search->link) continue;
+    if (search->pattern->destination != destination) continue;
+    
+    if (search->resume) {
+      resume = (resume_t*)(search->link);
+      if (search_pattern_fits_file(search->pattern, file, 0) &&
+	  (resume->size == file->size)) {
+	cnt++;
+	if (!search_find_file(search, file)) {
+	  file2 = file_dup(file);
+	  search->results = g_list_append(search->results, file2);
+	  resume_insert_file(resume, file2);
+	}
+      }
+    } else {
+      if (search_pattern_fits_file(search->pattern, file, 0)) {
+	cnt++;
+	if (!search_find_file(search, file)) {
+	  file2 = file_dup(file);
+	  search->results = g_list_append(search->results, file2);
+	  search_insert_file(GTK_WIDGET(search->link), file2);
+	}
+      }
     }
   }
+  
 #ifdef SEARCH_DEBUG
   if (cnt == 0) {
     printf("could not find search for [%s]\n", file->longname);
@@ -688,7 +771,7 @@ void search_pattern_load() {
   FILE* file;
   char filename[1024];
   char line[500];
-  search_pattern_t* search;
+  search_pattern_t* search = NULL;
   char* pos;
   char* pos2;
 
@@ -707,10 +790,14 @@ void search_pattern_load() {
     } else if (!strncasecmp(line, "Pattern", 7)) {
       pos = strchr(line, '[')+1;
       pos2 = strrchr(line, ']');
-      pos2[0] = 0;
-      search = search_pattern_new(pos);
-      global.search_pattern = 
-	g_list_append(global.search_pattern, search);
+      if (!pos || !pos2) {
+	g_warning("corrupted searches.save");
+      } else {
+	pos2[0] = 0;
+	search = search_pattern_new(pos);
+	global.search_pattern = 
+	  g_list_append(global.search_pattern, search);
+      }
     } else if (search) {
       if (!strncasecmp(line, "contains", 8)) {
 	pos = strchr(line, '=')+1;
@@ -782,6 +869,8 @@ void search_pattern_save() {
   FILE* file;
   char filename[1024];
   GtkWidget* temp;
+  char* pos;
+  char* pos2;
 
   strcpy(filename, global.lopster_home);
   strcat(filename, "/searches.save");
@@ -790,9 +879,13 @@ void search_pattern_save() {
     return;
   }
 
+
   for (dlist = global.search_pattern; dlist; dlist = dlist->next) {
     search = (search_pattern_t*)(dlist->data);
-    fprintf(file, "PATTERN [%s]\n", search->name);
+    pos2 = strdup(search->name);
+    for (pos = pos2; *pos; pos++) if (*pos == '\n') *pos = ' ';
+    fprintf(file, "PATTERN [%s]\n", pos2);
+    free(pos2);
     fprintf(file, "Contains=%s\n", search->include);
     fprintf(file, "Excludes=%s\n", search->exclude);
     fprintf(file, "MaxResults=%d\n", search->max_results);
@@ -925,7 +1018,7 @@ void search_pattern_get(search_pattern_t* search) {
   if (GTK_WIDGET_IS_SENSITIVE(temp))
     search->size_hi += gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp))*1024;
 
-  temp = lookup_widget(global.win, "spinbutton25");
+  temp = lookup_widget(global.win, "spinbutton26");
   if (GTK_WIDGET_IS_SENSITIVE(temp))
     search->duration_lo = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp))*60;
   else search->duration_lo = 0;
@@ -1127,6 +1220,7 @@ void search_pattern_save_current() {
 
 void search_pattern_show(search_pattern_t* pattern) {
   GtkWidget* temp;
+  char* text;
 
   if (pattern) {
     temp = lookup_widget(global.win, "search_artist");
@@ -1165,7 +1259,7 @@ void search_pattern_show(search_pattern_t* pattern) {
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), pattern->size_hi/1024/1024);
     temp = lookup_widget(global.win, "spinbutton29");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), (pattern->size_lo/1024)%1024);
-    temp = lookup_widget(global.win, "spinbutton25");
+    temp = lookup_widget(global.win, "spinbutton26");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), pattern->duration_lo/60);
     temp = lookup_widget(global.win, "spinbutton27");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), pattern->duration_lo%60);
@@ -1173,6 +1267,49 @@ void search_pattern_show(search_pattern_t* pattern) {
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), pattern->duration_hi/60);
     temp = lookup_widget(global.win, "spinbutton25");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), pattern->duration_hi%60);
+  } else {
+    temp = lookup_widget(global.win, "search_artist");
+    text = gtk_entry_get_text(GTK_ENTRY(temp));
+    if (!text || !(*text)) {
+      temp = lookup_widget(global.win, "search_results");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 100);
+      temp = lookup_widget(global.win, "combo_entry17");
+      gtk_entry_set_text(GTK_ENTRY(temp), _("mp3"));
+      temp = lookup_widget(global.win, "combo_entry19");
+      gtk_entry_set_text(GTK_ENTRY(temp), Destination_long(DEST_NAPSTER));
+      temp = lookup_widget(global.win, "entry76");
+      gtk_entry_set_text(GTK_ENTRY(temp), _("not specified"));
+      temp = lookup_widget(global.win, "combo_bitrate2");
+      gtk_entry_set_text(GTK_ENTRY(temp), _("not specified"));
+      temp = lookup_widget(global.win, "entry77");
+      gtk_entry_set_text(GTK_ENTRY(temp), _("not specified"));
+      temp = lookup_widget(global.win, "combo_freq2");
+      gtk_entry_set_text(GTK_ENTRY(temp), _("not specified"));
+      temp = lookup_widget(global.win, "entry78");
+      gtk_entry_set_text(GTK_ENTRY(temp), _("not specified"));
+      temp = lookup_widget(global.win, "combo_speed2");
+      gtk_entry_set_text(GTK_ENTRY(temp), _("not specified"));
+      temp = lookup_widget(global.win, "spinbutton30");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+      temp = lookup_widget(global.win, "spinbutton31");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+      temp = lookup_widget(global.win, "spinbutton28");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+      temp = lookup_widget(global.win, "spinbutton29");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+      temp = lookup_widget(global.win, "spinbutton26");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+      temp = lookup_widget(global.win, "spinbutton27");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+      temp = lookup_widget(global.win, "spinbutton24");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+      temp = lookup_widget(global.win, "spinbutton25");
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), 0);
+    }
+    temp = lookup_widget(global.win, "search_artist");
+    gtk_entry_set_text(GTK_ENTRY(temp), "");
+    temp = lookup_widget(global.win, "entry69");
+    gtk_entry_set_text(GTK_ENTRY(temp), "");
   }
   if (!pattern || !g_list_find(global.search_pattern, pattern)) {
     temp = lookup_widget(global.win, "button174");
@@ -1226,6 +1363,8 @@ search_t* search_new() {
   search->results = NULL;
   search->id = search_find_free_id();
   search->resume = 0;
+  search->queued = 0;
+  search->local = 0;
   search->pattern = NULL;
   search->link = NULL;
   
@@ -1374,13 +1513,13 @@ search_fields_detach() {
   gtk_widget_set_sensitive(widget, FALSE);
   widget = lookup_widget(global.win, "button218");
   gtk_widget_set_sensitive(widget, FALSE);
-  global.extra_win |= (1<<7);
+  global.extra_win |= (1<<(G_SIZE-1));
 
   if ((global.geometry[7].x >= 0) &&
       (global.geometry[7].y >= 0)) {
     gtk_window_reposition(GTK_WINDOW(window),
-			  global.geometry[7].x,
-			  global.geometry[7].y);
+			  global.geometry[G_SIZE-1].x,
+			  global.geometry[G_SIZE-1].y);
   }
   return window;
 }
@@ -1407,3 +1546,70 @@ void search_search(search_pattern_t* pattern) {
     }
   }
 }
+
+void search_queue(search_t* search) {
+  GtkCList* clist;
+
+  if (search->results) {
+    printf("cant queue search: already got results\n");
+    return;
+  }
+
+  search->queued = 1;
+
+  if (search->resume) {
+    resume_list_update((resume_t*)(search->link), 0);
+  } else {
+    clist = GTK_CLIST(gtk_object_get_data(GTK_OBJECT(search->link), "clist"));
+    strcpy(tstr[0], "Search queued....");
+    tstr[1][0] = tstr[2][0] = tstr[3][0] = tstr[4][0] = tstr[5][0] = 
+      tstr[6][0] = tstr[7][0] = 0;
+    gtk_clist_append(clist, &list[0]);
+  }
+}
+
+int search_unqueue(search_t* search) {
+  GtkCList* clist;
+
+  if (global.status.connection < 2) return 0;
+  if (SERVER->network == N_UNKNOWN) return 0;
+
+  search->queued = 0;
+  if (search->resume) {
+    resume_list_update((resume_t*)(search->link), 0);
+  } else {
+    clist = GTK_CLIST(gtk_object_get_data(GTK_OBJECT(search->link), "clist"));
+    gtk_clist_clear(clist);
+  }
+
+  return 1;
+}
+
+void search_do(search_t* search) {
+  search_pattern_t* pattern;
+  resume_t* resume;
+
+  if (global.status.connection < 2) return;
+
+  pattern = search->pattern;
+  if (!pattern) return;
+  
+  if (!search_unqueue(search)) return;
+
+  if (search->resume) {
+    resume = (resume_t*)(search->link);
+    resume_list_update(resume, 0);
+  }
+
+  if (pattern->destination == DEST_LIBRARY) {
+    lib_search(pattern);
+  } else if (pattern->destination == DEST_HOTLIST) {
+    browse_do_search(pattern);
+  } else if (pattern->destination == DEST_NAPSTER) {
+    napster_search(pattern, search->local);
+  } else if (pattern->destination == DEST_SEARCH) {
+    search_search(pattern);
+  } else {
+    printf("unknown search destination");
+  }
+}
