@@ -4,6 +4,8 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/time.h>
+#include <time.h>
 #include <unistd.h>
 #include <netdb.h>
 #include <errno.h>
@@ -148,6 +150,7 @@ server_t* search_server() {
   text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
   
   if (!strcasecmp(_("Last Server"), text) && global.last_server) {
+    free(text);
     return global.last_server;
   } else {
     des = strrchr(text, ' ');
@@ -175,13 +178,21 @@ void add_server(int meta) {
   char* addr = NULL;
   char* des = NULL;
   unsigned long uaddr;
-  char t[1024];
+  char t[2048];
 
   temp1 = lookup_widget(global.win, "entry73");
   addr = gtk_entry_get_text(GTK_ENTRY(temp1));
   temp2 = lookup_widget(global.win, "entry72");
   des = gtk_entry_get_text(GTK_ENTRY(temp2));
 
+  if (!addr || !(*addr)) {
+    sprintf(t, _("Enter a server address!\n"));
+    temp1 = lookup_widget(global.win, "text14");
+    gtk_text_insert(GTK_TEXT(temp1), NULL, NULL, NULL, 
+		    t, strlen(t));
+    return;
+  }
+
   uaddr = resolv_address(addr);
   if (uaddr) {
     server = server_new();
@@ -203,11 +214,12 @@ void add_server(int meta) {
       temp1 = lookup_widget(global.win, "text14");
       gtk_text_insert(GTK_TEXT(temp1), NULL, NULL, NULL, 
 		      t, strlen(t));
-      server_destroy(server);;
+      server_destroy(server);
     } else {
       gtk_entry_set_text(GTK_ENTRY(temp2), "");
       gtk_entry_set_text(GTK_ENTRY(temp1), "");
     }
+    write_rc();
   } else {
     sprintf(t, _("Address [%s] is not valid (no internet connection?)\n"), addr);
     temp1 = lookup_widget(global.win, "text14");
@@ -348,7 +360,7 @@ void napigator_read(gpointer data, int source,
     res = pos2-pos1+1;
     recv(source, buf2, res, 0);
     buf2[res-1] = 0;
-
+    //    printf(": [%s]\n", buf2);
     ip = arg(buf2, 0);
     if (!ip) goto close;
     port = arg(NULL, 0);
@@ -467,7 +479,7 @@ void napigator_get_list() {
     temp = lookup_widget(global.win, "text14");
     gtk_text_insert(GTK_TEXT(temp), NULL, NULL, NULL, 
 		    t, strlen(t));
-    socket_destroy(socket, 1);
+    socket_destroy(socket, NULL);
     return;
   }
 
@@ -863,14 +875,13 @@ void add_server_entry(GtkWidget* popup, link_t* server,
   if (server->links) {
     if (action) {
       menu_entry = gtk_menu_item_new_with_label(action);
-    } else if ((server->ping > 0) && !func) {
+    } else if (server->ping && !func) {
       name = g_strdup_printf("%s (%s)", server->server, server->ping);
       menu_entry = gtk_menu_item_new_with_label (name);
       g_free(name);
     } else {
       menu_entry = gtk_menu_item_new_with_label (server->server);
     }
-    gtk_widget_ref (menu_entry);
     gtk_widget_show (menu_entry);
     gtk_container_add (GTK_CONTAINER (popup), menu_entry);
 
@@ -879,7 +890,6 @@ void add_server_entry(GtkWidget* popup, link_t* server,
     gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_entry), menu);
     
     menu_entry = gtk_menu_item_new_with_label (server->server);
-    gtk_widget_ref (menu_entry);
     gtk_widget_show (menu_entry);
     gtk_container_add (GTK_CONTAINER (menu), menu_entry);
 
@@ -887,7 +897,6 @@ void add_server_entry(GtkWidget* popup, link_t* server,
       gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
 			  func, NULL);
     menu_entry = gtk_menu_item_new ();
-    gtk_widget_ref (menu_entry);
     gtk_widget_show (menu_entry);
     gtk_container_add (GTK_CONTAINER (menu), menu_entry);
     gtk_widget_set_sensitive (menu_entry, FALSE);
@@ -899,17 +908,14 @@ void add_server_entry(GtkWidget* popup, link_t* server,
     }
   } else if (action) {
     menu_entry = gtk_menu_item_new_with_label(action);
-    gtk_widget_ref (menu_entry);
     gtk_widget_show (menu_entry);
     gtk_container_add (GTK_CONTAINER (popup), menu_entry);
 
     menu = gtk_menu_new ();
-    gtk_widget_ref (menu);
     gtk_widget_show (menu);
     gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_entry), menu);
 
     menu_entry = gtk_menu_item_new_with_label (server->server);
-    gtk_widget_ref (menu_entry);
     gtk_widget_show (menu_entry);
     gtk_container_add (GTK_CONTAINER (menu), menu_entry);
 
@@ -917,13 +923,12 @@ void add_server_entry(GtkWidget* popup, link_t* server,
       gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
 			  func, NULL);
   } else {
-    if ((server->ping > 0) && !func){
+    if (server->ping && !func){
       name = g_strdup_printf("%s (%s)", server->server, server->ping);
       menu_entry = gtk_menu_item_new_with_label (name);
       g_free(name);
     } else
       menu_entry = gtk_menu_item_new_with_label (server->server);
-    gtk_widget_ref (menu_entry);
     gtk_widget_show (menu_entry);
     gtk_container_add (GTK_CONTAINER (popup), menu_entry);
 
@@ -1012,6 +1017,17 @@ void add_server_entry2(GtkWidget* popup, link_t* server) {
   }
 }
 
+void
+on_show_idle_activate  (GtkMenuItem     *menuitem,
+			gpointer         user_data)
+{
+    GtkWidget* temp;
+    
+    temp = lookup_widget(global.win, "frame320");
+    if (GTK_WIDGET_VISIBLE(temp)) gtk_widget_hide(temp);
+    else gtk_widget_show(temp);
+}
+
 GtkWidget* create_server2_popup (void) {
   GtkWidget *popup;
   GtkAccelGroup *popup_accels;
@@ -1023,78 +1039,90 @@ GtkWidget* create_server2_popup (void) {
 
   if (global.status.connection < 2) {
     mode = gtk_menu_item_new_with_label (_("Not connected"));
-    gtk_widget_ref (mode);
     gtk_widget_show (mode);
     gtk_container_add (GTK_CONTAINER (popup), mode);
     gtk_widget_set_sensitive(mode, FALSE);
-    return popup;
   } else if (!opennap_version(0, 0)) {
     mode = gtk_menu_item_new_with_label (_("Opennap only"));
-    gtk_widget_ref (mode);
     gtk_widget_show (mode);
     gtk_container_add (GTK_CONTAINER (popup), mode);
     gtk_widget_set_sensitive(mode, FALSE);
-    return popup;
-  }
-
-  if (global.links) {
-    if (global.user.level > L_MOD) {
-      if (opennap_version(0, 37)) {
-	add_server_entry2(popup, global.allowed_links);
-
+  } else {
+    if (global.links) {
+      if (global.user.level > L_MOD) {
+	if (opennap_version(0, 37)) {
+	  add_server_entry2(popup, global.allowed_links);
+	  
+	  mode = gtk_menu_item_new ();
+	  gtk_widget_show (mode);
+	  gtk_container_add (GTK_CONTAINER (popup), mode);
+	  gtk_widget_set_sensitive (mode, FALSE);
+	}
+	
+	add_server_entry(popup, global.links, on_server_delink, 
+			 _("Delink Server"));
+	
 	mode = gtk_menu_item_new ();
 	gtk_widget_ref (mode);
 	gtk_widget_show (mode);
 	gtk_container_add (GTK_CONTAINER (popup), mode);
 	gtk_widget_set_sensitive (mode, FALSE);
+	
+	if (global.user.level > L_ADMIN) {
+	  add_server_entry(popup, global.links, on_server_shutdown, 
+			   _("Shutdown Server"));
+	  mode = gtk_menu_item_new ();
+	  gtk_widget_show (mode);
+	  gtk_container_add (GTK_CONTAINER (popup), mode);
+	  gtk_widget_set_sensitive (mode, FALSE);
+	}
       }
       
-      add_server_entry(popup, global.links, on_server_delink, 
-		       _("Delink Server"));
+      add_server_entry(popup, global.links, NULL, _("Linked Servers"));
       
       mode = gtk_menu_item_new ();
-      gtk_widget_ref (mode);
-      gtk_widget_show (mode);
-      gtk_container_add (GTK_CONTAINER (popup), mode);
-      gtk_widget_set_sensitive (mode, FALSE);
-
-      add_server_entry(popup, global.links, on_server_shutdown, 
-		       _("Shutdown Server"));
-
-      mode = gtk_menu_item_new ();
-      gtk_widget_ref (mode);
       gtk_widget_show (mode);
       gtk_container_add (GTK_CONTAINER (popup), mode);
       gtk_widget_set_sensitive (mode, FALSE);
     }
     
-    add_server_entry(popup, global.links, NULL, _("Linked Servers"));
-
+    if (global.links)
+      mode = gtk_menu_item_new_with_label (_("Refresh linked servers"));
+    else
+      mode = gtk_menu_item_new_with_label (_("Get linked servers"));
+    gtk_widget_show (mode);
+    gtk_container_add (GTK_CONTAINER (popup), mode);
+    gtk_signal_connect (GTK_OBJECT (mode), "activate",
+			GTK_SIGNAL_FUNC (on_server_links),
+			NULL);
+    
+    mode = gtk_menu_item_new_with_label (_("Show server stats"));
+    gtk_widget_show (mode);
+    gtk_container_add (GTK_CONTAINER (popup), mode);
+    gtk_signal_connect (GTK_OBJECT (mode), "activate",
+			GTK_SIGNAL_FUNC (lopster_sstats),
+			NULL);
+  }
+  {
+    GtkWidget* temp;
+    
     mode = gtk_menu_item_new ();
-    gtk_widget_ref (mode);
     gtk_widget_show (mode);
     gtk_container_add (GTK_CONTAINER (popup), mode);
     gtk_widget_set_sensitive (mode, FALSE);
+    
+    mode = gtk_check_menu_item_new_with_label (_("Show server idle bar"));
+    gtk_widget_show (mode);
+    temp = lookup_widget(global.win, "frame320");
+    if (GTK_WIDGET_VISIBLE(temp)) {
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mode), TRUE);
+    }
+    gtk_container_add (GTK_CONTAINER (popup), mode);
+    gtk_signal_connect (GTK_OBJECT (mode), "activate",
+			GTK_SIGNAL_FUNC (on_show_idle_activate),
+			NULL);
   }
 
-  if (global.links)
-    mode = gtk_menu_item_new_with_label (_("Refresh linked servers"));
-  else
-    mode = gtk_menu_item_new_with_label (_("Get linked servers"));
-  gtk_widget_ref (mode);
-  gtk_widget_show (mode);
-  gtk_container_add (GTK_CONTAINER (popup), mode);
-  gtk_signal_connect (GTK_OBJECT (mode), "activate",
-                      GTK_SIGNAL_FUNC (on_server_links),
-                      NULL);
-
-  mode = gtk_menu_item_new_with_label (_("Show server stats"));
-  gtk_widget_ref (mode);
-  gtk_widget_show (mode);
-  gtk_container_add (GTK_CONTAINER (popup), mode);
-  gtk_signal_connect (GTK_OBJECT (mode), "activate",
-                      GTK_SIGNAL_FUNC (lopster_sstats),
-                      NULL);
   return popup;
 }
 
@@ -1111,11 +1139,12 @@ void add_allowed_link(char* server1, char* server2) {
       g_list_append(global.allowed_links->links, link);
   }
   
+  if (link_search(link, server2)) return;
+
   link2 = (link_t*)malloc(sizeof(link_t));
   link2->server = strdup(server2);
   link2->links = NULL;
 
-  if (link_search(link, server2)) return;
   
   link->links =
     g_list_append(link->links, link2);
@@ -1353,7 +1382,7 @@ void problems_get() {
   if (!connect_socket(socket, "TCP", SOCK_STREAM)) {
     client_message(_("Error"), 
 		   _("Could not connect to lopster.sourceforge.net\n"));
-    socket_destroy(socket, 1);
+    socket_destroy(socket, NULL);
     return;
   }
 
@@ -1362,3 +1391,152 @@ void problems_get() {
 		  GTK_SIGNAL_FUNC(problems_request),
 		  socket);  
 }
+
+/////////////////////////////////
+
+void latest_version_read(gpointer data, int source, 
+			 GdkInputCondition condition) {
+  int res;
+  char buf[1025];
+  char buf2[1025];
+  char* pos1;
+  char* pos2;
+  char* version;
+  socket_t* socket = (socket_t*)data;
+
+  if (condition != GDK_INPUT_READ) {
+    socket_destroy(socket, 0);
+    return;
+  }
+  switch (res = recv(source, buf, 1024, MSG_PEEK)) {
+  case -1:
+    socket_destroy(socket, 0);
+    return;
+  case 0:
+    socket_destroy(socket, 0);
+    return;
+  default:
+    break;
+  }
+
+  buf[res] = 0;
+  socket->cnt = 0;
+
+  pos1 = buf;
+  while ((pos2 = strchr(pos1, '\n')) != NULL) {
+    res = pos2-pos1+1;
+    recv(source, buf2, res, 0);
+    buf2[res-1] = 0;
+    /////
+    version = arg(buf2, 0);
+    if (version && !strcasecmp(version, "Version")) {
+      version = arg(NULL, 0);
+      if (version) {
+	if (socket->data) {
+	  check_version_dialog(version);
+	} else if (!version_is_up_to_date(version)) {
+	  new_version_dialog(version);
+	}
+	socket_destroy(socket, 0);
+      }
+    }
+    /////
+    pos1 = pos2+1;
+  }
+  return;
+}
+
+void latest_version_read_header(gpointer data, int source, 
+				GdkInputCondition condition) {
+  int res;
+  int n;
+  char buf[1025];
+  char buf2[1025];
+  char* pos1;
+  char* pos2;
+  char* pos3;
+  socket_t* socket = (socket_t*)data;
+
+  if (condition != GDK_INPUT_READ) {
+    socket_destroy(socket, 0);
+    return;
+  }
+  switch (res = recv(source, buf, 1024, MSG_PEEK)) {
+  case -1:
+    socket_destroy(socket, 0);
+    return;
+  case 0:
+    socket_destroy(socket, 0);
+    return;
+  default:
+    break;
+  }
+
+  pos1 = buf;
+  while ((pos2 = strchr(pos1, '\n')) != NULL) {
+    n = pos2-pos1+1;
+    recv(source, buf2, n, 0);
+    buf2[n-2] = 0;
+    if (strlen(buf2) == 0) {
+      gdk_input_remove(socket->input);
+      socket->input =
+	gdk_input_add(socket->fd, GDK_INPUT_READ, 
+		      GTK_SIGNAL_FUNC(latest_version_read), socket);
+      
+      return;
+    }
+    if (!strncasecmp(buf2, "HTTP", 4)) {
+      pos3 = arg(buf2, 0);
+      pos3 = arg(NULL, 0);
+      if (!pos3 || strcmp(pos3, "200")) {
+	//	printf("error getting latest version [%s]\n", pos3?pos3:"???");
+	socket_destroy(socket, 0);
+	return;
+      }
+    }
+    pos1 = pos2+1;
+  }
+}
+
+void latest_version_request(gpointer data, int source, 
+		      GdkInputCondition condition) {
+  socket_t* socket = (socket_t*)data;
+  char t[1024];
+
+  if (condition != GDK_INPUT_WRITE) {
+    socket_destroy(socket, 0);
+    return;
+  }
+
+  gdk_input_remove(socket->input);
+
+  strcpy(t, "GET /LATEST_VERSION HTTP/1.0\nHost: lopster.sourceforge.net\n\n");
+  send(source, t, strlen(t), 0);
+   
+  socket->input = 
+    gdk_input_add(source, GDK_INPUT_READ,
+		  latest_version_read_header, socket);
+}
+
+void latest_version_get(int just_check) {
+  long addr;
+
+  socket_t* socket = socket_new(S_HTTP);
+  socket->data = (void*)just_check;
+
+  addr = resolv_address("lopster.sourceforge.net");
+  if (!addr) return;
+
+  socket->ip_long = addr;
+  socket->port = htons(80);
+  
+  if (!connect_socket(socket, "TCP", SOCK_STREAM)) {
+    socket_destroy(socket, NULL);
+    return;
+  }
+
+  socket->input =
+    gdk_input_add(socket->fd, GDK_INPUT_WRITE, 
+		  GTK_SIGNAL_FUNC(latest_version_request),
+		  socket);  
+}
