@@ -20,8 +20,15 @@
 #include "search.h"
 #include "browse.h"
 #include "share.h"
+#include "scheme.h"
+#include "md5.h"
 #include "handler.h"
 
+#define FRAMES_FLAG     0x0001
+#define BYTES_FLAG      0x0002
+#define TOC_FLAG        0x0004
+#define VBR_SCALE_FLAG  0x0008
+
 /*
   load -------------.
                      >-- commit
@@ -48,91 +55,195 @@ unsigned int get_bits(unsigned char* buffer, int start, int no) {
   return res;
 }
 
+static int ExtractI4(unsigned char *buf) {
+  int x;
+  /* big endian extract */ 
+  x = buf[0];
+  x <<= 8;
+  x |= buf[1];
+  x <<= 8;
+  x |= buf[2];
+  x <<= 8;
+  x |= buf[3];
+  
+  return x;
+}
+
+char* read_xing(FILE* fd, int offset) {
+  unsigned char buffer[1024];
+  unsigned char* buf;
+
+  buf = buffer;
+  if (fread(buffer, 1, offset+120, fd) < offset+120) return NULL;
+
+  buf += offset;
+  if (buf[0] != 'X') return 0;	/* fail */
+  if (buf[1] != 'i') return 0;	/* header not found */
+  if (buf[2] != 'n') return 0;
+  if (buf[3] != 'g') return 0;
+  
+  buf += 4;
+  return buf;
+}
+
 void file_parse_mp3_header(file_t *file) {
   FILE* fd;
   unsigned char buffer[4];  // header
+  unsigned char* buf;
   int mpeg_version;
   int mpeg_layer;
   int res;
-  long framesize;
+  int index;
   long totalframes;
+  int padding;
+  int stereo;
+  int protection;
+  int lsf;
+  double bpf;
+  static int bs[4] = {0, 384, 1152, 1152};
+  static int s1[4] = {0, 48000, 144000, 144000};
+  double tpf;
+  int xing_offset;
+  int pos;
   short br[2][4][16] = {
     {  // mpeg 2
       { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
-      { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 }, // 
       { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },      // 
-      { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 }       // 
+      { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },      // 
+      { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 }  // 
     },
     {  // mpeg1
       { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
-      { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 },     // layer III
+      { 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0 },  // layer I
       { 0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0 },    // layer II
-      { 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0 }  // layer I
+      { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 }     // layer III
     }
   };
 
-  int freq[2][4] = {
-    {22050, 24000, 16000, 0},
-    {44100, 48000, 32000, 0}
+  int freq[2][2][4] = {
+    {
+      {11025, 12000, 8000, 0},   // mpeg 2.5
+      {0, 0, 0},                 // reserved
+    },
+    {
+      {22050, 24000, 16000, 0},  // mpeg 2
+      {44100, 48000, 32000, 0}   // mpeg 1
+    },
   };
 
   if ((fd = fopen(file->longname, "r")) != NULL) {
     if (fread(buffer, sizeof(char), 4, fd) != 4) {
-      //      printf("could not read header [%s]\n", file->longname);
+      printf("could not read header [%s]\n", file->longname);
       return;
     }
-    // i think this is not correct sometimes
     while (get_bits(buffer, 0, 11) != 0x7ff) {
       buffer[0] = buffer[1];
       buffer[1] = buffer[2];
       buffer[2] = buffer[3];
       if (fread(&buffer[3], sizeof(char), 1, fd) != 1) {
-	//	printf("could not read header [%s]\n", file->longname);
+	printf("could not read header [%s]\n", file->longname);
 	return;
       }
     }
   } else {
-//    printf("could not open file for reading header [%s]\n", file->longname);
+    //    printf("could not open file for reading header [%s]\n", file->longname);
     return;
   }
 
   // sync word
-  res = get_bits(buffer, 0, 12);
+  res = get_bits(buffer, 0, 11);
   
+  // sync word
+  index = get_bits(buffer, 11, 1);
+
   // MPEG Version
   mpeg_version = get_bits(buffer, 12, 1);
+  if (mpeg_version) lsf = 0x0;
+  else lsf = 0x1;
 
   // MPEG Layer
   mpeg_layer = get_bits(buffer, 13, 2);
+  mpeg_layer = 4 - mpeg_layer;
+  if (mpeg_layer > 3) return;
 
   // Protection Bit
-  res = get_bits(buffer, 15, 1);
-
+  protection = get_bits(buffer, 15, 1);
+  
   // Bitrate
   res = get_bits(buffer, 16, 4);
+  if (res > 14) return;
   file->bitrate = br[mpeg_version][mpeg_layer][res];
 
   // Fequency
   res = get_bits(buffer, 20, 2);
-  file->frequency = freq[mpeg_version][res];
+  file->frequency = freq[index][mpeg_version][res];
   if ((file->bitrate == 0) || (file->frequency == 0)) {
     //    printf("corrupted header! [%s]\n", file->longname);
     return;
   }
 
-  framesize = 144000 * file->bitrate / file->frequency;
-  totalframes = (file->size / (framesize + 1)) - 1;
-  file->duration = (time_t) (totalframes * 1152 / file->frequency);
-
   // Padding Bit
-  res = get_bits(buffer, 22, 1);
+  padding = get_bits(buffer, 22, 1);
   
   // Private Bit
   res = get_bits(buffer, 23, 1);
   
   // Mode
   res = get_bits(buffer, 24, 2);
+  if (res == 3) stereo = 0;
+  else stereo = 1;
+
+  tpf = (double) bs[mpeg_layer] / (file->frequency << lsf);
+
+  if (mpeg_version) {           /* mpeg1 */
+    if (stereo) xing_offset = 32;
+    else xing_offset = 17;
+  } else {                      /* mpeg2 */
+    if (stereo) xing_offset = 17;
+    else xing_offset = 9;
+  }
+
+  pos = ftell(fd);
+  if ((buf = read_xing(fd, xing_offset))) {
+    int head_flags;
+    int vbr_scale = -1;
+    int bytes;
+
+    head_flags = ExtractI4(buf);
+    buf += 4;
+    if (head_flags & FRAMES_FLAG) {
+      totalframes = ExtractI4(buf);
+      buf += 4;
+    } else totalframes = 1;
+    if (head_flags & BYTES_FLAG) {
+      bytes = ExtractI4(buf);
+      buf += 4;
+    } else bytes = 1;
+    if (head_flags & TOC_FLAG) {
+      buf += 100;
+    }
+    if (head_flags & VBR_SCALE_FLAG) {
+      vbr_scale = ExtractI4(buf);
+      buf += 4;
+    }
+    file->bitrate = 
+      (gint) ((bytes * 8) / (tpf * totalframes * 1000));
+    bpf = (double)(file->size-pos) / totalframes;
+    file->vbr = 1;
+  } else {
+    bpf = file->bitrate * s1[mpeg_layer] / (file->frequency << lsf);
+    totalframes = (int)((file->size-pos) / bpf);
+    file->vbr = 0;
+  }
   
+  file->duration = (time_t) (totalframes * tpf);
+
+#ifdef SHARE_DEBUG
+  printf("mpeg %d layer %d, br %d, freq %d bpf %.1f, frames %ld, duration %d, vbr:%d\n",
+	 lsf+1, mpeg_layer, file->bitrate, file->frequency, bpf, 
+	 totalframes, file->duration, file->vbr);
+#endif
+
   // Mode Extention
   res = get_bits(buffer, 26, 2);
   
@@ -149,42 +260,39 @@ void file_parse_mp3_header(file_t *file) {
 }
 
 void file_parse_header(file_t *file) {
-  if (file->mime_type == MIME_MP3) {
+  char* suf;
+
+  suf = strrchr(file->longname, '.');
+  if (suf) suf++;
+  
+  if (suf && !strcasecmp(suf, "MP3")) {
     file_parse_mp3_header(file);
+  } else if (suf && !strcasecmp(suf, "OGG")) {
+    // for now set dummy values
+    file->bitrate = 32;
+    file->frequency = 32000;
+    file->duration = 60;
   } else {
-    file->bitrate = 8;
-    file->frequency = 0;
-    file->duration = 0;
+    file->bitrate = 32;
+    file->frequency = 32000;
+    file->duration = 60;
   }
 }
 
 void file_calc_md5(file_t *file) {
-  //  file->md5 = MD5File(file->longname, file->md5);
-  file->md5 = strdup("NOTCOMPUTED");
-}
+  int i1;
+  char str[33];
+  char hex[16] = "0123456789abcdef";
 
-gint
-lib_compare (GtkCList      *clist,
-	     gconstpointer  ptr1,
-	     gconstpointer  ptr2) {
-  char *text1 = NULL;
-  char *text2 = NULL;
-
-  GtkCListRow *row1 = (GtkCListRow *) ptr1;
-  GtkCListRow *row2 = (GtkCListRow *) ptr2;
-
-  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
-  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
-
-  if (!text2)
-    return (text1 != NULL);
-
-  if (!text1)
-    return -1;
-
-  if (gtk_events_pending()) gtk_main_iteration();
-  
-  return strcasecmp (text1, text2);
+  if (file->md5) free(file->md5);
+  file->md5 = NULL;
+  if (global.options.dummy_md5) {
+    for (i1 = 0; i1 < 32; i1++) str[i1] = hex[random()%16];
+    str[32] = 0;
+    file->md5 = g_strdup_printf("%s-%ld", str, file->size);
+  } else {
+    file->md5 = MD5File(file->longname, file->md5);
+  }
 }
 
 suffix_t* check_suffix(char* name, int mime) {
@@ -206,38 +314,41 @@ suffix_t* check_suffix(char* name, int mime) {
 void lib_commit_file (file_t* file) {
   char t2[2048];
   char t[1024];
+  suffix_t* suffix;
 
   if (global.status.connection < 2) return;
 
   if ((file->flags & FLAG_TO_SHARE) &&
       ((file->flags & FLAG_SHARED) == 0)) {
-    if (file->mime_type == MIME_MP3) {
+    suffix = get_suffix(file->winname);
+    if ((suffix && suffix->as_mp3) || (!opennap_version(0, 37))) {
       strcpy(t2, "\"");
       strcat(t2, file->winname);
       strcat(t2, "\" ");
       strcat(t2, file->md5);
-      sprintf(t, " %ld %d %d %d",
-	      file->size, file->bitrate, 
-	      file->frequency, file->duration);
+      if (opennap_version(0, 0))
+	sprintf(t, " %ld %d %d %d",
+		file->size, find_valid_bitrate(file->bitrate),
+		file->frequency, file->duration);
+      else
+	sprintf(t, " %ld %d %d %d", file->size, file->bitrate,
+		file->frequency, file->duration);
       strcat(t2, t);
       send_command(CMD_CLIENT_ADD_FILE, t2);
-      file->flags &= 0xff^FLAG_TO_SHARE;
-      file->flags |= FLAG_SHARED;
-      global.user.unshared--;
-      lib_tree_update_file(file);
-    } else if (opennap_version(0, 37)) {
+    } else  {
       strcpy(t2, "\"");
       strcat(t2, file->winname);
       strcat(t2, "\"");
       sprintf(t, " %ld %s %s",
-	      file->size, file->md5, MimeNames_(file->mime_type));
+	      file->size, file->md5, 
+	      MimeNames_((file->mime_type==MIME_NONE)?(MIME_TEXT):(file->mime_type)));
       strcat(t2, t);
       send_command(CMD_CLIENT_SHARE_FILE, t2);
-      file->flags &= 0xff^FLAG_TO_SHARE;
-      file->flags |= FLAG_SHARED;
-      global.user.unshared--;
-      lib_tree_update_file(file);
     }
+    file->flags &= (0xff^FLAG_TO_SHARE);
+    file->flags |= FLAG_SHARED;
+    global.user.unshared--;
+    lib_tree_update_file(file);
   }
 }
 
@@ -252,13 +363,7 @@ gint lib_commit_idle(gpointer data) {
   if (global.status.building > 1) return 0;
   if (global.status.connection < 2) return 0;
 
-  if (!cur_file) {
-    if (global.options.login_mode & L_NOT_SHARE) {
-      global.options.login_mode &= 0xff^L_NOT_SHARE;
-      return 0;
-    }
-    cur_file = global.user.files;
-  }
+  if (!cur_file) cur_file = global.user.files;
 
   for (i1 = 0; i1 < 100; i1++) {
     if (!cur_file) break;
@@ -276,7 +381,7 @@ gint lib_commit_idle(gpointer data) {
 
 void lib_update_file (file_t* file) {
   struct stat stats;
-  int size;
+  unsigned long size;
 
   if (stat(file->longname, &stats) < 0) {
     size = file->size;
@@ -284,6 +389,7 @@ void lib_update_file (file_t* file) {
     size = stats.st_size;
   }
   if (file->size != size) {
+    //    printf("%lu - %lu\n", file->size, size);
     global.user.bytes -= file->size;
     file->size = size;
     global.user.bytes += file->size;
@@ -426,7 +532,7 @@ int add_shared_directory(int mime, char* dirname) {
       //      if ((buf.st_mode&S_IFLNK) == 0)
       add_shared_directory(mime, name);
     } else {
-      if (check_suffix(entry->d_name, mime)) {
+      if ((mime == MIME_NONE) || check_suffix(entry->d_name, mime)) {
 	file = lib_search_file(name);
 	if (!file) {
 	  file = file_create_from_local(name);
@@ -438,7 +544,7 @@ int add_shared_directory(int mime, char* dirname) {
 	  file->flags &= 0xff^FLAG_TO_REMOVE;
 	}
       }
-      if (gtk_events_pending()) gtk_main_iteration();
+      while (gtk_events_pending()) gtk_main_iteration();
     }
   }
   closedir(dir);
@@ -450,7 +556,6 @@ void lib_refresh() {
   int i1 = 0;
   GList* dlist = NULL;
   GtkCList* clist;
-  GtkWidget* temp;
   char* text;
 
 #ifdef SHARE_DEBUG
@@ -463,9 +568,6 @@ void lib_refresh() {
 
   lib_set_flag(FLAG_TO_REMOVE, NULL);
 
-  temp = lookup_widget(global.win, "entry84");
-  gtk_entry_set_text(GTK_ENTRY(temp), "");
-
   clist = GTK_CLIST(lookup_widget(global.win, "lib_tree"));
   gtk_clist_freeze(clist);
   for (i1 = 0; i1 < MIME_SIZE; i1++) {
@@ -479,7 +581,12 @@ void lib_refresh() {
   }
   gtk_clist_thaw(clist);
   
-  lib_set_modified();
+  if (global.auto_save) {
+    text = strdup(global.auto_save);
+    lib_save(text);
+    free(text);
+  }
+  //  lib_set_modified();
 
   gtk_idle_add(lib_remove_idle, NULL);
   
@@ -492,11 +599,6 @@ file_t* file_create_from_local(char* longname) {
   int mime;
 
   mime = get_mimetype(longname);
-  if (mime == 0) {
-    g_warning(_("file has unknown mime type"));
-    return NULL;
-  }
-
   new_file = (file_t*)malloc(sizeof(file_t));
   
   new_file->mime_type = mime;
@@ -508,9 +610,9 @@ file_t* file_create_from_local(char* longname) {
 
   new_file->size = 0;
   new_file->user = strdup("");
-  
   new_file->ip = 0;
-  new_file->md5 = strdup("NOTCOMPUTED");;
+  
+  new_file->md5 = strdup("00000000000000000000000000000000");
 
   new_file->bitrate = 32;
   new_file->frequency = 32000;
@@ -519,6 +621,7 @@ file_t* file_create_from_local(char* longname) {
   new_file->flags = 0;
   new_file->status = FILE_NONE;
   new_file->local = 1;
+  new_file->vbr = 0;
 
   return new_file;
 }
@@ -560,7 +663,7 @@ void lib_insert_file_real(file_t *file) {
     strcpy(t, file->longname);
     dir[0] = 0;
   }
-  
+
   /////////////////
   pos = strtok(t, "/");
   node = NULL;
@@ -611,10 +714,10 @@ void lib_insert_file_real(file_t *file) {
 				       (GtkDestroyNotify)destroy_dir_row);
     } else {
       sprintf(tstr[1], "%.2f MB", (double)(file->size)/1024/1024);
-      sprintf(tstr[2], "%d", file->bitrate);
+      if (file->vbr) sprintf(tstr[2], "av. %d", file->bitrate);
+      else sprintf(tstr[2], "%d", file->bitrate);
       sprintf(tstr[3], "%d", file->frequency);
-      sprintf(tstr[4], "%d:%s%d", file->duration/60, 
-	      (file->duration%60)<10?"0":"", file->duration%60);
+      print_time_short(tstr[4], file->duration);
       
       node = gtk_ctree_insert_node(ctree,
 				   node, NULL, list, 5,
@@ -635,12 +738,14 @@ void lib_insert_file(file_t *file) {
   lib_insert_file_real(file);
 
 #ifdef SHARE_DEBUG
-  printf("adding [%s]\n", file->longname);
+  printf("adding [%s] %lu\n", file->longname, file->size);
 #endif
   global.user.shared++;
   global.user.unshared++;
   global.user.bytes += file->size;
-  file->flags |= FLAG_TO_SHARE;
+  if ((global.options.login_mode & L_NOT_SHARE) == 0) {
+    file->flags |= FLAG_TO_SHARE;
+  }
 
   global.user.files = g_list_append(global.user.files, file); 
 }
@@ -648,7 +753,9 @@ void lib_insert_file(file_t *file) {
 void lib_tree_update_file(file_t *file) {
   GtkCTree* ctree;
   GtkCTreeNode* node;
-  static GdkColor color = { 0, 0xeeee, 0xeeee, 0xeeee };
+  style_t* style;
+
+  style = style_get(global.scheme, "library_unshared");
 
   ctree = GTK_CTREE(lookup_widget(global.win, "lib_tree"));
   node = gtk_ctree_find_by_row_data(ctree, NULL, file);  
@@ -656,7 +763,8 @@ void lib_tree_update_file(file_t *file) {
 
   sprintf(tstr[0], "%.2f MB", (double)(file->size)/1024/1024);
   gtk_ctree_node_set_text(ctree, node, 1, tstr[0]);
-  sprintf(tstr[0], "%d", file->bitrate);
+  if (file->vbr) sprintf(tstr[0], "av. %d", file->bitrate);
+  else sprintf(tstr[0], "%d", file->bitrate);
   gtk_ctree_node_set_text(ctree, node, 2, tstr[0]);
   sprintf(tstr[0], "%d", file->frequency);
   gtk_ctree_node_set_text(ctree, node, 3, tstr[0]);
@@ -664,10 +772,15 @@ void lib_tree_update_file(file_t *file) {
 	  (file->duration%60)<10?"0":"", file->duration%60);
   gtk_ctree_node_set_text(ctree, node, 4, tstr[0]);
 
-  if (file->flags & FLAG_SHARED)
+  if (!style) return;
+  
+  if (file->flags & FLAG_SHARED) {
+    gtk_ctree_node_set_foreground(ctree, node, NULL);
     gtk_ctree_node_set_background(ctree, node, NULL);
-  else
-    gtk_ctree_node_set_background(ctree, node, &color);
+  } else {
+    gtk_ctree_node_set_foreground(ctree, node, style->fore);
+    gtk_ctree_node_set_background(ctree, node, style->back);
+  }
 }
 
 
@@ -707,7 +820,7 @@ void lib_del_flag(int val, GtkCTreeNode* node) {
 			   (GtkCTreeFunc)rec_del_flag, &val);
 }
 
-void rec_unshare_flag (GtkCTree *ctree, GtkCTreeNode *node,
+void rec_replace_flag (GtkCTree *ctree, GtkCTreeNode *node,
 		       int* data) {
   file_t* file;
   char* vals;
@@ -720,11 +833,11 @@ void rec_unshare_flag (GtkCTree *ctree, GtkCTreeNode *node,
   if (file->flags & vals[0]) {
     file->flags &= (0xff^(vals[0]));
     file->flags |= vals[1];
-    global.user.unshared++;
+    //    global.user.unshared++;
   }
 }
 
-void lib_unshare_flag(int val1, int val2, GtkCTreeNode* node) {
+void lib_replace_flag(int val1, int val2, GtkCTreeNode* node) {
   GtkCTree* ctree;
   char data[2];
 
@@ -733,7 +846,36 @@ void lib_unshare_flag(int val1, int val2, GtkCTreeNode* node) {
 
   ctree = GTK_CTREE(lookup_widget(global.win, "lib_tree"));
   gtk_ctree_post_recursive(ctree, node, 
-			   (GtkCTreeFunc)rec_unshare_flag, data);
+			   (GtkCTreeFunc)rec_replace_flag, data);
+}
+
+void rec_xor_flag (GtkCTree *ctree, GtkCTreeNode *node,
+		   int* data) {
+  file_t* file;
+  char* vals;
+
+  vals = (char*)data;
+
+  if (!GTK_CTREE_ROW (node)->is_leaf) return;
+
+  file = gtk_ctree_node_get_row_data(ctree, node);
+  if (file->flags & vals[0]) {
+    file->flags &= (0xff^vals[1]);
+  } else {
+    file->flags |= vals[1];
+  }
+}
+
+void lib_xor_flag(int val1, int val2, GtkCTreeNode* node) {
+  GtkCTree* ctree;
+  char data[2];
+
+  data[0] = val1;
+  data[1] = val2;
+
+  ctree = GTK_CTREE(lookup_widget(global.win, "lib_tree"));
+  gtk_ctree_post_recursive(ctree, node, 
+			   (GtkCTreeFunc)rec_xor_flag, data);
 }
 
 file_t* lib_search_file(char* longname) {
@@ -781,23 +923,45 @@ void lib_sort() {
 }
 
 file_t* load_file_entry(FILE* fd) {
+  char* filename;
+  char* md5;
+  char* size;
+  char* bitrate;
+  char* freq;
+  char* duration;
+  char* mime;
+
   char line[2048];
   file_t* file;
-  char* mime;
   
   if (!fgets(line, 2048-1, fd)) return NULL;
 
   file = (file_t*)malloc(sizeof(file_t));
-  file->longname = strdup(arg(line, 0));
+  filename = arg(line, 0);
+  md5 = arg(NULL, 0);
+  size = arg(NULL, 0);
+  bitrate = arg(NULL, 0);
+  freq = arg(NULL, 0);
+  duration = arg(NULL, 0);
+  mime = arg(NULL, 0);
+
+  if (!filename || ! md5 || !size || !bitrate || !freq || !duration) return NULL;
+
+  file->longname = strdup(filename);
   file->shortname = strdup(extract_short_name(file->longname));
   file->winname = strdup(file->longname);
   convert_to_win(file->winname);
-  file->md5 = strdup(arg(NULL, 0));
-  file->size = strtoul(arg(NULL, 0), NULL, 10);
-  file->bitrate = atoi(arg(NULL, 0));
-  file->frequency = atoi(arg(NULL, 0));
-  file->duration = atoi(arg(NULL, 0));
-  mime = arg(NULL, 0);
+  file->md5 = strdup(md5);
+  file->size = strtoul(size, NULL, 10);
+  if (!strncmp(bitrate, "vbr", 3)) {
+    file->bitrate = atoi(bitrate+3);
+    file->vbr = 1;
+  } else {
+    file->bitrate = atoi(bitrate);
+    file->vbr = 0;
+  }
+  file->frequency = atoi(freq);
+  file->duration = atoi(duration);
   if (!mime) file->mime_type = MIME_MP3;
   else file->mime_type = atoi(mime);
   file->user = strdup("");
@@ -877,6 +1041,10 @@ int lib_load(char* fname) {
   global.auto_save = strdup(t);
 
   gtk_idle_add(lib_load_idle, (gpointer)fd);
+  /*
+    gdk_input_add(fileno(fd), GDK_INPUT_READ,
+		GTK_SIGNAL_FUNC(lib_load_func), fd);
+  */
   return 1;
 }
 
@@ -896,8 +1064,8 @@ void lib_save(char* name) {
   }
   for (dlist = global.user.files; dlist; dlist = dlist->next) {
     file = (file_t*)(dlist->data);
-    fprintf(fd, "\"%s\" %s %ld %d %d %d %d\n", 
-	    file->longname, file->md5, file->size,
+    fprintf(fd, "\"%s\" %s %ld %s%d %d %d %d\n", 
+	    file->longname, file->md5, file->size, file->vbr?"vbr":"",
 	    file->bitrate, file->frequency, file->duration, file->mime_type);
   }
   fclose(fd);
@@ -1097,7 +1265,33 @@ create_lib_popup2 (void)
     gtk_container_add (GTK_CONTAINER (lib_popup), separator);
     gtk_widget_set_sensitive (separator, FALSE);
     
-    item = gtk_menu_item_new_with_label (_("Delete (from Disk)"));
+    item = gtk_menu_item_new_with_label (_("Remove All"));
+    gtk_widget_show (item);
+    gtk_container_add (GTK_CONTAINER (lib_popup), item);
+    gtk_signal_connect (GTK_OBJECT (item), "activate",
+			GTK_SIGNAL_FUNC (on_remove_all_activate),
+			NULL);
+    
+    item = gtk_menu_item_new_with_label (_("Unshare All"));
+    gtk_widget_show (item);
+    gtk_container_add (GTK_CONTAINER (lib_popup), item);
+    gtk_signal_connect (GTK_OBJECT (item), "activate",
+			GTK_SIGNAL_FUNC (on_unshare_all_activate),
+			NULL);
+    
+    item = gtk_menu_item_new_with_label (_("Share All"));
+    gtk_widget_show (item);
+    gtk_container_add (GTK_CONTAINER (lib_popup), item);
+    gtk_signal_connect (GTK_OBJECT (item), "activate",
+			GTK_SIGNAL_FUNC (on_share_all_activate),
+			NULL);
+
+    separator = gtk_menu_item_new ();
+    gtk_widget_show (separator);
+    gtk_container_add (GTK_CONTAINER (lib_popup), separator);
+    gtk_widget_set_sensitive (separator, FALSE);
+    
+    item = gtk_menu_item_new_with_label (_("Delete File (from Disk)"));
     gtk_widget_show (item);
     gtk_container_add (GTK_CONTAINER (lib_popup), item);
     gtk_signal_connect (GTK_OBJECT (item), "activate",
@@ -1124,4 +1318,22 @@ void lib_set_modified() {
 
   temp = lookup_widget(global.win, "entry84");
   gtk_entry_set_text(GTK_ENTRY(temp), "");
+  if (global.auto_save) free(global.auto_save);
+  global.auto_save = NULL;
+}
+
+
+void lib_compare_with_browse(user_t* userinfo) {
+  GList* dlist;
+  file_t* file;
+
+  for (dlist = global.user.files; dlist; dlist = dlist->next) {
+    file = (file_t*)(dlist->data);
+    if (!browse_search_file(userinfo, file->winname) &&
+	(file->flags & FLAG_SHARED)) {
+      file->flags &= 0xff^FLAG_SHARED;
+      global.user.unshared++;
+    }
+  }
+  gtk_idle_add(lib_update_idle, NULL);
 }
