@@ -10,6 +10,7 @@ void lib_remove_file(file_t* file);
 gint lib_remove_idle(gpointer data);
 void lib_commit_file (file_t* file);
 gint lib_commit_idle(gpointer data);
+int lib_update_idle(gpointer data);
 void lib_search(search_pattern_t* search);
 
 gint send_shared_files(gpointer data, gint source, 
@@ -26,9 +27,11 @@ suffix_t* check_suffix(char* name, int mime);
 
 void lib_set_flag(int val, GtkCTreeNode* node);
 void lib_del_flag(int val, GtkCTreeNode* node);
-void lib_unshare_flag(int val1, int val2, GtkCTreeNode* node);
+void lib_replace_flag(int val1, int val2, GtkCTreeNode* node);
+void lib_xor_flag(int val1, int val2, GtkCTreeNode* node);
 
 GtkWidget* create_files_popup();
 void lib_update_stats();
 GtkWidget* create_lib_popup2 (void);
 void lib_set_modified();
+void lib_compare_with_browse(user_t* userinfo);
