@@ -1,3 +1,5 @@
+extern const int StatusInfo[S_NUMBER];
+
 char* extract_filename(char* filename);
 char* status_names(int status);
 void convert_local_name(char* fname);
@@ -5,7 +7,6 @@ void convert_local_name(char* fname);
 GtkWidget* create_upload_popup(transfer_t* transfer);
 GtkWidget* create_download_popup(transfer_t* transfer);
 
-int check_dcc_message(char *from, char *message);
 transfer_t* transfer_new();
 void transfer_destroy(transfer_t* transfer);
 void transfer_set_md5(transfer_t* transfer, char* md5);
@@ -18,7 +19,7 @@ void transfer_connect_and_start(socket_t* socket);
 file_t* lib_search_transfer(transfer_t* transfer);
 
 void download_start(socket_t* socket, int force);
-void upload_start(socket_t* socket);
+void upload_start(socket_t* socket, int force);
 void download_file(file_t* file, int withdir);
 void cmd_download_ack(char* data);
 void cmd_upload_request(char* data);
@@ -39,7 +40,7 @@ void cmd_linkspeed_response(char* data);
 FILE* open_download_file(transfer_t* transfer);
 FILE* open_upload_file(transfer_t* transfer);
 
-void transfer_end(transfer_t* transfer, int mode);
+void transfer_end(socket_t* socket, int mode);
 
 // outgoing
 gint await_conn_ack(gpointer data, gint source, 
@@ -65,7 +66,8 @@ int transfer_timeout_cb(gpointer data);
 int transfer_remote_timeout(gpointer data);
 
 user_info_t* detect_user_info(char* user, int in_download);
-void change_user_info(char* user, int max);
+user_info_t* change_user_info(char* user, int max, int down);
+user_info_t* search_user_info(char* user, int down);
 
 void downloads_disable();
 void downloads_enable();
@@ -91,3 +93,8 @@ void on_toggle_autodelete(GtkMenuItem     *menuitem,
 int transfer_grab_resume(transfer_t* transfer, resume_t* resume);
 void transfer_ungrab_resume(transfer_t* transfer, int mode);
 resume_t* transfer_create_resume(transfer_t* transfer);
+void transfer_cancel_running();
+
+void dcc_send_file(char* nick, char* file);
+void dcc_select_file(char* nick);
+int check_dcc_message(char *from, char *message);
